@@ -5,58 +5,7 @@
  * WriteBMP(fp, pic, ptype, w, h, r, g, b, numcols, style);
  */
 
-/* Copyright Notice
- * ================
- * Copyright 1989, 1990, 1991, 1992, 1993 by John Bradley
- * 
- * Permission to use, copy, and distribute XV in its entirety, for 
- * non-commercial purposes, is hereby granted without fee, provided that
- * this license information and copyright notice appear in all copies.
- * 
- * Note that distributing XV 'bundled' in with ANY product is considered
- * to be a 'commercial purpose'.
- *
- * Also note that any copies of XV that are distributed MUST be built
- * and/or configured to be in their 'unregistered copy' mode, so that it
- * is made obvious to the user that XV is shareware, and that they should
- * consider donating, or at least reading this License Info.
- * 
- * The software may be modified for your own purposes, but modified
- * versions may NOT be distributed without prior consent of the author.
- * 
- * This software is provided 'as-is', without any express or implied
- * warranty.  In no event will the author be held liable for any damages
- * arising from the use of this software.
- * 
- * If you would like to do something with XV that this copyright
- * prohibits (such as distributing it with a commercial product, 
- * using portions of the source in some other program, etc.), please
- * contact the author (preferably via email).  Arrangements can
- * probably be worked out.
- *
- * XV is shareware for PERSONAL USE only.  You may use XV for your own
- * amusement, and if you find it nifty, useful, generally cool, or of
- * some value to you, your non-deductable donation would be greatly
- * appreciated.  $25 is the suggested donation, though, of course,
- * larger donations are quite welcome.  Folks who donate $25 or more
- * can receive a Real Nice bound copy of the XV manual for no extra
- * charge.
- * 
- * Commercial, government, and institutional users MUST register their
- * copies of XV, for the exceedingly REASONABLE price of just $25 per
- * workstation/X terminal.  Site licenses are available for those who
- * wish to run XV on a large number of machines.  Contact the author
- * for more details.
- *
- * The author may be contacted via:
- *    US Mail:  John Bradley
- *              1053 Floyd Terrace
- *              Bryn Mawr, PA  19010
- *
- *    Phone:    (215) 898-8813
- *    EMail:    bradley@cis.upenn.edu
- */
-
+#include "copyright.h"
 
 #include "xv.h"
 
@@ -74,32 +23,28 @@
 #define BI_RLE8 1
 #define BI_RLE4 2
 
+#define WIN_OS2_OLD 12
+#define WIN_NEW     40
+#define OS2_NEW     64
 
 static long filesize;
 
-#ifdef __STDC__
-  static int  loadBMP1(FILE *, byte *, int, int);
-  static int  loadBMP4(FILE *, byte *, int, int, int);
-  static int  loadBMP8(FILE *, byte *, int, int, int);
-  static int  loadBMP24(FILE *, byte *, int, int);
-  static unsigned int getshort(FILE *);
-  static unsigned int getint(FILE *);
-  static void putshort(FILE *, int);
-  static void putint(FILE *, int);
-  static void writeBMP1(FILE *, byte *, int, int);
-  static void writeBMP4(FILE *, byte *, int, int);
-  static void writeBMP8(FILE *, byte *, int, int);
-  static void writeBMP24(FILE *, byte *, int, int);
-  static int  bmpError(char *, char *);
-#else
-  static int loadBMP1(), loadBMP4(), loadBMP8(), loadBMP24();
-  static unsigned int getshort(), getint();
-  static void putshort(), putint();
-  static void writeBMP1(), writeBMP4(), writeBMP8(), writeBMP24();
-  static int  bmpError();
-#endif
+static int   loadBMP1   PARM((FILE *, byte *, u_int, u_int));
+static int   loadBMP4   PARM((FILE *, byte *, u_int, u_int, u_int));
+static int   loadBMP8   PARM((FILE *, byte *, u_int, u_int, u_int));
+static int   loadBMP24  PARM((FILE *, byte *, u_int, u_int));
+static u_int getshort   PARM((FILE *));
+static u_int getint     PARM((FILE *));
+static void  putshort   PARM((FILE *, int));
+static void  putint     PARM((FILE *, int));
+static void  writeBMP1  PARM((FILE *, byte *, int, int));
+static void  writeBMP4  PARM((FILE *, byte *, int, int));
+static void  writeBMP8  PARM((FILE *, byte *, int, int));
+static void  writeBMP24 PARM((FILE *, byte *, int, int));
+static int   bmpError   PARM((char *, char *));
 
 
+#define FERROR(fp) (ferror(fp) || feof(fp))
 
 /*******************************************/
 int LoadBMP(fname, pinfo)
@@ -112,6 +57,7 @@ int LoadBMP(fname, pinfo)
   unsigned int bfSize, bfOffBits, biSize, biWidth, biHeight, biPlanes;
   unsigned int biBitCount, biCompression, biSizeImage, biXPelsPerMeter;
   unsigned int biYPelsPerMeter, biClrUsed, biClrImportant;
+  int bPad;
   char         *cmpstr;
   byte         *pic24, *pic8;
   char          buf[512], *bname;
@@ -121,7 +67,7 @@ int LoadBMP(fname, pinfo)
   pic8 = pic24 = (byte *) NULL;
   bname = BaseName(fname);
 
-  fp=fopen(fname,"r");
+  fp = xv_fopen(fname,"r");
   if (!fp) return (bmpError(bname, "couldn't open file"));
   
   fseek(fp, 0L, 2);      /* figure out the file size */
@@ -139,17 +85,33 @@ int LoadBMP(fname, pinfo)
   bfOffBits = getint(fp);
 
   biSize          = getint(fp);
-  biWidth         = getint(fp);
-  biHeight        = getint(fp);
-  biPlanes        = getshort(fp);
-  biBitCount      = getshort(fp);
-  biCompression   = getint(fp);
-  biSizeImage     = getint(fp);
-  biXPelsPerMeter = getint(fp);
-  biYPelsPerMeter = getint(fp);
-  biClrUsed       = getint(fp);
-  biClrImportant  = getint(fp);
-  
+
+  if (biSize == WIN_NEW || biSize == OS2_NEW) {
+    biWidth         = getint(fp);
+    biHeight        = getint(fp);
+    biPlanes        = getshort(fp);
+    biBitCount      = getshort(fp);
+    biCompression   = getint(fp);
+    biSizeImage     = getint(fp);
+    biXPelsPerMeter = getint(fp);
+    biYPelsPerMeter = getint(fp);
+    biClrUsed       = getint(fp);
+    biClrImportant  = getint(fp);
+  }
+
+  else {    /* old bitmap format */
+    biWidth         = getshort(fp);          /* Types have changed ! */
+    biHeight        = getshort(fp);
+    biPlanes        = getshort(fp);
+    biBitCount      = getshort(fp);
+    
+    /* Not in old versions so have to compute them*/
+    biSizeImage = (((biPlanes * biBitCount*biWidth)+31)/32)*4*biHeight;
+    
+    biCompression   = BI_RGB; 
+    biXPelsPerMeter = biYPelsPerMeter = 0;
+    biClrUsed       = biClrImportant  = 0;
+  }
 
   if (DEBUG>1) {
     fprintf(stderr,"\nLoadBMP:\tbfSize=%d, bfOffBits=%d\n",bfSize,bfOffBits);
@@ -161,7 +123,7 @@ int LoadBMP(fname, pinfo)
 	    biXPelsPerMeter, biYPelsPerMeter, biClrUsed, biClrImportant);
   }
 
-  if (ferror(fp)) { bmpError(bname,"EOF reached in file header"); goto ERROR; }
+  if (FERROR(fp)) { bmpError(bname,"EOF reached in file header"); goto ERROR; }
 
 
   /* error checking */
@@ -187,25 +149,31 @@ int LoadBMP(fname, pinfo)
   }
 
 
-  
-  /* skip ahead to colormap, using biSize */
-  c = biSize - 40;    /* 40 bytes read from biSize to biClrImportant */
-  for (i=0; i<c; i++) getc(fp);
-
+  bPad = 0;
+  if (biSize != WIN_OS2_OLD) {
+    /* skip ahead to colormap, using biSize */
+    c = biSize - 40;    /* 40 bytes read from biSize to biClrImportant */
+    for (i=0; i<c; i++) getc(fp);
+    
+    bPad = bfOffBits - (biSize + 14);
+  }
 
   /* load up colormap, if any */
   if (biBitCount!=24) {
     int i, cmaplen;
 
-    cmaplen = 1 << biBitCount;
+    cmaplen = (biClrUsed) ? biClrUsed : 1 << biBitCount;
     for (i=0; i<cmaplen; i++) {
       pinfo->b[i] = getc(fp);
       pinfo->g[i] = getc(fp);
       pinfo->r[i] = getc(fp);
-      getc(fp);         /* unused */
+      if (biSize != WIN_OS2_OLD) {
+	getc(fp);
+	bPad -= 4;
+      }
     }
 
-    if (ferror(fp)) 
+    if (FERROR(fp)) 
       { bmpError(bname,"EOF reached in BMP colormap"); goto ERROR; }
 
     if (DEBUG>1) {
@@ -217,15 +185,24 @@ int LoadBMP(fname, pinfo)
     }
   }
 
+  if (biSize != WIN_OS2_OLD) {
+    /* Waste any unused bytes between the colour map (if present)
+       and the start of the actual bitmap data. */
+    
+    while (bPad > 0) {
+      (void) getc(fp);
+      bPad--;
+    }
+  }
 
   /* create pic8 or pic24 */
 
   if (biBitCount==24) {
-    pic24 = (byte *) calloc(biWidth * biHeight * 3, 1);
+    pic24 = (byte *) calloc((size_t) biWidth * biHeight * 3, (size_t) 1);
     if (!pic24) return (bmpError(bname, "couldn't malloc 'pic24'"));
   }
   else {
-    pic8 = (byte *) calloc(biWidth * biHeight, 1);
+    pic8 = (byte *) calloc((size_t) biWidth * biHeight, (size_t) 1);
     if (!pic8) return(bmpError(bname, "couldn't malloc 'pic8'"));
   }
 
@@ -258,12 +235,16 @@ int LoadBMP(fname, pinfo)
   else if (biCompression == BI_RLE8) cmpstr = ", RLE8 compressed";
 
   pinfo->w = biWidth;  pinfo->h = biHeight;
+  pinfo->normw = pinfo->w;   pinfo->normh = pinfo->h;
   pinfo->frmType = F_BMP;
   pinfo->colType = F_FULLCOLOR;
 
-  sprintf(pinfo->fullInfo, "BMP, %d bit%s per pixel%s.  (%ld bytes)",
+  sprintf(pinfo->fullInfo, "%sBMP, %d bit%s per pixel%s.  (%ld bytes)",
+	  ((biSize==WIN_OS2_OLD) ? "Old OS/2 " :
+	   (biSize==WIN_NEW)     ? "Windows "  : ""),
 	  biBitCount,  (biBitCount == 1) ? "" : "s",
 	  cmpstr, filesize);
+
   sprintf(pinfo->shrtInfo, "%dx%d BMP.", biWidth, biHeight);
   pinfo->comment = (char *) NULL;
 
@@ -280,7 +261,7 @@ int LoadBMP(fname, pinfo)
 static int loadBMP1(fp, pic8, w, h)
      FILE *fp;
      byte *pic8;
-     int  w,h;
+     u_int  w,h;
 {
   int   i,j,c,bitnum,padw;
   byte *pp;
@@ -302,10 +283,10 @@ static int loadBMP1(fp, pic8, w, h)
 	c <<= 1;
       }
     }
-    if (ferror(fp)) break;
+    if (FERROR(fp)) break;
   }
 
-  return (ferror(fp));
+  return (FERROR(fp));
 }  
 
 
@@ -314,65 +295,65 @@ static int loadBMP1(fp, pic8, w, h)
 static int loadBMP4(fp, pic8, w, h, comp)
      FILE *fp;
      byte *pic8;
-     int  w,h,comp;
+     u_int  w,h,comp;
 {
   int   i,j,c,c1,x,y,nybnum,padw,rv;
   byte *pp;
-
-
+  
+  
   rv = 0;
   c = c1 = 0;
-
+  
   if (comp == BI_RGB) {   /* read uncompressed data */
     padw = ((w + 7)/8) * 8; /* 'w' padded to a multiple of 8pix (32 bits) */
-
+    
     for (i=h-1; i>=0; i--) {
       pp = pic8 + (i * w);
       if ((i&0x3f)==0) WaitCursor();
-
+      
       for (j=nybnum=0; j<padw; j++,nybnum++) {
 	if ((nybnum & 1) == 0) { /* read next byte */
 	  c = getc(fp);
 	  nybnum = 0;
 	}
-
+	
 	if (j<w) {
 	  *pp++ = (c & 0xf0) >> 4;
 	  c <<= 4;
 	}
       }
-      if (ferror(fp)) break;
+      if (FERROR(fp)) break;
     }
   }
-
+  
   else if (comp == BI_RLE4) {  /* read RLE4 compressed data */
     x = y = 0;  
     pp = pic8 + x + (h-y-1)*w;
-
+    
     while (y<h) {
       c = getc(fp);  if (c == EOF) { rv = 1;  break; }
-
+      
       if (c) {                                   /* encoded mode */
 	c1 = getc(fp);
 	for (i=0; i<c; i++,x++,pp++) 
 	  *pp = (i&1) ? (c1 & 0x0f) : ((c1>>4)&0x0f);
       }
-
+      
       else {    /* c==0x00  :  escape codes */
 	c = getc(fp);  if (c == EOF) { rv = 1;  break; }
-
+	
 	if      (c == 0x00) {                    /* end of line */
 	  x=0;  y++;  pp = pic8 + x + (h-y-1)*w;
 	} 
-
+	
 	else if (c == 0x01) break;               /* end of pic8 */
-
+	
 	else if (c == 0x02) {                    /* delta */
 	  c = getc(fp);  x += c;
 	  c = getc(fp);  y += c;
 	  pp = pic8 + x + (h-y-1)*w;
 	}
-
+	
 	else {                                   /* absolute mode */
 	  for (i=0; i<c; i++, x++, pp++) {
 	    if ((i&1) == 0) c1 = getc(fp);
@@ -382,15 +363,15 @@ static int loadBMP4(fp, pic8, w, h, comp)
 	  if (((c&3)==1) || ((c&3)==2)) getc(fp);  /* read pad byte */
 	}
       }  /* escape processing */
-      if (ferror(fp)) break;
+      if (FERROR(fp)) break;
     }  /* while */
   }
   
   else {
     fprintf(stderr,"unknown BMP compression type 0x%0x\n", comp);
   }
-
-  if (ferror(fp)) rv = 1;
+  
+  if (FERROR(fp)) rv = 1;
   return rv;
 }  
 
@@ -400,11 +381,11 @@ static int loadBMP4(fp, pic8, w, h, comp)
 static int loadBMP8(fp, pic8, w, h, comp)
      FILE *fp;
      byte *pic8;
-     int  w,h,comp;
+     u_int  w,h,comp;
 {
   int   i,j,c,c1,padw,x,y,rv;
   byte *pp;
-
+  
   rv = 0;
 
   if (comp == BI_RGB) {   /* read uncompressed data */
@@ -418,7 +399,7 @@ static int loadBMP8(fp, pic8, w, h, comp)
 	c = getc(fp);  if (c==EOF) rv = 1;
 	if (j<w) *pp++ = c;
       }
-      if (ferror(fp)) break;
+      if (FERROR(fp)) break;
     }
   }
 
@@ -458,7 +439,7 @@ static int loadBMP8(fp, pic8, w, h, comp)
 	  if (c & 1) getc(fp);  /* odd length run: read an extra pad byte */
 	}
       }  /* escape processing */
-      if (ferror(fp)) break;
+      if (FERROR(fp)) break;
     }  /* while */
   }
   
@@ -466,7 +447,7 @@ static int loadBMP8(fp, pic8, w, h, comp)
     fprintf(stderr,"unknown BMP compression type 0x%0x\n", comp);
   }
 
-  if (ferror(fp)) rv = 1;
+  if (FERROR(fp)) rv = 1;
   return rv;
 }  
 
@@ -476,11 +457,12 @@ static int loadBMP8(fp, pic8, w, h, comp)
 static int loadBMP24(fp, pic24, w, h)
      FILE *fp;
      byte *pic24;
-     int  w,h;
+     u_int  w,h;
 {
-  int   i,j,padb;
+  int   i,j,padb,rv;
   byte *pp;
 
+  rv = 0;
 
   padb = (4 - ((w*3) % 4)) & 0x03;  /* # of pad bytes to read at EOscanline */
 
@@ -489,17 +471,19 @@ static int loadBMP24(fp, pic24, w, h)
     if ((i&0x3f)==0) WaitCursor();
     
     for (j=0; j<w; j++) {
-      *pp++ = getc(fp);   /* red   */
-      *pp++ = getc(fp);   /* green */
-      *pp++ = getc(fp);   /* blue  */
+      pp[2] = getc(fp);   /* blue */
+      pp[1] = getc(fp);   /* green */
+      pp[0] = getc(fp);   /* red */
+      pp += 3;
     }
 
     for (j=0; j<padb; j++) getc(fp);
 
-    if (ferror(fp)) break;
+    rv = (FERROR(fp));
+    if (rv) break;
   }
 
-  return (ferror(fp));
+  return rv;
 }  
 
 
@@ -561,11 +545,11 @@ static byte pc2nc[256],r1[256],g1[256],b1[256];
 
 /*******************************************/
 int WriteBMP(fp,pic824,ptype,w,h,rmap,gmap,bmap,numcols,colorstyle)
-FILE *fp;
-byte *pic824;
-int   ptype,w,h;
-byte *rmap, *gmap, *bmap;
-int   numcols, colorstyle;
+     FILE *fp;
+     byte *pic824;
+     int   ptype,w,h;
+     byte *rmap, *gmap, *bmap;
+     int   numcols, colorstyle;
 {
   /*
    * if PIC8, and colorstyle == F_FULLCOLOR, F_GREYSCALE, or F_REDUCED,
@@ -592,7 +576,7 @@ int   numcols, colorstyle;
     /* generate a faked 8-bit per pixel image with a grayscale cmap,
        so that it can just fall through existing 8-bit code */
 
-    graypic = (byte *) malloc(w*h);
+    graypic = (byte *) malloc((size_t) w*h);
     if (!graypic) FatalError("unable to malloc in WriteBMP()");
 
     for (i=0,sp=pic824,dp=graypic; i<w*h; i++,sp+=3, dp++) {
@@ -657,13 +641,13 @@ int   numcols, colorstyle;
   /* compute filesize and write it */
   i = 14 +                /* size of bitmap file header */
       40 +                /* size of bitmap info header */
-      (cmaplen * 4) +     /* size of colormap */
+      (nc * 4) +          /* size of colormap */
       bperlin * h;        /* size of image data */
 
   putint(fp, i);
   putshort(fp, 0);        /* reserved1 */
   putshort(fp, 0);        /* reserved2 */
-  putint(fp, 14 + 40 + (cmaplen * 4));  /* offset from BOfile to BObitmap */
+  putint(fp, 14 + 40 + (nc * 4));  /* offset from BOfile to BObitmap */
 
   putint(fp, 40);         /* biSize: size of bitmap info header */
   putint(fp, w);          /* biWidth */
@@ -679,7 +663,7 @@ int   numcols, colorstyle;
 
 
   /* write out the colormap */
-  for (i=0; i<cmaplen; i++) {
+  for (i=0; i<nc; i++) {
     if (colorstyle == F_GREYSCALE) {
       j = MONO(r1[i],g1[i],b1[i]);
       putc(j,fp);  putc(j,fp);  putc(j,fp);  putc(0,fp);
@@ -700,7 +684,11 @@ int   numcols, colorstyle;
 
   if (graypic) free(graypic);
 
-  if (ferror(fp)) return -1;
+#ifndef VMS
+  if (FERROR(fp)) return -1;
+#else
+  if (!FERROR(fp)) return -1;
+#endif
   
   return 0;
 }
@@ -811,9 +799,10 @@ static void writeBMP24(fp, pic24, w, h)
     if ((i&0x3f)==0) WaitCursor();
 
     for (j=0; j<w; j++) {
-      putc(*pp++, fp);
-      putc(*pp++, fp);
-      putc(*pp++, fp);
+      putc(pp[2], fp);
+      putc(pp[1], fp);
+      putc(pp[0], fp);
+      pp += 3;
     }
 
     for (j=0; j<padb; j++) putc(0, fp);
