@@ -2,64 +2,21 @@
  * xvgam.c
  *
  * callable functions:
- *   <many>
- */
-
-/* Copyright Notice
- * ================
- * Copyright 1989, 1990, 1991, 1992, 1993 by John Bradley
- * 
- * Permission to use, copy, and distribute XV in its entirety, for 
- * non-commercial purposes, is hereby granted without fee, provided that
- * this license information and copyright notice appear in all copies.
- * 
- * Note that distributing XV 'bundled' in with ANY product is considered
- * to be a 'commercial purpose'.
- *
- * Also note that any copies of XV that are distributed MUST be built
- * and/or configured to be in their 'unregistered copy' mode, so that it
- * is made obvious to the user that XV is shareware, and that they should
- * consider donating, or at least reading this License Info.
- * 
- * The software may be modified for your own purposes, but modified
- * versions may NOT be distributed without prior consent of the author.
- * 
- * This software is provided 'as-is', without any express or implied
- * warranty.  In no event will the author be held liable for any damages
- * arising from the use of this software.
- * 
- * If you would like to do something with XV that this copyright
- * prohibits (such as distributing it with a commercial product, 
- * using portions of the source in some other program, etc.), please
- * contact the author (preferably via email).  Arrangements can
- * probably be worked out.
- *
- * XV is shareware for PERSONAL USE only.  You may use XV for your own
- * amusement, and if you find it nifty, useful, generally cool, or of
- * some value to you, your non-deductable donation would be greatly
- * appreciated.  $25 is the suggested donation, though, of course,
- * larger donations are quite welcome.  Folks who donate $25 or more
- * can receive a Real Nice bound copy of the XV manual for no extra
- * charge.
- * 
- * Commercial, government, and institutional users MUST register their
- * copies of XV, for the exceedingly REASONABLE price of just $25 per
- * workstation/X terminal.  Site licenses are available for those who
- * wish to run XV on a large number of machines.  Contact the author
- * for more details.
- *
- * The author may be contacted via:
- *    US Mail:  John Bradley
- *              1053 Floyd Terrace
- *              Bryn Mawr, PA  19010
- *
- *    Phone:    (215) 898-8813
- *    EMail:    bradley@cis.upenn.edu
+ *   <too many to list>
  */
 
+#include "copyright.h"
 
 #include "xv.h"
-#include "bitmaps.h"
+
+#include "bits/h_rotl"
+#include "bits/h_rotr"
+#include "bits/h_flip"
+#include "bits/h_sinc"
+#include "bits/h_sdec"
+#include "bits/h_sat"
+#include "bits/h_desat"
+
 
 #define GAMW 664
 #define GAMH 518
@@ -128,9 +85,10 @@ struct gamstate { struct hmap hmap[N_HMAP];
 		  int wht_stval, wht_satval, wht_enab;
 		  int satval;
 		  GRAF_STATE istate, rstate, gstate, bstate;
-		} undo[MAXUNDO], preset[4], defstate; 
+		};
 
-struct gamstate *defLoadState;
+static struct gamstate undo[MAXUNDO], preset[4], defstate; 
+static struct gamstate *defLoadState;
 
 static int uptr, uhead, utail;
 
@@ -146,7 +104,7 @@ typedef struct huedial {
 		 int    satval;   /* saturation value on non-range dial */
 		 BUTT   hdbutt[N_HDBUTT];
 		 CBUTT  enabCB;
-		 void (*drawobj)();
+		 void (*drawobj)PARM((void));
 	       } HDIAL;
 
 
@@ -162,72 +120,53 @@ static struct hmap hmap[N_HMAP];
 static int    hremap[360];
 
 static int    defAutoApply;
-
 static int    hsvnonlinear = 0;
 
+static void printUTime       PARM((char *));
 
-#ifdef __STDC__
-static void computeHSVlinear(void);
-static void changedGam(void);
-static void drawGam(int,int,int,int);
-static void drawBut(int,int,int,int);
-static void drawArrow(int, int);
-static void drawCmap(void);
-static void clickCmap(int,int,int);
-static void clickGam(int,int);
-static void selectCell(int,int);
-static int  deladdCell(int, int);
-static void doCmd(int);
-static void SetHSVmode(void);
-static void applyGamma(int);
-static void calcHistEQ(int *, int *, int *);
-static void saveGamState(void);
-static void gamUndo(void);
-static void gamRedo(void);
-static void ctrls2gamstate(struct gamstate *);
-static void gamstate2ctrls(struct gamstate *);
-static void rndCols(void);
-static void saveCMap(struct cmapstate *);
-static void restoreCMap(struct cmapstate *);
-static void parseResources(void);
-static void makeResources(void);
+static void computeHSVlinear PARM((void));
+static void changedGam       PARM((void));
+static void drawGam          PARM((int,int,int,int));
+static void drawBut          PARM((int,int,int,int));
+static void drawArrow        PARM((int, int));
+static void drawCmap         PARM((void));
+static void clickCmap        PARM((int,int,int));
+static void clickGam         PARM((int,int));
+static void selectCell       PARM((int,int));
+static int  deladdCell       PARM((int, int));
+static void doCmd            PARM((int));
+static void SetHSVmode       PARM((void));
+static void applyGamma       PARM((int));
+static void calcHistEQ       PARM((int *, int *, int *));
+static void saveGamState     PARM((void));
+static void gamUndo          PARM((void));
+static void gamRedo          PARM((void));
+static void ctrls2gamstate   PARM((struct gamstate *));
+static void gamstate2ctrls   PARM((struct gamstate *));
+static void rndCols          PARM((void));
+static void saveCMap         PARM((struct cmapstate *));
+static void restoreCMap      PARM((struct cmapstate *));
+static void parseResources   PARM((void));
+static void makeResources    PARM((void));
 
-static void dragGamma(void);
-static void dragHueDial(void);
-static void dragEditColor(void);
+static void dragGamma        PARM((void));
+static void dragHueDial      PARM((void));
+static void dragEditColor    PARM((void));
 
-static void HDCreate(HDIAL *, Window, int, int, int, int, int, int, char *,
- 		     u_long, u_long);
-static void HDRedraw(HDIAL *, int);
-static int  HDClick(HDIAL *, int, int);
-static int  HDTrack(HDIAL *, int, int);
-static int  hdg2xdg(int);
-static void pol2xy(int, int, double, int, int *, int *);
-static int  computeHDval(HDIAL *, int, int);
-static void initHmap(void);
-static void init1hmap(int);
-static void dials2hmap(void);
-static void hmap2dials(void);
-static void build_hremap(void);
+static void HDCreate         PARM((HDIAL *, Window, int, int, int, int, 
+				   int, int, char *, u_long, u_long));
 
-#else
-static void computeHSVlinear();
-static void changedGam(), drawGam(), drawBut(), drawArrow(), drawCmap();
-static void clickCmap(), clickGam(), selectCell();
-static void doCmd(), applyGamma(), calcHistEQ();
-static int  deladdCell();
-static void SetHSVmode(), saveGamState(), gamUndo(), gamRedo();
-static void ctrls2gamstate(), gamstate2ctrls();
-static void rndCols(), saveCMap(), restoreCMap(), parseResources();
-static void makeResources();
-
-static void dragGamma(), dragHueDial(), dragEditColor();
-
-static void HDCreate(), HDRedraw(), pol2xy();
-static int  HDClick(), HDTrack(), hdg2xdg(), computeHDval();
-static void initHmap(), init1hmap(), dials2hmap(), hmap2dials();
-static void build_hremap();
-#endif
+static void HDRedraw         PARM((HDIAL *, int));
+static int  HDClick          PARM((HDIAL *, int, int));
+static int  HDTrack          PARM((HDIAL *, int, int));
+static int  hdg2xdg          PARM((int));
+static void pol2xy           PARM((int, int, double, int, int *, int *));
+static int  computeHDval     PARM((HDIAL *, int, int));
+static void initHmap         PARM((void));
+static void init1hmap        PARM((int));
+static void dials2hmap       PARM((void));
+static void hmap2dials       PARM((void));
+static void build_hremap     PARM((void));
 
 
 
@@ -608,7 +547,7 @@ XEvent *xev;
 		 hicol, locol, infobg);
 
       XSetForeground(theDisp, theGC, infofg);
-      ULineString(hsvF, "HSV Modification", 3, 2+ASCENT);
+      ULineString(hsvF, 3, 2+ASCENT, "HSV Modification");
 
       HDRedraw(&srcHD, HD_ALL);
       HDRedraw(&dstHD, HD_ALL);
@@ -623,7 +562,7 @@ XEvent *xev;
       Draw3dRect(rgbF, 0,0, RGBF_WIDE-1, RGBF_HIGH-1, R3D_IN, 2,
 		 hicol, locol, infobg);
       XSetForeground(theDisp, theGC, infofg);
-      ULineString(rgbF, "RGB Modification", 3, 2+ASCENT);
+      ULineString(rgbF, 3, 2+ASCENT, "RGB Modification");
     }
 
     else rv = 0;
@@ -748,6 +687,8 @@ XEvent *xev;
     stlen = XLookupString(e,buf,128,&ks,(XComposeStatus *) NULL);
     buf[stlen] = '\0';
 
+    RemapKeyCheck(ks, buf, &stlen);
+
     if (!stlen) return 0;
 
     else if (e->window == intGraf.win) rv = GrafKey(&intGraf,buf);
@@ -877,7 +818,7 @@ static void drawCmap()
 	     hicol, locol, infobg);
 
   XSetForeground(theDisp, theGC, infofg);
-  ULineString(cmapF, "Colormap Editing", 3, 2+ASCENT);
+  ULineString(cmapF, 3, 2+ASCENT, "Colormap Editing");
 
   for (i=0; i<G_NBUTTS; i++) {
     if (gbut[i].win == cmapF) BTRedraw(&gbut[i]);
@@ -932,8 +873,8 @@ void RedrawCMap()
   XSetForeground(theDisp, theGC, infofg);
 
   if (picType != PIC8) {
-    CenterString(cmapF, "No colormap in 24-bit mode.", 
-		 CMAPX + CMAPW/2, CMAPY + CMAPH/2);
+    CenterString(cmapF, CMAPX + CMAPW/2, CMAPY + CMAPH/2, 
+		 "No colormap in 24-bit mode.");
     return;
   }
       
@@ -1127,7 +1068,7 @@ int x,y,but;
 	lastcell = curcell;
 
 	j = XGrabPointer(theDisp, cmapF, False, 0, GrabModeAsync, 
-		     GrabModeAsync, None, None, CurrentTime);
+			 GrabModeAsync, None, None, (Time) CurrentTime);
 	while (1) {
 	  Window       rW,cW;
 	  int          rx,ry,x,y;
@@ -1157,7 +1098,7 @@ int x,y,but;
 	  }
 	} /* while */
 
-	XUngrabPointer(theDisp, CurrentTime);
+	XUngrabPointer(theDisp, (Time) CurrentTime);
 
 	/* restore all cells that aren't in curgroup, unless curgroup=0,
 	   in which case restore everything...  nasty */
@@ -1399,7 +1340,7 @@ void GenerateFSGamma()
      the output range 0-255.  The reponse curve will be slightly 's' shaped. */
 
   int i,j;
-  static int x[4] = {0,32,224,255};
+  static int x[4] = {0,16,240,255};
   static int y[4] = {0, 0,255,255};
   double yf[4];
 
@@ -1663,18 +1604,20 @@ static void applyGamma(cmapchange)
   int i,j;
   byte oldr[256], oldg[256], oldb[256];
 
+  if (!pic || !epic) return;   /* called before image exists.  ignore */
+
   if (picType == PIC8) {
     /* save current 'desired' colormap */
-    xvbcopy(rMap, oldr, numcols);
-    xvbcopy(gMap, oldg, numcols);
-    xvbcopy(bMap, oldb, numcols);
+    xvbcopy((char *) rMap, (char *) oldr, (size_t) numcols);
+    xvbcopy((char *) gMap, (char *) oldg, (size_t) numcols);
+    xvbcopy((char *) bMap, (char *) oldb, (size_t) numcols);
 
     GammifyColors();
 
     /* if current 'desired' colormap hasn't changed, don't DO anything */
-    if (!xvbcmp(rMap, oldr, numcols) && 
-	!xvbcmp(gMap, oldg, numcols) && 
-	!xvbcmp(bMap, oldb, numcols)) return;
+    if (!xvbcmp((char *) rMap, (char *) oldr, (size_t) numcols) && 
+	!xvbcmp((char *) gMap, (char *) oldg, (size_t) numcols) && 
+	!xvbcmp((char *) bMap, (char *) oldb, (size_t) numcols)) return;
 
     /* special case: if using R/W color, just modify the colors and leave */
     if (allocMode==AM_READWRITE && rwthistime && 
@@ -1709,7 +1652,14 @@ static void applyGamma(cmapchange)
     }
     
     FreeColors();
-    SortColormap();
+
+    {
+      byte trans[256];
+      SortColormap(pic,pWIDE,pHIGH,&numcols,rMap,gMap,bMap,
+		   colAllocOrder, trans);
+      ColorCompress8(trans);
+    }
+
     AllocColors();
 
 
@@ -1741,7 +1691,8 @@ static void calcHistEQ(histeq, rminv, rmaxv)
     }
 
     /* compute intensity histogram */
-    for (i=eWIDE*eHIGH,ep=epic; i>0; i--, ep++) hist[rgb[*ep]]++;
+    if (epic) for (i=eWIDE*eHIGH,ep=epic; i>0; i--, ep++) hist[rgb[*ep]]++;
+         else for (i=pWIDE*pHIGH,ep=pic;  i>0; i--, ep++) hist[rgb[*ep]]++;
 
     /* compute minv/maxv values */
     for (i=0; i<256 && !hist[i]; i++);
@@ -1757,13 +1708,21 @@ static void calcHistEQ(histeq, rminv, rmaxv)
     for (i=0; i<256; i++) hist[i] = 0;
 
     minv = 255;  maxv = 0;
-    for (i=0,ep=epic; i<eWIDE*eHIGH; i++,ep+=3) {
-      v = MONO(ep[0],ep[1],ep[2]);
-
-      if (v<minv) minv = v;
-      if (v>maxv) maxv = v;
-      
-      hist[v]++;
+    if (epic) {
+      for (i=0,ep=epic; i<eWIDE*eHIGH; i++,ep+=3) {
+	v = MONO(ep[0],ep[1],ep[2]);
+	if (v<minv) minv = v;
+	if (v>maxv) maxv = v;
+	hist[v]++;
+      }
+    }
+    else {
+      for (i=0,ep=pic; i<pWIDE*pHIGH; i++,ep+=3) {
+	v = MONO(ep[0],ep[1],ep[2]);
+	if (v<minv) minv = v;
+	if (v>maxv) maxv = v;
+	hist[v]++;
+      }
     }
     
     *rminv = minv;  *rmaxv = maxv;
@@ -1818,6 +1777,8 @@ void DoHistEq()
     XClearWindow(theDisp, intGraf.gwin);
     RedrawGraf(&intGraf, 1);
   }
+
+  computeHSVlinear();
 }
 
 
@@ -1856,6 +1817,8 @@ void DoNorm()
   if (maxv>254) { gs.hands[2].x = gs.hands[2].y = 254; }
 
   SetGrafState(&intGraf, &gs);
+
+  computeHSVlinear();
 }
 
 
@@ -1874,6 +1837,8 @@ void GammifyColors()
       rMap[i] = rcmap[i];
       gMap[i] = gcmap[i];
       bMap[i] = bcmap[i];
+      if (!ncols) 
+	cols[i] = (rMap[i] + gMap[i] + bMap[i] >= 128*3) ? white : black;
     }
   }
 }
@@ -1905,7 +1870,7 @@ int col;
     vi = (int) floor((v * 255.0) + 0.5);
     if (DEBUG>1) fprintf(stderr," (vi=%d)",vi);
 
-    v = intGraf.func[vi] / 255.0;
+    v = ((double) intGraf.func[vi]) / 255.0;
     if (DEBUG>1) fprintf(stderr," (v=%f)",v);
 
     if (h>=0) {
@@ -1936,6 +1901,10 @@ int col;
   rMap[col] = rGraf.func[rv];  
   gMap[col] = gGraf.func[gv];
   bMap[col] = bGraf.func[bv];
+
+  if (!ncols) 
+    cols[col] = (rMap[col] + gMap[col] + bMap[col] >= 128*3) ? white : black;
+
   if (DEBUG>1) fprintf(stderr," -> %d,%d,%d\n",rMap[col],gMap[col],bMap[col]);
 }
 
@@ -2259,7 +2228,7 @@ static void parseResources()
 
     for (j=0; j<6; j++) {                       /* xv.*.huemap resources */
       sprintf(tmp, "%s.huemap%d", gname, j+1);
-      if (rd_str_cl(tmp, "Setting.Huemap")) {   /* got one */
+      if (rd_str_cl(tmp, "Setting.Huemap",0)) {   /* got one */
 	int fst, fen, tst, ten;
 	char fcw[32], tcw[32];
 
@@ -2284,7 +2253,7 @@ static void parseResources()
     }
 
     sprintf(tmp, "%s.whtmap", gname);           /* xv.*.whtmap resource */
-    if (rd_str_cl(tmp, "Setting.Whtmap")) {        /* got one */
+    if (rd_str_cl(tmp, "Setting.Whtmap",0)) {        /* got one */
       int wst, wsat, enab;
       if (DEBUG) fprintf(stderr,"parseResource 'xv.%s: %s'\n",tmp, def_str);
       if (sscanf(def_str,"%d %d %d", &wst, &wsat, &enab) != 3) {
@@ -2300,7 +2269,7 @@ static void parseResources()
     }
 
     sprintf(tmp, "%s.satval", gname);           /* xv.*.satval resource */
-    if (rd_str_cl(tmp, "Setting.Satval")) {         /* got one */
+    if (rd_str_cl(tmp, "Setting.Satval",0)) {         /* got one */
       int sat;
       if (DEBUG) fprintf(stderr,"parseResource 'xv.%s: %s'\n",tmp, def_str);
       if (sscanf(def_str,"%d", &sat) != 1) {
@@ -2323,7 +2292,7 @@ static void parseResources()
       default: sprintf(tmp, "%s.bgraf", gname);  gsgst = &gs.bstate; break;
       }
 
-      if (rd_str_cl(tmp, "Setting.Graf")) {       /* got one */
+      if (rd_str_cl(tmp, "Setting.Graf",0)) {       /* got one */
 	strcpy(tmp1, def_str);
 	xvbcopy((char *) gsgst, (char *) &gstat, sizeof(GRAF_STATE));
 	if (DEBUG) fprintf(stderr,"parseResource 'xv.%s: %s'\n",tmp, tmp1);
@@ -2397,7 +2366,8 @@ static void makeResources()
     sprintf(tmp, "%s: %s\n", rname, tmp1);
     strcat(rsrc, tmp);
   }
-  XStoreBytes(theDisp, rsrc, strlen(rsrc));
+
+  NewCutBuffer(rsrc);
 }
     
 
@@ -2434,7 +2404,7 @@ static void dragHueDial()
 
 
 /*****************************/
-static void dragEditColor ()
+static void dragEditColor()
 {
   /* called through DIAL.drawobj
      while color editor ctrls are being dragged
@@ -2443,8 +2413,6 @@ static void dragEditColor ()
   
   if (dragCB.val && dragCB.active) ApplyEditColor(0);
 }
- 
-
 
 
     
@@ -2487,38 +2455,27 @@ u_long fg,bg;
   hd->drawobj = NULL;
 
   if (!hdb_pixmaps_built) {
-    hdbpix1[HDB_ROTL]   = XCreatePixmapFromBitmapData(theDisp, win,
-                                    h_rotl_bits, PW,PH, 1, 0, 1);
-    hdbpix1[HDB_ROTR]   = XCreatePixmapFromBitmapData(theDisp, win,
-                                    h_rotr_bits, PW,PH, 1, 0, 1);
-    hdbpix1[HDB_FLIP]   = XCreatePixmapFromBitmapData(theDisp, win,
-                                    h_flip_bits, PW,PH, 1, 0, 1);
-    hdbpix1[HDB_EXPND]  = XCreatePixmapFromBitmapData(theDisp, win,
-                                    h_sinc_bits, PW,PH, 1, 0, 1);
-    hdbpix1[HDB_SHRNK]  = XCreatePixmapFromBitmapData(theDisp, win,
-                                    h_sdec_bits, PW,PH, 1, 0, 1);
-
-    hdbpix2[HDB_SAT]    = XCreatePixmapFromBitmapData(theDisp, win,
-                                    h_sat_bits, PW,PH, 1, 0, 1);
-    hdbpix2[HDB_DESAT]  = XCreatePixmapFromBitmapData(theDisp, win,
-                                    h_desat_bits, PW,PH, 1, 0, 1);
+    hdbpix1[HDB_ROTL]   = MakePix1(win, h_rotl_bits, PW,PH);
+    hdbpix1[HDB_ROTR]   = MakePix1(win, h_rotr_bits, PW,PH);
+    hdbpix1[HDB_FLIP]   = MakePix1(win, h_flip_bits, PW,PH);
+    hdbpix1[HDB_EXPND]  = MakePix1(win, h_sinc_bits, PW,PH);
+    hdbpix1[HDB_SHRNK]  = MakePix1(win, h_sdec_bits, PW,PH);
+    hdbpix2[HDB_SAT]    = MakePix1(win, h_sat_bits,  PW,PH);
+    hdbpix2[HDB_DESAT]  = MakePix1(win, h_desat_bits,PW,PH);
 
     hdbpix2[HDB_ROTL]  = hdbpix1[HDB_ROTL];
     hdbpix2[HDB_ROTR]  = hdbpix1[HDB_ROTR];
   }
 
     
+#define BCOLS fg,bg,hicol,locol
+
   if (hd->range) {
-    BTCreate(&hd->hdbutt[HDB_ROTL], win, x-50,y+60,18,18,NULL,fg,bg,
-	     hicol,locol);
-    BTCreate(&hd->hdbutt[HDB_ROTR], win, x-30,y+60,18,18,NULL,fg,bg,
-	     hicol,locol);
-    BTCreate(&hd->hdbutt[HDB_FLIP], win, x-10,y+60,18,18,NULL,fg,bg,
-	     hicol, locol);
-    BTCreate(&hd->hdbutt[HDB_EXPND],win, x+10,y+60,18,18,NULL,fg,bg,
-	     hicol, locol);
-    BTCreate(&hd->hdbutt[HDB_SHRNK],win, x+30,y+60,18,18,NULL,fg,bg, 
-	     hicol, locol);
+    BTCreate(&hd->hdbutt[HDB_ROTL], win, x-50,y+60,18,18,NULL, BCOLS);
+    BTCreate(&hd->hdbutt[HDB_ROTR], win, x-30,y+60,18,18,NULL, BCOLS);
+    BTCreate(&hd->hdbutt[HDB_FLIP], win, x-10,y+60,18,18,NULL, BCOLS);
+    BTCreate(&hd->hdbutt[HDB_EXPND],win, x+10,y+60,18,18,NULL, BCOLS);
+    BTCreate(&hd->hdbutt[HDB_SHRNK],win, x+30,y+60,18,18,NULL, BCOLS);
 
     for (i=0; i<N_HDBUTT; i++) {
       hd->hdbutt[i].pix = hdbpix1[i];
@@ -2528,15 +2485,11 @@ u_long fg,bg;
   }
 
   else {
-    BTCreate(&hd->hdbutt[HDB_ROTL], win, x-39,y+60,18,18,NULL,fg,bg,
-	     hicol, locol);
-    BTCreate(&hd->hdbutt[HDB_ROTR], win, x-19,y+60,18,18,NULL,fg,bg,
-	     hicol, locol);
-    BTCreate(&hd->hdbutt[HDB_DESAT],win, x+1, y+60,18,18,NULL,fg,bg,
-	     hicol, locol);
-    BTCreate(&hd->hdbutt[HDB_SAT],  win, x+21,y+60,18,18,NULL,fg,bg,
-	     hicol,locol);
-    CBCreate(&hd->enabCB, win, x+23, y-44, "", fg, bg, hicol,locol);
+    BTCreate(&hd->hdbutt[HDB_ROTL], win, x-39,y+60,18,18,NULL, BCOLS);
+    BTCreate(&hd->hdbutt[HDB_ROTR], win, x-19,y+60,18,18,NULL, BCOLS);
+    BTCreate(&hd->hdbutt[HDB_DESAT],win, x+1, y+60,18,18,NULL, BCOLS);
+    BTCreate(&hd->hdbutt[HDB_SAT],  win, x+21,y+60,18,18,NULL, BCOLS);
+    CBCreate(&hd->enabCB, win, x+23, y-44, "", BCOLS);
     hd->enabCB.val = 1;
 
     for (i=0; i<N_HDBUTT2; i++) {
@@ -2545,6 +2498,7 @@ u_long fg,bg;
       hd->hdbutt[i].ph = PH;
     }
   }
+#undef BCOLS
 }
 
 
@@ -2581,7 +2535,7 @@ int flags;
 
       tstr[0] = colstr[i];  tstr[1] = '\0';
       pol2xy(hd->x, hd->y, a, HD_RADIUS+10, &x, &y);
-      CenterString(hd->win, tstr, x,y);
+      CenterString(hd->win, x, y, tstr);
     }
   }
 
@@ -2704,16 +2658,16 @@ int flags;
     }
       
     XDrawImageString(theDisp, hd->win, theGC,
-		     hd->x - XTextWidth(monofinfo, vstr, strlen(vstr))/2,
-		     hd->y + HD_RADIUS + 24, vstr, strlen(vstr));
+		     hd->x - XTextWidth(monofinfo, vstr, (int) strlen(vstr))/2,
+		     hd->y + HD_RADIUS + 24, vstr, (int) strlen(vstr));
     XSetFont(theDisp, theGC, mfont);
   }
 
 
   if (flags & HD_TITLE) {
     XSetForeground(theDisp, theGC, hd->fg);
-    ULineString(hd->win, hd->str, hd->x - HD_RADIUS - 15,
-		hd->y - HD_RADIUS - 4);
+    ULineString(hd->win, hd->x - HD_RADIUS - 15, hd->y - HD_RADIUS - 4, 
+		hd->str);
   }
 
 
@@ -2730,7 +2684,7 @@ int flags;
   if (!hd->range && !hd->enabCB.val) {  /* draw dimmed */
     XSync(theDisp, False);
     DimRect(hd->win, hd->x-HD_RADIUS-15, hd->y-HD_RADIUS-4-ASCENT,
-	    2*HD_RADIUS+30,(2*HD_RADIUS+4+ASCENT+80), hd->bg);
+	    (u_int) 2*HD_RADIUS+30, (u_int) (2*HD_RADIUS+4+ASCENT+80), hd->bg);
     XSync(theDisp, False);
     CBRedraw(&hd->enabCB);
   }
@@ -3269,7 +3223,7 @@ byte *GammifyPic24(pic24, wide, high)
 
   printUTime("NonLINEAR");
 
-  outpic = (byte *) malloc(wide * high * 3);
+  outpic = (byte *) malloc((size_t) wide * high * 3);
   if (!outpic) return outpic;
 
   pp = pic24;  op = outpic;
