@@ -1,9 +1,6 @@
 /*
  * xvmisc.c - random 'handy' routines used in XV
  *
- *  Author:    John Bradley, University of Pennsylvania
- *                (bradley@cis.upenn.edu)
- *
  *  Contains:
  *     Window CreateWindow(name, clname, geom, w, h, fg, bg, usesize)
  *     void   CenterString(win, str, x, y)
@@ -12,10 +9,12 @@
  *     void   FakeButtonPress(bptr)
  *     void   FakeKeyPress(win, keysym);
  *     void   GenExpose(win, x, y, w, h);
+ *     void   RemapKeyCheck(ks, buf, stlen)
  *     void   xvDestroyImage(XImage *);
  *     void   DimRect(win, x, y, w, h, bg);
  *     void   Draw3dRect(win, x,y,w,h, inout, bwidth, hicol, locol);
- *     void   SetCropString(active)
+ *     void   SetCropString()
+ *     void   SetSelectionString()
  *     void   Warning()
  *     void   FatalError(str)
  *     void   Quit(int)
@@ -23,75 +22,32 @@
  *     void   WaitCursor()
  *     void   SetCursors(int)
  *     char  *BaseName(char *)
- *     void   DrawTempGauge(win, x,y,w,h, percent, fg,bg,hi,lo)
+ *     void   DrawTempGauge(win, x,y,w,h, percent, fg,bg,hi,lo, str)
+ *     void   ProgressMeter(min, max, val, str);
  *     void   xvbcopy(src, dst, length)
  *     int    xvbcmp (s1,  s2,  length)
  *     void   xvbzero(s, length)
+ *     char  *xv_strstr(s1, s2)
  *     void   Timer(milliseconds)
  */
 
-/* Copyright Notice
- * ================
- * Copyright 1989, 1990, 1991, 1992, 1993 by John Bradley
- * 
- * Permission to use, copy, and distribute XV in its entirety, for 
- * non-commercial purposes, is hereby granted without fee, provided that
- * this license information and copyright notice appear in all copies.
- * 
- * Note that distributing XV 'bundled' in with ANY product is considered
- * to be a 'commercial purpose'.
- *
- * Also note that any copies of XV that are distributed MUST be built
- * and/or configured to be in their 'unregistered copy' mode, so that it
- * is made obvious to the user that XV is shareware, and that they should
- * consider donating, or at least reading this License Info.
- * 
- * The software may be modified for your own purposes, but modified
- * versions may NOT be distributed without prior consent of the author.
- * 
- * This software is provided 'as-is', without any express or implied
- * warranty.  In no event will the author be held liable for any damages
- * arising from the use of this software.
- * 
- * If you would like to do something with XV that this copyright
- * prohibits (such as distributing it with a commercial product, 
- * using portions of the source in some other program, etc.), please
- * contact the author (preferably via email).  Arrangements can
- * probably be worked out.
- *
- * XV is shareware for PERSONAL USE only.  You may use XV for your own
- * amusement, and if you find it nifty, useful, generally cool, or of
- * some value to you, your non-deductable donation would be greatly
- * appreciated.  $25 is the suggested donation, though, of course,
- * larger donations are quite welcome.  Folks who donate $25 or more
- * can receive a Real Nice bound copy of the XV manual for no extra
- * charge.
- * 
- * Commercial, government, and institutional users MUST register their
- * copies of XV, for the exceedingly REASONABLE price of just $25 per
- * workstation/X terminal.  Site licenses are available for those who
- * wish to run XV on a large number of machines.  Contact the author
- * for more details.
- *
- * The author may be contacted via:
- *    US Mail:  John Bradley
- *              1053 Floyd Terrace
- *              Bryn Mawr, PA  19010
- *
- *    Phone:    (215) 898-8813
- *    EMail:    bradley@cis.upenn.edu
- */
-
+#include "copyright.h"
 
 #define NEEDSTIME
 #include "xv.h"
-#include "bitmaps.h"
 
-#ifdef __STDC__
-static void set_cursors(Cursor, Cursor);
-#else
-static void set_cursors();
-#endif
+#include "bits/fc_left"
+#include "bits/fc_leftm"
+#include "bits/fc_left1"
+#include "bits/fc_left1m"
+#include "bits/fc_mid"
+#include "bits/fc_midm"
+#include "bits/fc_right1"
+#include "bits/fc_right1m"
+#include "bits/fc_right"
+#include "bits/fc_rightm"
+
+static void set_cursors PARM((Cursor, Cursor));
 
 static Atom atom_DELWIN = 0;
 static Atom atom_PROTOCOLS = 0;
@@ -174,9 +130,9 @@ unsigned long fg, bg;
   xswamask = CWBackPixel | CWBorderPixel | CWColormap;
   if (!usesize) xswamask |= CWBitGravity;
 
-  win = XCreateWindow(theDisp, rootW, x, y, w, h, 
-			bwidth, dispDEEP, InputOutput,
-			theVisual, xswamask, &xswa);
+  win = XCreateWindow(theDisp, rootW, x, y, (u_int) w, (u_int) h, 
+		      (u_int) bwidth, (int) dispDEEP, InputOutput,
+		      theVisual, xswamask, &xswa);
   if (!win) return(win);   /* leave immediately if couldn't create */
 
 
@@ -200,23 +156,32 @@ unsigned long fg, bg;
 
 
 /**************************************************/
-void CenterString(win,str,x,y)
-Window win;
-char *str;
-int x,y;
+void DrawString(win,x,y,str)
+     Window win;
+     int x,y;
+     char *str;
 {
-  XDrawString(theDisp, win, theGC, CENTERX(mfinfo, x, str),
-	    CENTERY(mfinfo, y), str, strlen(str));
+  XDrawString(theDisp, win, theGC, x, y, str, (int) strlen(str));
 }
 
   
 /**************************************************/
-void ULineString(win,str,x,y)
-Window win;
-char *str;
-int x,y;
+void CenterString(win,x,y,str)
+     Window win;
+     int x,y;
+     char *str;
 {
-  XDrawString(theDisp, win, theGC, x, y, str, strlen(str));
+  DrawString(win, CENTERX(mfinfo, x, str), CENTERY(mfinfo, y), str);
+}
+
+  
+/**************************************************/
+void ULineString(win,x,y,str)
+     Window win;
+     int x,y;
+     char *str;
+{
+  DrawString(win, x, y, str);
   XDrawLine(theDisp, win, theGC, x, y+DESCENT-1, 
 	    x+StringWidth(str), y+DESCENT-1);
 }
@@ -224,12 +189,79 @@ int x,y;
   
 /**************************************************/
 int StringWidth(str)
-char *str;
+     char *str;
 {
-  return(XTextWidth(mfinfo, str, strlen(str)));
+  return(XTextWidth(mfinfo, str, (int) strlen(str)));
 }
 
   
+/**************************************************/
+int CursorKey(ks, shift, dotrans)
+     KeySym ks;
+     int    shift, dotrans;
+{
+  /* called by the KeyPress/KeyRelease event handler to determine if a 
+     given keypress is a cursor key.  More complex than you'd think, since
+     certain Sun Keyboards generate a variety of odd keycodes, and not all
+     keyboards *have* all these keys.  Note that 'shifted' arrow keys
+     are not translated to PageUp/Down, since it's up to the particular
+     case whether that translation should occur */
+
+  /* if we don't have the new R5-R6 'XK_KP_*' cursor movement
+     symbols, make them. */
+
+#ifndef XK_KP_Up
+#  define XK_KP_Up         XK_Up
+#  define XK_KP_Down       XK_Down
+#  define XK_KP_Left       XK_Left
+#  define XK_KP_Right      XK_Right
+#  define XK_KP_Prior      XK_Prior
+#  define XK_KP_Next       XK_Next
+#  define XK_KP_Home       XK_Home
+#  define XK_KP_End        XK_End
+#endif
+
+
+
+  int  i = CK_NONE;
+
+  if      (ks==XK_Up    || ks==XK_KP_Up    || 
+	   ks==XK_KP_8  || ks==XK_F28)             i=CK_UP;
+
+  else if (ks==XK_Down  || ks==XK_KP_Down  || 
+	   ks==XK_KP_2  || ks==XK_F34)             i=CK_DOWN;
+
+  else if (ks==XK_Left  || ks==XK_KP_Left  || 
+	   ks==XK_KP_4  || ks==XK_F30)             i=CK_LEFT;
+
+  else if (ks==XK_Right || ks==XK_KP_Right || 
+	   ks==XK_KP_6  || ks==XK_F32)             i=CK_RIGHT;
+
+  else if (ks==XK_Prior || ks==XK_KP_Prior ||
+	   ks==XK_KP_9  || ks==XK_F29)             i=CK_PAGEUP;
+
+  else if (ks==XK_Next  || ks==XK_KP_Next  ||
+	   ks==XK_KP_3  || ks==XK_F35)             i=CK_PAGEDOWN;
+
+  else if (ks==XK_Home  || ks==XK_KP_Home  ||
+	   ks==XK_KP_7  || ks==XK_F27)             i=CK_HOME;
+
+  else if (ks==XK_End   || ks==XK_KP_End   ||
+	   ks==XK_KP_1  || ks==XK_F33)             i=CK_END;
+
+  else i = CK_NONE;
+
+  if (dotrans && shift) {
+    if      (i==CK_PAGEUP)   i=CK_HOME;
+    else if (i==CK_PAGEDOWN) i=CK_END;
+    else if (i==CK_UP)       i=CK_PAGEUP;
+    else if (i==CK_DOWN)     i=CK_PAGEDOWN;
+  }
+  
+  return i;
+}
+
+
 /***********************************/
 void FakeButtonPress(bp)
 BUTT *bp;
@@ -287,8 +319,9 @@ void FakeKeyPress(win, ksym)
 
 /***********************************/
 void GenExpose(win, x, y, w, h)
-     Window win;
-     int x,y,w,h;
+     Window       win;
+     int          x, y;
+     unsigned int w, h;
 {
   /* generates an expose event on 'win' of the specified rectangle.  Unlike
      XClearArea(), it doesn't clear the rectangular region */
@@ -306,6 +339,24 @@ void GenExpose(win, x, y, w, h)
 }    
 
 
+/***********************************/
+void RemapKeyCheck(ks, buf, stlen)
+     KeySym ks;
+     char   *buf;
+     int    *stlen;
+{
+  /* remap weirdo keysyms into normal key events */
+  if (ks == 0x1000ff00) {  /* map 'Remove' key on DEC keyboards -> ^D */
+    buf[0] = '\004';
+    buf[1] = '\0';
+    *stlen  = 1;
+  }
+}
+
+
+
+
+
 /***********************************/
 void xvDestroyImage(image)
      XImage *image;
@@ -325,55 +376,58 @@ void xvDestroyImage(image)
 
 /***********************************/
 void DimRect(win, x, y, w, h, bg)
-Window win;
-int x,y,w,h;
-u_long bg;
+     Window win;
+     int    x, y;
+     u_int  w, h;
+     u_long bg;
 {
   /* stipple a rectangular region by drawing 'bg' where there's 1's 
      in the stipple pattern */
 
-  XSetFillStyle(theDisp, theGC, FillStippled);
-  XSetStipple(theDisp, theGC, dimStip);
+  XSetFillStyle (theDisp, theGC, FillStippled);
+  XSetStipple   (theDisp, theGC, dimStip);
   XSetForeground(theDisp, theGC, bg);
-  XFillRectangle(theDisp,win,theGC,x,y,w,h);
-  XSetFillStyle(theDisp, theGC, FillSolid);
+  XFillRectangle(theDisp, win,   theGC, x, y, (u_int) w, (u_int) h);
+  XSetFillStyle (theDisp, theGC, FillSolid);
 }
 
 
 
 /**************************************************/
 void Draw3dRect(win, x,y,w,h, inout, bwidth, hi, lo, bg)
-     Window   win;
-     int      x,y,w,h,inout,bwidth;
+     Window        win;
+     int           x,y,inout,bwidth;
+     unsigned int  w,h;
      unsigned long hi, lo, bg;
 {
-  int i,r;
+  int i,r,x1,y1;
+
+  x1 = x + (int) w;
+  y1 = y + (int) h;
 
   if (ctrlColor) {
     /* draw top-left */
-    if (inout == R3D_OUT) XSetForeground(theDisp, theGC, hi);
-    else XSetForeground(theDisp, theGC, lo);
+    XSetForeground(theDisp, theGC, (inout==R3D_OUT) ? hi : lo);
 
     for (i=0; i<bwidth; i++) {
-      XDrawLine(theDisp, win, theGC, x+i, y+h-i, x+i, y+i);
-      XDrawLine(theDisp, win, theGC, x+i, y+i, x+w-i, y+i);
+      XDrawLine(theDisp, win, theGC, x+i, y1-i, x+i,  y+i);
+      XDrawLine(theDisp, win, theGC, x+i, y+i,  x1-i, y+i);
     }
     
     /* draw bot-right */
-    if (inout == R3D_OUT) XSetForeground(theDisp, theGC, lo);
-    else XSetForeground(theDisp, theGC, hi);
+    XSetForeground(theDisp, theGC, (inout==R3D_OUT) ? lo : hi);
 
     for (i=0; i<bwidth; i++) {
-      XDrawLine(theDisp, win, theGC, x+i+1, y+h-i, x+w-i, y+h-i);
-      XDrawLine(theDisp, win, theGC, x+w-i, y+h-i, x+w-i, y+i+1);
+      XDrawLine(theDisp, win, theGC, x+i+1, y1-i, x1-i, y1-i);
+      XDrawLine(theDisp, win, theGC, x1-i,  y1-i, x1-i, y+i+1);
     }
 
     /* draw diagonals */
     XSetForeground(theDisp, theGC, bg);
 
     for (i=0; i<bwidth; i++) {
-      XDrawPoint(theDisp, win, theGC, x+i, y+h-i);
-      XDrawPoint(theDisp, win, theGC, x+w-i, y+i);
+      XDrawPoint(theDisp, win, theGC, x+i,  y1-i);
+      XDrawPoint(theDisp, win, theGC, x1-i, y+i);
     }
   }
 }
@@ -381,47 +435,31 @@ void Draw3dRect(win, x,y,w,h, inout, bwidth, hi, lo, bg)
 
 
 /**************************************************/
-void SetCropString(active)
-int active;
+void SetCropString()
 {
   /* sets the crop string in the info box to be correct.  should
      be called whenever 'but[BCROP].active', cXOFF,cYOFF,cWIDE,cHIGH 
      are changed */
 
-  /* if 'active' there's a cropping rectangle drawn on the picture.
-     the string should reflect that.  */
+  if (cpic != pic) 
+    SetISTR(ISTR_CROP, "%dx%d rectangle starting at %d,%d",  
+	    cWIDE, cHIGH, cXOFF, cYOFF);
+  else SetISTR(ISTR_CROP, "<none>");
+}
 
-  if (active) {
-    /* figure out current cropping rectangle in 'pic' coordinates */
-    int x,y,x1,y1,w,h;
-    int cx,cy,dx,dy;
-    
-    if (crx1<crx2) cx=crx1; else cx=crx2;
-    if (cry1<cry2) cy=cry1; else cy=cry2;
-    dx = abs(crx1-crx2);  dy = abs(cry1-cry2);
 
-    x = cXOFF + (cx * cWIDE) / eWIDE;
-    y = cYOFF + (cy * cHIGH) / eHIGH;
-    x1 = cXOFF + ((cx+dx) * cWIDE) / eWIDE;
-    y1 = cYOFF + ((cy+dy) * cHIGH) / eHIGH;
-    w = (x1 - x) + 1;
-    h = (y1 - y) + 1;
+/**************************************************/
+void SetSelectionString()
+{
+  /* sets the Selection string in the info box to be correct.  should
+     be called whenever the selection may have changed */
 
-    if (w<1) w = 1;
-    if (x+w > pWIDE) w = pWIDE - x;
-    if (h<1) h = 1;
-    if (y+h > pHIGH) h = pHIGH - y;
-
-    SetISTR(ISTR_CROP, "%dx%d rectangle starting at %d,%d", w, h, x, y);
-  }
-
-  else {   /* cropping rectangle is turned off */
-    if (cpic != pic)
-      SetISTR(ISTR_CROP, "%dx%d rectangle starting at %d,%d", 
-	      cWIDE, cHIGH, cXOFF, cYOFF);
-    else
-      SetISTR(ISTR_CROP, "<none>");
+  if (HaveSelection()) {
+    int x,y,w,h;
+    GetSelRCoords(&x,&y,&w,&h);
+    SetISTR(ISTR_SELECT, "%dx%d rectangle starting at %d,%d", w, h, x, y);
   }
+  else SetISTR(ISTR_SELECT, "<none>");
 }
 
 
@@ -452,7 +490,7 @@ void FatalError (identifier)
 
 /***********************************/
 void Quit(i)
-int i;
+     int i;
 { 
   /* called when the program exits.  frees everything explictly created
      EXCEPT allocated colors.  This is used when 'useroot' is in operation,
@@ -497,6 +535,28 @@ int i;
 
   KillPageFiles(pageBaseName, numPages);
 
+
+  if (autoDelete) {  /* delete all files listed on command line */
+    int j;  char str[MAXPATHLEN+1];
+
+    for (j=0; j<orignumnames; j++) {
+      if (origlist[j][0] != '/') {  /* relative path, prepend 'initdir' */
+	sprintf(str,"%s/%s", initdir, origlist[j]);
+	if (unlink(str)) {
+	  fprintf(stderr,"%s: can't delete '%s' - %s\n", 
+		  cmd, str, ERRSTR(errno));
+	}
+      }
+      else {
+	if (unlink(origlist[j])) {
+	  fprintf(stderr,"%s: can't delete '%s' - %s\n", 
+		  cmd, origlist[j], ERRSTR(errno));
+	}
+      }
+    }
+  }
+
+  XSync(theDisp, False);
   exit(i);
 }
 
@@ -515,30 +575,31 @@ void LoadFishCursors()
 
   flcurs = fl1curs = fmcurs = fr1curs = frcurs = (Pixmap) NULL;
 
-  flpix = XCreatePixmapFromBitmapData(theDisp, ctrlW, fc_left_bits,
-	     fc_w, fc_h, 1, 0, 1);
-  flmpix= XCreatePixmapFromBitmapData(theDisp, ctrlW, fc_leftm_bits,
-	     fc_w, fc_h, 1, 0, 1);
+  flpix = XCreatePixmapFromBitmapData(theDisp, ctrlW, (char *) fc_left_bits,
+	     fc_w, fc_h, 1L, 0L, 1);
+  flmpix= XCreatePixmapFromBitmapData(theDisp, ctrlW, (char *) fc_leftm_bits,
+	     fc_w, fc_h, 1L, 0L, 1);
 
-  fl1pix = XCreatePixmapFromBitmapData(theDisp, ctrlW, fc_left1_bits,
-	     fc_w, fc_h, 1, 0, 1);
-  fl1mpix= XCreatePixmapFromBitmapData(theDisp, ctrlW, fc_left1m_bits,
-	     fc_w, fc_h, 1, 0, 1);
+  fl1pix = XCreatePixmapFromBitmapData(theDisp, ctrlW, (char *) fc_left1_bits,
+	     fc_w, fc_h, 1L, 0L, 1);
+  fl1mpix= XCreatePixmapFromBitmapData(theDisp, ctrlW, (char *) fc_left1m_bits,
+	     fc_w, fc_h, 1L, 0L, 1);
 
-  fmpix = XCreatePixmapFromBitmapData(theDisp, ctrlW, fc_mid_bits,
-	     fc_w, fc_h, 1, 0, 1);
-  fmmpix= XCreatePixmapFromBitmapData(theDisp, ctrlW, fc_midm_bits,
-	     fc_w, fc_h, 1, 0, 1);
+  fmpix = XCreatePixmapFromBitmapData(theDisp, ctrlW, (char *) fc_mid_bits,
+	     fc_w, fc_h, 1L, 0L, 1);
+  fmmpix= XCreatePixmapFromBitmapData(theDisp, ctrlW, (char *) fc_midm_bits,
+	     fc_w, fc_h, 1L, 0L, 1);
 
-  fr1pix = XCreatePixmapFromBitmapData(theDisp, ctrlW, fc_right1_bits,
-	     fc_w, fc_h, 1, 0, 1);
-  fr1mpix = XCreatePixmapFromBitmapData(theDisp, ctrlW, fc_right1m_bits,
-	     fc_w, fc_h, 1, 0, 1);
+  fr1pix = XCreatePixmapFromBitmapData(theDisp, ctrlW, (char *) fc_right1_bits,
+	     fc_w, fc_h, 1L, 0L, 1);
+  fr1mpix = XCreatePixmapFromBitmapData(theDisp, ctrlW, 
+					(char *) fc_right1m_bits,
+	     fc_w, fc_h, 1L, 0L, 1);
 
-  frpix = XCreatePixmapFromBitmapData(theDisp, ctrlW, fc_right_bits,
-	     fc_w, fc_h, 1, 0, 1);
-  frmpix = XCreatePixmapFromBitmapData(theDisp, ctrlW, fc_rightm_bits,
-	     fc_w, fc_h, 1, 0, 1);
+  frpix = XCreatePixmapFromBitmapData(theDisp, ctrlW, (char *) fc_right_bits,
+	     fc_w, fc_h, 1L, 0L, 1);
+  frmpix = XCreatePixmapFromBitmapData(theDisp, ctrlW, (char *) fc_rightm_bits,
+	     fc_w, fc_h, 1L, 0L, 1);
 
   if (!flpix || !flmpix || !fmpix || !fmmpix || !frpix || !frmpix
       || !fl1pix || !fl1mpix || !fr1pix || !fr1mpix) return;
@@ -556,12 +617,34 @@ void LoadFishCursors()
     { flcurs = fmcurs = frcurs = (Cursor) NULL; }
 }
 
-static int fishno=0;
+
+static int    fishno=0;
+static int    waiting=0;
+static time_t lastwaittime;
 
 
 /***********************************/
 void WaitCursor()
 {
+  XWMHints xwmh;
+  time_t   nowT;
+
+  if (!waiting) {
+    time(&lastwaittime);
+    waiting=1;
+    xwmh.input       = True;
+    xwmh.icon_pixmap = riconPix;  
+    xwmh.icon_mask   = riconmask;  
+    xwmh.flags = (InputHint | IconPixmapHint | IconMaskHint) ;
+    if (!useroot && mainW) XSetWMHints(theDisp, mainW, &xwmh);
+    if ( useroot && ctrlW) XSetWMHints(theDisp, ctrlW, &xwmh);
+  }
+  else {    /* waiting */
+    time(&nowT);
+    if (nowT == lastwaittime) return;
+    lastwaittime = nowT;
+  }
+
   SetCursors(fishno);
   fishno = (fishno+1) % 8;
 }
@@ -569,17 +652,28 @@ void WaitCursor()
 
 /***********************************/
 void SetCursors(n)
-int n;
+     int n;
 {
   Cursor c;
+  XWMHints xwmh;
 
   c = cross;
   /* if n < 0   sets normal cursor in all windows
      n = 0..6   cycles through fish cursors */
 
   if (n<0) {
+    if (waiting) {
+      waiting=0;
+      xwmh.input       = True;
+      xwmh.icon_pixmap = iconPix;  
+      xwmh.icon_mask   = iconmask;  
+      xwmh.flags = (InputHint | IconPixmapHint | IconMaskHint) ;
+      if (!useroot && mainW) XSetWMHints(theDisp, mainW, &xwmh);
+      if ( useroot && ctrlW) XSetWMHints(theDisp, ctrlW, &xwmh);
+    }
+
     if (showzoomcursor) set_cursors(zoom, arrow);
-                   else set_cursors(cross, arrow);
+                   else set_cursors(arrow, arrow);
     fishno = 0;
   }
 
@@ -613,6 +707,7 @@ static void set_cursors(mainc, otherc)
   if (psW)   XDefineCursor(theDisp, psW, otherc);
 
   SetBrowseCursor(otherc);
+  SetTextCursor(otherc);
   
 #ifdef HAVE_JPEG
   if (jpegW) XDefineCursor(theDisp, jpegW, otherc);
@@ -643,58 +738,95 @@ char *BaseName(fname)
 
   
 /***************************************************/
-void DrawTempGauge(win, x,y,w,h, ratio, fg,bg,hi,lo)
+void DrawTempGauge(win, x,y,w,h, ratio, fg,bg,hi,lo, str)
      Window win;
      int    x,y,w,h;
      double ratio;
      u_long fg,bg,hi,lo;
+     char   *str;
 {
-  /* draws a 'temprature'-style horizontal progess meter in the specified
+  /* draws a 'temprature'-style horizontal progress meter in the specified
      window, at the specified location */
 
-  int barwide, maxwide;
+  int barwide, maxwide, numchars, tw;
+
+  if (!win) return;
+
+  numchars = tw = 0;
+  if (str && str[0] != '\0') {
+    numchars = (int) strlen(str);
+    while ( (XTextWidth(mfinfo, str, numchars) > w) && numchars>0) numchars--;
+  }
+  if (numchars) tw = XTextWidth(mfinfo, str, numchars);
 
   XSetForeground(theDisp, theGC, fg);
-  XDrawRectangle(theDisp, win, theGC, x,y,w,h);
+  XDrawRectangle(theDisp, win, theGC, x,y, (u_int) w, (u_int) h);
 
   if (ctrlColor) {
-    Draw3dRect(win, x+1,y+1,w-2,h-2, R3D_IN, 2, hi, lo, bg);
-    maxwide = w-5;
+    Draw3dRect(win, x+1,y+1, (u_int) w-2, (u_int) h-2, R3D_IN, 2, hi, lo, bg);
+    maxwide = w-4;
     barwide = (int) (maxwide * ratio + 0.5);
 
     XSetForeground(theDisp, theGC, fg);
-    XFillRectangle(theDisp, win, theGC, x+3, y+3, barwide, h-5);
+    XFillRectangle(theDisp,win,theGC, x+3, y+3, (u_int) barwide, (u_int) h-5);
 
-    XDrawLine(theDisp, win, theGC, x+3+barwide, y+h/2 + 0, x+w-3, y+h/2 + 0);
+    if (barwide < maxwide) {
+      if (numchars) {
+	XSetForeground(theDisp, theGC, bg);
+	XFillRectangle(theDisp, win, theGC, x+3+barwide, y+3, 
+		       (u_int) (maxwide-barwide), (u_int) (h-5));
 
-    XSetForeground(theDisp, theGC, lo);
-    XDrawLine(theDisp, win, theGC, x+3+barwide, y+h/2 + 1, x+w-3, y+h/2 + 1);
-
-    XSetForeground(theDisp, theGC, hi);
-    XDrawLine(theDisp, win, theGC, x+3+barwide, y+h/2 + 2, x+w-3, y+h/2 + 2);
-
-    XSetForeground(theDisp, theGC, bg);
-    XFillRectangle(theDisp, win, theGC, x+3+barwide, y+3, 
-		   (maxwide-barwide), (h/2 - 3));
-
-    XFillRectangle(theDisp, win, theGC, x+3+barwide, y+h/2 + 3, 
-		   (maxwide-barwide), ((h-3) - (h/2+3)) + 1);
+	XSetForeground(theDisp, theGC, fg);
+	XDrawString(theDisp, win, theGC, CENTERX(mfinfo, (x+w/2), str), 
+		    CENTERY(mfinfo, (y+h/2)), str, numchars);
+      }
+      else {
+	XDrawLine(theDisp,win,theGC,x+3+barwide, y+h/2 + 0, x+w-3, y+h/2 + 0);
+      
+	XSetForeground(theDisp, theGC, lo);
+	XDrawLine(theDisp,win,theGC,x+3+barwide, y+h/2 + 1, x+w-3, y+h/2 + 1);
+	
+	XSetForeground(theDisp, theGC, hi);
+	XDrawLine(theDisp,win,theGC,x+3+barwide, y+h/2 + 2, x+w-3, y+h/2 + 2);
+	
+	XSetForeground(theDisp, theGC, bg);
+	XFillRectangle(theDisp, win, theGC, x+3+barwide, y+3, 
+		       (u_int) (maxwide-barwide), (u_int) (h/2 - 3));
+	
+	XFillRectangle(theDisp, win, theGC, x+3+barwide, y+h/2 + 3, 
+		       (u_int) (maxwide-barwide),(u_int)((h-3) - (h/2+3)) + 1);
+      }
+    }
   }
+
   else {
     maxwide = w-1;
     barwide = (int) (maxwide * ratio + 0.5);
 
     XSetForeground(theDisp, theGC, fg);
-    XFillRectangle(theDisp, win, theGC, x+1, y+1, barwide, h-1);
+    XFillRectangle(theDisp,win,theGC, x+1, y+1, (u_int) barwide, (u_int) h-1);
 
-    XDrawLine(theDisp, win, theGC, x+1+barwide, y+h/2, x+w-1, y+h/2);
+    if (barwide < maxwide) {
+      if (numchars) {
+	XSetForeground(theDisp, theGC, bg);
+	XFillRectangle(theDisp, win, theGC, x+1+barwide, y+1, 
+		       (u_int) (maxwide-barwide), (u_int) (h-1));
 
-    XSetForeground(theDisp, theGC, bg);
-    XFillRectangle(theDisp, win, theGC, x+1+barwide, y+1, 
-		   maxwide-barwide, h/2 - 1);
-
-    XFillRectangle(theDisp, win, theGC, x+1+barwide, y+h/2 + 1, 
-		   maxwide-barwide, ((h-1) - (h/2+1)) + 1);
+	XSetForeground(theDisp, theGC, fg);
+	XDrawString(theDisp, win, theGC, CENTERX(mfinfo, (x+w/2), str), 
+		    CENTERY(mfinfo, (y+h/2)), str, numchars);
+      }
+      else {
+	XDrawLine(theDisp, win, theGC, x+1+barwide, y+h/2, x+w-1, y+h/2);
+	
+	XSetForeground(theDisp, theGC, bg);
+	XFillRectangle(theDisp, win, theGC, x+1+barwide, y+1, 
+		       (u_int) (maxwide-barwide), (u_int) (h/2 - 1));
+	
+	XFillRectangle(theDisp, win, theGC, x+1+barwide, y+h/2 + 1, 
+		       (u_int)(maxwide-barwide),(u_int)(((h-1) - (h/2+1))+1));
+      }
+    }
   }
 
   XFlush(theDisp);
@@ -702,6 +834,59 @@ void DrawTempGauge(win, x,y,w,h, ratio, fg,bg,hi,lo)
     
 
 
+/***************************************************/
+void ProgressMeter(min, max, val, str)
+     int min, max, val;
+     char *str;
+{
+  /* called during 'long' operations (algorithms, smoothing, etc.) to
+     give some indication that the program will ever finish.  Draws a 
+     temperature gauge in either mainW (if not useRoot), or ctrlW.
+     Tries to be clever:  only draws gauge if it looks like the operation is
+     going to take more than a few seconds.  Calling with val == max removes
+     the gauge. */
+
+  static int    waiting  = 0;
+  static time_t lastTime = 0;
+  time_t        nowTime;
+  double        doneness;
+  Window        win;
+  int           xpos,ypos;
+
+  if (useroot) { win=ctrlW;  xpos=10;  ypos=3; }
+          else { win=mainW;  xpos=5;   ypos=5; }
+  if (!win) return;
+
+  if (min >= max) return;
+  if (val==max && lastTime==0) return;  /* already erased meter */
+
+  doneness = ((double) (val-min)) / ((double) (max - min));
+
+  time(&nowTime);
+
+  if (lastTime == 0) {   /* first time we're calling for this meter */
+    lastTime = nowTime;
+    waiting = 1;
+  }
+  else if (waiting) {    /* haven't drawn meter yet... */
+    if ((nowTime - lastTime >= 2) && doneness < .5) waiting = 0;
+  }
+
+  if (!waiting) {     /* not waiting (or not waiting any longer) */
+    if (nowTime == lastTime && val<max) return;  /* max one draw per second */
+    lastTime = nowTime;
+    DrawTempGauge(win, xpos, ypos, 100,19, doneness, 
+		    infofg,infobg,hicol,locol,str);
+
+    if (val >= max) {            /* remove temp gauge */
+      XClearArea(theDisp, win, xpos,ypos, 100+1,19+1, True);
+      lastTime = 0;
+    }
+  }
+}
+
+
+
 /***************************************************/
 void XVDeletedFile(fullname)
      char *fullname;
@@ -729,7 +914,7 @@ void XVCreatedFile(fullname)
 /***************************************************/
 void xvbcopy(src, dst, len)
      char *src, *dst;
-     int   len;
+     size_t  len;
 {
   /* Modern OS's (Solaris, etc.) frown upon the use of bcopy(),
    * and only want you to use memcpy().  However, memcpy() is broken,
@@ -764,8 +949,8 @@ void xvbcopy(src, dst, len)
 
 /***************************************************/
 int xvbcmp (s1, s2, len)
-     char *s1, *s2;
-     int   len;
+     char   *s1, *s2;
+     size_t  len;
 {
   for ( ; len>0; len--, s1++, s2++) {
     if      (*s1 < *s2) return -1;
@@ -776,24 +961,101 @@ int xvbcmp (s1, s2, len)
 
 /***************************************************/
 void xvbzero(s, len)
-     char *s;
-     int   len;
+     char   *s;
+     size_t  len;
 {
   for ( ; len>0; len--) *s++ = 0;
 }
 
+
+/***************************************************/
+void xv_getwd(buf, buflen)
+     char   *buf;
+     size_t  buflen;
+{
+  /* gets the current working directory.  No trailing '/' */
+
+  char *rv; 
+
+#ifdef USE_GETCWD
+  rv = (char *) getcwd(buf, buflen);
+#else
+  rv = (char *) getwd(buf);
+#endif
+
+  if (!rv || strlen(rv)==0) {
+    if (((rv=(char *) getenv("PWD"))==NULL) && 
+	((rv=(char *) getenv("cwd"))==NULL)) rv = "./";
+    strcpy(buf, rv);
+  }
+}
+
+
+
 /***************************************************/
 
-static int timerdone;
+/* 
+ *	Source code for the "strstr" library routine.
+ *
+ * Copyright 1988 Regents of the University of California
+ * Permission to use, copy, modify, and distribute this
+ * software and its documentation for any purpose and without
+ * fee is hereby granted, provided that the above copyright
+ * notice appear in all copies.  The University of California
+ * makes no representations about the suitability of this
+ * software for any purpose.  It is provided "as is" without
+ * express or implied warranty.
+ */
 
-/*******/
-static void onalarm()
-/*******/
+char *xv_strstr(string, substring)
+     char *string;	        /* String to search. */
+     char *substring;		/* Substring to try to find in string. */
 {
-  timerdone=1;
+  register char *a, *b;
+
+  /* First scan quickly through the two strings looking for a
+   * single-character match.  When it's found, then compare the
+   * rest of the substring.
+   */
+
+  b = substring;
+  if (*b == 0) return string;
+
+  for ( ; *string != 0; string += 1) {
+    if (*string != *b) continue;
+
+    a = string;
+    while (1) {
+      if (*b == 0) return string;
+      if (*a++ != *b++) break;
+    }
+    b = substring;
+  }
+  return (char *) 0;
 }
 
 
+
+/***************************************************/
+
+/***************************************************/
+FILE *xv_fopen(fname, mode)
+     char *fname, *mode;
+{
+  FILE *fp;
+
+#ifndef VMS
+  fp = fopen(fname, mode);
+#else
+  fp = fopen(fname, mode, "ctx=stm");
+#endif
+
+  return fp;
+}
+
+
+
+
 /*******/
 void Timer(msec)   /* waits for 'n' milliseconds */
  int  msec;
@@ -827,9 +1089,9 @@ void Timer(msec)   /* waits for 'n' milliseconds */
 #endif
 
 
-#ifdef sco
+#if defined(SVR4) || defined(sco)
   {
-    int dummy;
+    struct pollfd dummy;
     poll(&dummy, 0, msec);
     return;
   }
@@ -846,36 +1108,17 @@ void Timer(msec)   /* waits for 'n' milliseconds */
 #endif
 
 
-#ifndef VMS          /* VMS hates multi-line #ifdef's */
-#  if !defined(sgi) && !defined(sco) && !defined(USLEEP) && !defined(NOTIMER)
+#ifndef VMS
   {
-    /* we aren't doing it any of the neat/clean/simple ways.  Do it the
-       old clunky way, using setitimer()... */
+    /* default Timer() method now uses 'select()', which probably works
+	on all systems *anyhow* (except for VMS...) */
     
-    struct itimerval it;
+    struct timeval time;
 
-    xvbzero((char *) &it, sizeof(it));
-    if (usec>=1000000L) {  /* more than 1 second */
-      it.it_value.tv_sec = usec / 1000000L;
-      usec %= 1000000L;
-    }
-
-    it.it_value.tv_usec = usec;
-    timerdone=0;
-    signal(SIGALRM,onalarm);
-    setitimer(ITIMER_REAL, &it, (struct itimerval *)0);
-    while (1) {
-      HOLD_SIG;                    /* note:  have to block, so that ALRM */
-      if (timerdone) break;        /* doesn't occur between 'if (timerdone)' */
-      else PAUSE_SIG;              /* and calling PAUSE_SIG */
-    }
-
-    RELEASE_SIG;                   /* turn ALRM blocking off */
-    signal(SIGALRM,SIG_DFL);
+    time.tv_sec = usec / 1000000L;
+    time.tv_usec = usec % 1000000L;
+    select(0, (fd_set *) NULL, (fd_set *) NULL, (fd_set *) NULL, &time);
   }
-#  endif  /* !(sgi || sco || USLEEP || NOTIMER) */
-#endif  /* !VMS */
-
+#endif /* VMS */
 }
 
-
