@@ -5,71 +5,22 @@
  * WritePBM(fp,pic,ptype,w,h,r,g,b,numcols,style,raw,cmt,comment)
  */
 
-/* Copyright Notice
- * ================
- * Copyright 1989, 1990, 1991, 1992, 1993 by John Bradley
- * 
- * Permission to use, copy, and distribute XV in its entirety, for 
- * non-commercial purposes, is hereby granted without fee, provided that
- * this license information and copyright notice appear in all copies.
- * 
- * Note that distributing XV 'bundled' in with ANY product is considered
- * to be a 'commercial purpose'.
- *
- * Also note that any copies of XV that are distributed MUST be built
- * and/or configured to be in their 'unregistered copy' mode, so that it
- * is made obvious to the user that XV is shareware, and that they should
- * consider donating, or at least reading this License Info.
- * 
- * The software may be modified for your own purposes, but modified
- * versions may NOT be distributed without prior consent of the author.
- * 
- * This software is provided 'as-is', without any express or implied
- * warranty.  In no event will the author be held liable for any damages
- * arising from the use of this software.
- * 
- * If you would like to do something with XV that this copyright
- * prohibits (such as distributing it with a commercial product, 
- * using portions of the source in some other program, etc.), please
- * contact the author (preferably via email).  Arrangements can
- * probably be worked out.
- *
- * XV is shareware for PERSONAL USE only.  You may use XV for your own
- * amusement, and if you find it nifty, useful, generally cool, or of
- * some value to you, your non-deductable donation would be greatly
- * appreciated.  $25 is the suggested donation, though, of course,
- * larger donations are quite welcome.  Folks who donate $25 or more
- * can receive a Real Nice bound copy of the XV manual for no extra
- * charge.
- * 
- * Commercial, government, and institutional users MUST register their
- * copies of XV, for the exceedingly REASONABLE price of just $25 per
- * workstation/X terminal.  Site licenses are available for those who
- * wish to run XV on a large number of machines.  Contact the author
- * for more details.
- *
- * The author may be contacted via:
- *    US Mail:  John Bradley
- *              1053 Floyd Terrace
- *              Bryn Mawr, PA  19010
- *
- *    Phone:    (215) 898-8813
- *    EMail:    bradley@cis.upenn.edu
- */
-
+#include "copyright.h"
 
 #include "xv.h"
 
 
 
-/* comments on error handling:
-   a truncated file is not considered a Major Error.  The file is loaded, the
-   rest of the pic is filled with 0's.
-
-   a file with garbage characters in it is an unloadable file.  All allocated
-   stuff is tossed, and LoadPBM returns non-zero
-
-   not being able to malloc is a Fatal Error.  The program is aborted. */
+/*
+ * comments on error handling:
+ * a truncated file is not considered a Major Error.  The file is loaded, the
+ * rest of the pic is filled with 0's.
+ *
+ * a file with garbage characters in it is an unloadable file.  All allocated
+ * stuff is tossed, and LoadPBM returns non-zero
+ *
+ * not being able to malloc is a Fatal Error.  The program is aborted.
+ */
 
 
 #define TRUNCSTR "File appears to be truncated."
@@ -77,18 +28,13 @@
 static int garbage;
 static long numgot, filesize;
 
-#ifdef __STDC__
-static int loadpbm(FILE *, PICINFO *, int);
-static int loadpgm(FILE *, PICINFO *, int, int);
-static int loadppm(FILE *, PICINFO *, int, int);
-static int getint (FILE *, PICINFO *);
-static int getbit (FILE *, PICINFO *);
-static int pbmError(char *, char *);
-#else
-static int loadpbm(), loadpgm(), loadppm();
-static int getint(),  getbit();
-static int pbmError();
-#endif
+static int loadpbm  PARM((FILE *, PICINFO *, int));
+static int loadpgm  PARM((FILE *, PICINFO *, int, int));
+static int loadppm  PARM((FILE *, PICINFO *, int, int));
+static int getint   PARM((FILE *, PICINFO *));
+static int getbit   PARM((FILE *, PICINFO *));
+static int getshort PARM((FILE *));
+static int pbmError PARM((char *, char *));
 
 static char *bname;
 
@@ -104,7 +50,7 @@ int LoadPBM(fname, pinfo)
   int    c, c1;
   int    maxv, rv;
 
-  garbage = maxv = 0;
+  garbage = maxv = rv = 0;
   bname = BaseName(fname);
 
   pinfo->pic     = (byte *) NULL;
@@ -112,7 +58,7 @@ int LoadPBM(fname, pinfo)
 
 
   /* open the file */
-  fp=fopen(fname,"r");
+  fp = xv_fopen(fname,"r");
   if (!fp) return (pbmError(bname, "can't open file"));
 
   /* compute file length */
@@ -131,6 +77,7 @@ int LoadPBM(fname, pinfo)
 
   /* read in header information */
   pinfo->w = getint(fp, pinfo);  pinfo->h = getint(fp, pinfo);
+  pinfo->normw = pinfo->w;   pinfo->normh = pinfo->h;
 
   /* if we're not reading a bitmap, read the 'max value' */
   if ( !(c1=='1' || c1=='4')) {
@@ -161,7 +108,7 @@ int LoadPBM(fname, pinfo)
   fclose(fp);
 
   if (!rv) {
-    if (pinfo->pic) free(pinfo->comment);
+    if (pinfo->pic) free(pinfo->pic);
     if (pinfo->comment) free(pinfo->comment);
     pinfo->pic     = (byte *) NULL;
     pinfo->comment = (char *) NULL;
@@ -183,7 +130,7 @@ static int loadpbm(fp, pinfo, raw)
   int   i,j,bit,w,h;
 
   w = pinfo->w;  h = pinfo->h;
-  pic8 = (byte *) calloc(w * h, 1);
+  pic8 = (byte *) calloc((size_t) w * h, (size_t) 1);
   if (!pic8) return pbmError(bname, "couldn't malloc 'pic8'");
 
   pinfo->pic  = pic8;
@@ -245,11 +192,11 @@ static int loadpgm(fp, pinfo, raw, maxv)
      int      raw, maxv;
 {
   byte *pix, *pic8;
-  int   i,j,bitshift,w,h;
+  int   i,j,bitshift,w,h, holdmaxv;
 
 
   w = pinfo->w;  h = pinfo->h;
-  pic8 = (byte *) calloc(w*h,1);
+  pic8 = (byte *) calloc((size_t) w*h, (size_t) 1);
   if (!pic8) return(pbmError(bname, "couldn't malloc 'pic8'"));
 
 
@@ -262,6 +209,7 @@ static int loadpgm(fp, pinfo, raw, maxv)
 
 
   /* if maxv>255, keep dropping bits until it's reasonable */
+  holdmaxv = maxv;
   bitshift = 0;
   while (maxv>255) { maxv = maxv>>1;  bitshift++; }
 
@@ -270,16 +218,27 @@ static int loadpgm(fp, pinfo, raw, maxv)
     pinfo->r[i] = pinfo->g[i] = pinfo->b[i] = (i*255)/maxv;
 
 
+  numgot = 0;
+
   if (!raw) {
-    numgot = 0;
     for (i=0, pix=pic8; i<h; i++) {
       if ((i&0x3f)==0) WaitCursor();
       for (j=0; j<w; j++, pix++)
-	*pix = (getint(fp, pinfo) >> bitshift);
+	*pix = (byte) (getint(fp, pinfo) >> bitshift);
+    }
+  }
+  else { /* raw */
+    if (holdmaxv>255) {
+      for (i=0, pix=pic8; i<h; i++) {
+	if ((i&0x3f)==0) WaitCursor();
+	for (j=0; j<w; j++, pix++)
+	  *pix = (byte) (getshort(fp) >> bitshift);
+      }
+    }
+    else {
+      numgot = fread(pic8, (size_t) 1, (size_t) w*h, fp);  /* read raw data */
     }
   }
-
-  else numgot = fread(pic8, 1, w*h, fp);   /* read raw data */
 
   if (numgot != w*h) pbmError(bname, TRUNCSTR);   /* warning only */
 
@@ -298,12 +257,12 @@ static int loadppm(fp, pinfo, raw, maxv)
      int      raw, maxv;
 {
   byte *pix, *pic24, scale[256], *pic8;
-  int   i,j,bitshift, w, h;
+  int   i,j,bitshift, w, h, holdmaxv;
 
   w = pinfo->w;  h = pinfo->h;
 
   /* allocate 24-bit image */
-  pic24 = (byte *) calloc(w*h*3,1);
+  pic24 = (byte *) calloc((size_t) w*h*3, (size_t) 1);
   if (!pic24) FatalError("couldn't malloc 'pic24'");
 
   pinfo->pic  = pic24;
@@ -315,22 +274,33 @@ static int loadppm(fp, pinfo, raw, maxv)
 
 
   /* if maxv>255, keep dropping bits until it's reasonable */
+  holdmaxv = maxv;
   bitshift = 0;
   while (maxv>255) { maxv = maxv>>1;  bitshift++; }
 
 
+  numgot = 0;
+
   if (!raw) {
-    numgot = 0;
     for (i=0, pix=pic24; i<h; i++) {
       if ((i&0x3f)==0) WaitCursor();
-
       for (j=0; j<w*3; j++, pix++)
-	*pix = (getint(fp, pinfo) >> bitshift);
+	*pix = (byte) (getint(fp, pinfo) >> bitshift);
     }
   }
-
-  else numgot = fread(pic24, 1, w*h*3, fp);    /* read raw data */
-
+  else { /* raw */
+    if (holdmaxv>255) {
+      for (i=0, pix=pic24; i<h; i++) {
+	if ((i&0x3f)==0) WaitCursor();
+	for (j=0; j<w*3; j++,pix++)
+	  *pix = (byte) (getshort(fp) >> bitshift);
+      }
+    }
+    else {
+      numgot = fread(pic24, (size_t) 1, (size_t) w*h*3, fp);  /* read data */
+    }
+  }
+  
   if (numgot != w*h*3) pbmError(bname, TRUNCSTR);
 
   if (garbage)
@@ -424,6 +394,26 @@ static int getint(fp, pinfo)
 
 
 
+/*******************************************/
+static int getshort(fp)
+     FILE    *fp;
+{
+  /* used in RAW mode to read 16-bit values */
+
+  int c1, c2;
+
+  c1 = getc(fp);
+  if (c1 == EOF) return 0;
+  c2 = getc(fp);
+  if (c2 == EOF) return 0;
+
+  numgot++;
+
+  return (c2 << 8) | c1;
+}
+
+
+
 /*******************************************/
 static int getbit(fp, pinfo)
      FILE *fp;
