@@ -1,39 +1,53 @@
-#ifdef HAVE_PDS
 /*
  * xvpds.c - load routine for astronomical PDS/VICAR format pictures
- * version of 3-13-92
+ * version of 6-4-93
  *
  * Anthony A. Datri
- * Convex Computer Corp
- * 3000 Waterview Parkway
- * Ricardson, TX 75080
- * datri@convex.com
+ * Siemens Corporate Research
+ * 755 College Road East
+ * Princeton NJ 08540
+ * aad@scr.siemens.com
  *
+ * Alexandra Aarhus-Dusenberg
+ * alexd@world.std.com
+ *
+ * 5-3-93    added leading spaces back into sscanf's -- some huffmanized
+ 	       files have them.
+ * 5-2-93    slapped in Peter G. Ford <pgf@space.mit.edu>'s patches for
+ 	     xv 3.00:
+   Adds code to Makefile to compile under SunOS 4.0.3 (as if anybody cares).
+   Improves PDS/VICAR file recognition.
+   Writes current directory name in Doggie's window title.
+   Handles (most) PDS and VICAR images correctly.
+   Choice of algorithm for 16->8 bit conversion--linear or histogram stretch.
+    (adds CONV24_HIST item in "24/8 bit" pull-down menu.)
+   Uses any "palette.tab" file in cwd to color PDS/VICAR image.
+ 
  * 9-2-91    began integration.	 Much of this code is lifted from vicar.c,
- *	     which I wrote for xloadimage.  This is a little simpler, though.
- *
+ 	     which I wrote for xloadimage.  This is a little simpler, though.
+ 
  * 10-17-91  pdsuncomp is called with system(), which typically feeds the
- *	     commandline to sh.  Make sure that your .profile adds wherever
- *	     you have pdsuncomp to the PATH, like
- *
- *		PATH=$PATH:/usr/local/bin
- *
+ 	     commandline to sh.  Make sure that your .profile adds wherever
+ 	     you have pdsuncomp to the PATH, like
+ 
+ 		PATH=$PATH:/usr/local/bin
+ 
  * 11-15-91  substituted vdcomp from Viking CD's for pdsuncomp. I added
- *           recognition of - and shut off various messages
- *
+             recognition of - and shut off various messages
+ 
  * 1-5-92    merged into xv rel 2
- *
+ 
  * 3-11-92   cleaned up some comments
- *
+ 
  * 3-24-92   Got some new CD's from NASA of mosics and other processed Viking
- *           stuff.  There are actually records terminated with CRNLCR in
- *           these images, as well as ones that identify the spacecraft name
- *           as {VIKING_ORBITER_1, VIKING_ORBITER_2}.  I hacked up the code
- *           yet further to deal with these.  There's a Sun 4 XView binary for
- *           an image display program on these discs, but it's nowhere near as
- *           neat as the good Mr. Bradley's XV.
- *
- *
+             stuff.  There are actually records terminated with CRNLCR in
+             these images, as well as ones that identify the spacecraft name
+             as {VIKING_ORBITER_1, VIKING_ORBITER_2}.  I hacked up the code
+             yet further to deal with these.  There's a Sun 4 XView binary for
+             an image display program on these discs, but it's nowhere near as
+             neat as the good Mr. Bradley's XV.
+ 
+ 
  * Sources of these CD's:
  *
  *  National Space Science Data Center
@@ -63,7 +77,8 @@
  * Huffman-encoded, and the encoding histogram follows the ASCII headers.
  * To decode these, we use a slightly modified version of "vdcomp.c" from the
  * NASA Viking CD-ROMS.  For xv to work, you need to have vdcomp compiled
- * and in your search path.  vdcomp.c should be included with this distribution.
+ * and in your search path.  vdcomp.c should be included with this
+distribution.
  *
  * I've heard that newer discs have FITS images on them.  If they do, support
  * for them will be added when I get one.  Until then, you can use fitstopgm.
@@ -90,6 +105,8 @@
 
 #include "xv.h"
 
+#ifdef HAVE_PDS
+
 
 #define PDSFIXED        (1)
 #define PDSVARIABLE     (2)
@@ -114,86 +131,34 @@
 #define COMMENTSIZE	50
 
 
-static char	scanbuff      [MAX_SIZE], 
-                rtbuff        [RTBUFFSIZE], 
+static int	lastwasinote = FALSE;
+static char	scanbuff         [MAX_SIZE], 
+                rtbuff         [RTBUFFSIZE], 
+		inote	   [20*COMMENTSIZE],
                 infobuff      [COMMENTSIZE],
 		spacecraft    [COMMENTSIZE],
 		target        [COMMENTSIZE],
 		filtname      [COMMENTSIZE],
+		gainmode      [COMMENTSIZE],
+		editmode      [COMMENTSIZE],
+		scanmode      [COMMENTSIZE],
+		exposure      [COMMENTSIZE],
+		shuttermode   [COMMENTSIZE],
 		mphase        [COMMENTSIZE],
 		iname         [COMMENTSIZE],
 		itime         [COMMENTSIZE],
                 garbage       [1020],
+		*tmptmp,
 		pdsuncompfname[FNAMESIZE];
 byte *image;
+static int elaphe;
 
 
-/* 
- * pds_strstr.c --
- *
- *	Source code for the "strstr" library routine.
- *
- * Copyright 1988 Regents of the University of California
- * Permission to use, copy, modify, and distribute this
- * software and its documentation for any purpose and without
- * fee is hereby granted, provided that the above copyright
- * notice appear in all copies.  The University of California
- * makes no representations about the suitability of this
- * software for any purpose.  It is provided "as is" without
- * express or implied warranty.
- */
-
-/*
- *----------------------------------------------------------------------
- *
- * pds_strstr --
- *
- *	Locate the first instance of a substring in a string.
- *
- * Results:
- *	If string contains substring, the return value is the
- *	location of the first matching instance of substring
- *	in string.  If string doesn't contain substring, the
- *	return value is 0.  Matching is done on an exact
- *	character-for-character basis with no wildcards or special
- *	characters.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-char *pds_strstr(string, substring)
-     char *string;	/* String to search. */
-     char *substring;		/* Substring to try to find in string. */
-{
-  register char *a, *b;
-
-  /* First scan quickly through the two strings looking for a
-   * single-character match.  When it's found, then compare the
-   * rest of the substring.
-   */
-
-  b = substring;
-  if (*b == 0) return string;
-
-  for ( ; *string != 0; string += 1) {
-    if (*string != *b) continue;
-
-    a = string;
-    while (1) {
-      if (*b == 0) return string;
-
-      if (*a++ != *b++) break;
-
-    }
-    b = substring;
-  }
-  return (char *) 0;
-}
-
 
+/* local function declarations */
+static int getpdsrec          PARM((FILE *, char *));
+static int Convert16BitImage  PARM((char *, PICINFO *, int));
+static int LoadPDSPalette     PARM((char *, PICINFO *));
 
 
 
@@ -201,7 +166,7 @@ char *pds_strstr(string, substring)
 /* The method of termination is unpredictable.  A pox on VMS.   */
 /* Sometimes we even get mixtures of CR's and LF's.  Sigh.      */
 
-int getpdsrec(f,buff)
+static int getpdsrec(f,buff)
      FILE *f;
      char *buff;
 {
@@ -285,18 +250,17 @@ int LoadPDS(fname, pinfo)
 {
   /* returns '1' on success, '0' on failure */
 
-  byte *pic8;
   int tempnum;
   FILE	*zf;
-  static int isfixed,teco,i,j,itype,
+  static int isfixed,teco,i,j,itype,vaxbyte,
              recsize,hrecsize,irecsize,isimage,labelrecs,labelsofar,
              x,y,lpsize,lssize,samplesize,returnp,labelsize,yy;
   char	*tmp;
   char  *ftypstr;
   unsigned long filesize;
+  char  sampletype[64];
 
   pinfo->type = PIC8;
-  pic8 = (byte *) NULL;
   isfixed = TRUE;
   returnp = isimage = FALSE;
   itype   = PDSTRASH;
@@ -306,12 +270,14 @@ int LoadPDS(fname, pinfo)
 
   (*pdsuncompfname) = (*iname) = (*target) = (*filtname) = (*garbage) = '\0';
   (*itime) = (*spacecraft) = (*scanbuff) = (*mphase) = (*rtbuff) = '\0';
+  (*sampletype) = (*inote) = (*infobuff) = (*gainmode) = (*editmode) = '\0';
+  (*scanmode) = (*exposure) = (*shuttermode) = '\0';
 
   /* there may be some wisdom in statically mallocing space for an 800x800
      image, since that's what the Voyager cd's have, or even a 2kx2k, since
      some of the Magellan images from Ames are that size */
 
-  zf=fopen(fname,"r");
+  zf = xv_fopen(fname,"r");
   if (!zf) {
     SetISTR(ISTR_WARNING,"LoadPDS: can't open %s\n",fname);
     return 0;
@@ -327,7 +293,7 @@ int LoadPDS(fname, pinfo)
      bloody VAX hoops to find out how long the record is.  Sigh.  Have
      people never heard of newlines? */
 
-  if ((teco=fread(rtbuff,13,1,zf)) != 1) {
+  if ((teco=fread(rtbuff,(size_t) 13, (size_t) 1,zf)) != 1) {
     SetISTR(ISTR_WARNING,"LoadPDS: couldn't read magic #: %d,%s",teco,fname);
     fclose(zf);
     return 0;
@@ -336,13 +302,13 @@ int LoadPDS(fname, pinfo)
   /* this makes sscanf so much happier */
   rtbuff[RTBUFFSIZE-1] = '\0';
 
-  if (strncmp(rtbuff, "NJPL1I00PDS",11) == 0) {
+  if (strncmp(rtbuff, "NJPL1I00PDS", (size_t) 11) == 0) {
     itype=PDSFIXED;
-  } else if (strncmp(rtbuff+2,"NJPL1I00PDS",11) == 0) {
+  } else if (strncmp(rtbuff+2,"NJPL1I00PDS", (size_t) 11) == 0) {
     itype=PDSVARIABLE;
-  } else if (strncmp(rtbuff,"CCSD3Z",6) == 0) {
+  } else if (strncmp(rtbuff,"CCSD3Z", (size_t) 6) == 0) {
     itype=VIKINGFIXED; /* unique variant of PDS on Viking discs (browse) */
-  } else if (strncmp(rtbuff+2,"CCSD3Z",6) == 0) {
+  } else if (strncmp(rtbuff+2,"CCSD3Z", (size_t) 6) == 0) {
     itype=VIKINGVARIABLE;
   } else if (sscanf(rtbuff,"LBLSIZE = %d%n",&labelsize,&labelsofar) == 1) {
     itype=VICAR;
@@ -372,18 +338,12 @@ int LoadPDS(fname, pinfo)
 
     if (isfixed == FALSE) {
       teco = ((*rtbuff) + ((*(rtbuff+1)) << 8));
-      fread(rtbuff,(teco % 2 ? teco + 1 : teco ) - 11,1,zf);
+      fread(rtbuff, (size_t) ((teco%2 ? teco+1 : teco) - 11),(size_t) 1, zf);
     }
 
-    /* we know that we've got a PDS file of some sort.	We have to
+    /* We know that we've got a PDS file of some sort.  We have to
        search through the labels looking for things because usage doesn't seem
-       to be consistent, or even predictable.  When I added this format to
-       xloadimage, I had the code spew out various peices of information, like
-       the name of the target, filter used, etc.  XV, however, doesn't really
-       provide a mechanism for this kind of thing, and I can't really stick it
-       into the info box without overhauling it.  Maybe later I'll add a
-       -verbose switch which this module will interpret as license to scribble
-       on stdout or stderr */
+       to be consistent, or even predictable. */
 
     for (;;) {
       if (isfixed) {
@@ -394,9 +354,8 @@ int LoadPDS(fname, pinfo)
       } else {
 	/* AAAARGGGGH!  We've got a bloody variable-length file where
 	 * the headers are preceded by a byte count, as a VAX 16-bit
-	 * integer of all things.  These guys seem to have had VMS on the
-	 * brain.  ASCII?  Wot's that?
-	 *
+	 * integer of all things.
+  	 *
 	 * "A compressed image file is composed of variable-length records
 	 * defined according to the ISO standard.  Each variable length
 	 * record starts with a record length indicator, stored as a 16-bit
@@ -416,7 +375,7 @@ int LoadPDS(fname, pinfo)
 	teco = i + (j << 8);
 	if (teco % 2) teco++;
 
-	if (fread(scanbuff,teco,1,zf) != 1) {
+	if (fread(scanbuff, (size_t) teco, (size_t) 1, zf) != 1) {
 	  SetISTR(ISTR_WARNING,"LoadPDS: bad fread reading labels\n");
 	  fclose(zf);
 	  return 0;
@@ -432,15 +391,16 @@ int LoadPDS(fname, pinfo)
 	 fool user wants them.  There seems to be disagreement about what
 	 the headers are called, since we might find, for example, either
 	 TARGET_BODY or TARGET_NAME.  Bloody 'ell.  I think the problem
-	 is that there isn't an actual *standard* for these formats, so
+	 is that there isn't an actual *standard* for these formats, or
+	 the standard isn't widely available, so
 	 people kinda make it up as they go along. */
 
       if (strcmp(scanbuff,"END") == 0) {
 	break;
-      } else if (sscanf(scanbuff,"RECORD_TYPE = %s",rtbuff) == 1) {
-	if (strncmp(rtbuff,"VARIABLE_LENGTH",15) == 0) {
+      } else if (sscanf(scanbuff," RECORD_TYPE = %s",rtbuff) == 1) {
+	if (strncmp(rtbuff,"VARIABLE_LENGTH", (size_t) 15) == 0) {
 	  /*		itype=PDSVARIABLE; */
-	} else if (strncmp(rtbuff,"FIXED_LENGTH",12) == 0) {
+	} else if (strncmp(rtbuff,"FIXED_LENGTH", (size_t) 12) == 0) {
 	  /*		itype=PDSFIXED;*/
 	} else {
 	  rtbuff[RTBUFFSIZE-1]='\0'; /* juuuust in case */
@@ -449,13 +409,16 @@ int LoadPDS(fname, pinfo)
 	  fclose(zf);
 	  return 0;
 	}
-      } else if (sscanf(scanbuff,"RECORD_BYTES = %d",&recsize) == 1) {
+	lastwasinote=FALSE;
+      } else if (sscanf(scanbuff," RECORD_BYTES = %d",&recsize) == 1) {
 	/* these default to RECORD_BYTES unless explicitly set */
 	if (hrecsize == 0) hrecsize=recsize;
 	    if (irecsize == 0) irecsize=recsize;
+	lastwasinote=FALSE;
 	continue;
-      } else if (sscanf(scanbuff,"FILE_TYPE = %s", rtbuff) != 0) {
-	if (strncmp(rtbuff,"IMAGE",5) == 0) {
+      } else if (sscanf(scanbuff," FILE_TYPE = %s", rtbuff) != 0) {
+	lastwasinote=FALSE;
+	if (strncmp(rtbuff,"IMAGE", (size_t) 5) == 0) {
 	  isimage=TRUE;
 	  continue;
 	} else {
@@ -464,87 +427,197 @@ int LoadPDS(fname, pinfo)
 	}
       } else if ((sscanf(scanbuff," HEADER_RECORDS = %d",&labelrecs) == 1) ||
 		 (sscanf(scanbuff," LABEL_RECORDS = %d", &labelrecs) == 1)) {
+	lastwasinote=FALSE;
 	continue;
       } else if (sscanf(scanbuff," IMAGE_LINES = %d",&y) == 1) {
-	isimage=TRUE; continue;
+	isimage=TRUE; lastwasinote=FALSE; continue;
       } else if (sscanf(scanbuff," LINE_SAMPLES = %d",&x) == 1) {
-	continue;
+	lastwasinote=FALSE; continue;
       } else if (sscanf(scanbuff," LINES = %d",&y) == 1) {
-	isimage=TRUE; continue;
+	isimage=TRUE; lastwasinote=FALSE; continue;
       } else if (sscanf(scanbuff," HEADER_RECORD_BYTES = %d",&hrecsize)==1) {
-	continue;
+	lastwasinote=FALSE; continue;
       } else if (sscanf(scanbuff," IMAGE_RECORD_BYTES = %d",&irecsize)==1) {
-	continue;
+	lastwasinote=FALSE; continue;
       } else if (sscanf(scanbuff," LINE_PREFIX_BYTES = %d",&lpsize)==1) {
-	continue;
+	lastwasinote=FALSE; continue;
       } else if (sscanf(scanbuff," LINE_SUFFIX_BYTES = %d",&lssize)==1) {
-	continue;
+	lastwasinote=FALSE; continue;
       } else if (sscanf(scanbuff," SAMPLE_BITS = %d", &samplesize) == 1) {
-	continue;
+	lastwasinote=FALSE; continue;
+      } else if (sscanf(scanbuff," SAMPLE_TYPE = %s", sampletype) == 1) {
+	lastwasinote=FALSE; continue;
       } else if (sscanf(scanbuff," SPACECRAFT_NAME = %s %s",
 			spacecraft,garbage) == 2 ) {
-	strcat(spacecraft,pds_strstr(scanbuff, spacecraft)+strlen(spacecraft));
-	continue;
+	strcat(spacecraft,xv_strstr(scanbuff, spacecraft)+strlen(spacecraft));
+	lastwasinote=FALSE;  continue;
       } else if (sscanf(scanbuff," SPACECRAFT_NAME = %s", spacecraft) == 1) {
-	continue;
+	lastwasinote=FALSE; continue;
 
       } else if (sscanf(scanbuff," TARGET_NAME = %s", target) == 1) {
-	continue;
+	lastwasinote=FALSE; continue;
       } else if (sscanf(scanbuff," TARGET_BODY = %s", target) == 1) {
-	continue;
+	lastwasinote=FALSE; continue;
+
+      } else if (sscanf(scanbuff," MISSION_PHASE_NAME = %s", mphase) == 1) {
+	lastwasinote=FALSE; continue;
+      } else if (sscanf(scanbuff," MISSION_PHASE = %s", mphase) == 1) {
+	lastwasinote=FALSE; continue;
+
+      } else if (sscanf(scanbuff," INSTRUMENT_NAME = %s", iname) == 1) {
+	lastwasinote=FALSE; continue;
+
+      } else if (sscanf(scanbuff," GAIN_MODE_ID = %s", gainmode) == 1) {
+	lastwasinote=FALSE; continue;
+
+      } else if (sscanf(scanbuff," INSTRUMENT_GAIN_STATE = %s",gainmode)==1) {
+	lastwasinote=FALSE; continue;
+	
+      } else if (sscanf(scanbuff," EDIT_MODE_ID = %s", editmode) == 1) {
+	lastwasinote=FALSE; continue;
+
+      } else if (sscanf(scanbuff," INSTRUMENT_EDIT_MODE = %s", editmode)==1) {
+	lastwasinote=FALSE; continue;
+
+      } else if (sscanf(scanbuff," SCAN_MODE_ID = %s", scanmode) == 1) {
+	lastwasinote=FALSE; continue;
+
+      } else if (sscanf(scanbuff," INSTRUMENT_SCAN_RATE = %s", scanmode)==1) {
+	lastwasinote=FALSE; continue;
+
+      } else if (sscanf(scanbuff," SHUTTER_MODE_ID = %s", shuttermode) == 1) {
+	lastwasinote=FALSE; continue;
+
+      } else if (sscanf(scanbuff," INSTRUMENT_SHUTTER_MODE = %s",
+			shuttermode) == 1) {
+	lastwasinote=FALSE; continue;
+
+      } else if (sscanf(scanbuff," SCAN_MODE_ID = %s", scanmode) == 1) {
+	lastwasinote=FALSE; continue;
+
+      } else if (sscanf(scanbuff," INSTRUMENT_SCAN_RATE = %s", scanmode)==1) {
+	lastwasinote=FALSE; continue;
 
       } else if (sscanf(scanbuff," SPACECRAFT_EVENT_TIME = %s", itime) == 1) {
-	continue;
+	lastwasinote=FALSE; continue;
+
       } else if (sscanf(scanbuff," IMAGE_TIME = %s", itime) == 1) {
-	continue;
+	lastwasinote=FALSE; continue;
+
       } else if (sscanf(scanbuff," FILTER_NAME = %s", filtname) == 1) {
+	lastwasinote=FALSE; continue;
+
+      } else if (sscanf(scanbuff," INSTRUMENT_FILTER_NAME = %s",filtname)==1) {
+	lastwasinote=FALSE; continue;
+
+      } else if ((sscanf(scanbuff," EXPOSURE_DURATION = %s", exposure) == 1)
+	      || (sscanf(scanbuff," INSTRUMENT_EXPOSURE_DURATION = %s",
+			 exposure) == 1)) {
+	tmptmp = index(scanbuff,'=');
+	tmptmp++;
+	while((*tmptmp) == ' ')
+	    tmptmp++;
+	strcpy(exposure,tmptmp);
+	lastwasinote=FALSE; continue;
+
+      } else if (sscanf(scanbuff, "NOTE = %s", inote) == 1) {
+	tmptmp = index(scanbuff,'='); tmptmp++;
+	while (((*tmptmp) == ' ') || ((*tmptmp)  == '"')) tmptmp++;
+	strcpy(inote,tmptmp);
+	strcat(inote," ");
+
+	/*   evil and somewhat risky:  A "note" (really, any textual
+	 *    field) can span records until " is reached.  If I ever
+	 *     get my hands on the clown who designed this format...
+	 *                             What we basically assume here
+	 *        is that a NOTE record that doesn't end with a " is
+	 *    followed by some number of continuations, one of which 
+	 *   will have a " in it.  If this turns out to not be true,
+	 *          well, we'll segmentation fault real soon. We use
+	 * lastwasinote as a semaphore to indicate that the previous
+	 *       record was an unfinished NOTE record.  We clear the	  
+	 *      flag in each of the above record types for potential
+	 *   error recovery, although it really breaks up the beauty
+	 * of the cascading sscanfs.  Dykstra'd love me for this one */
+
+	if (inote[strlen(inote)-1] != '"')
+	    lastwasinote=TRUE;
+	else lastwasinote=FALSE;
 	continue;
-      } else if (sscanf(scanbuff," INSTRUMENT_FILTER_NAME = %s", filtname) 
-		 == 1) {
+
+      } else if (lastwasinote) {
+	tmptmp=scanbuff;
+	while (((*tmptmp) == ' ') || ((*tmptmp)  == '"')) tmptmp++;
+	strcat(inote,tmptmp);
+	strcat(inote," ");
+	if (index(tmptmp,'"') != NULL)
+	    lastwasinote=FALSE;
 	continue;
       }
     }
 
+    elaphe=strlen(inote)-1;
+    while ( (inote[elaphe] == ' ') || (inote[elaphe] == '"')) {
+	inote[elaphe--]='\0';
+    }
     if ((isimage == TRUE) && (labelsize=(labelrecs * hrecsize))) {
-      if (samplesize!= 8) {
-	SetISTR(ISTR_WARNING,"This image is not an 8-bit PDS file.  Sorry.\n");
-	fclose(zf);
-	return 0;
-      }
+      ;
     } else {
       SetISTR(ISTR_WARNING,
 	      "This looks like a PDS/VICAR image, but not enough.\n");
       fclose(zf);
       return 0;
     }
-
-    /* Some day this should be generalized to read <>8-bit files.  If I 
-       ever find one that's interesting, I'll do the work.  (That's a
-       hint that you should let me know if *you've* one some.) */
-
+    
+    vaxbyte = strncmp(sampletype, "VAX_", (size_t) 4) == 0 ||
+      strncmp(sampletype, "LSB_", (size_t) 4) == 0;
+    
   } else if (itype == VICAR) {
     /* we've got a VICAR file.  Let's find out how big the puppy is */
+    ungetc(' ', zf);
+    --labelsofar;
 
-    if (fread(scanbuff,labelsize-labelsofar,1,zf) == 1) {
-      if ((tmp = (char *) pds_strstr(scanbuff,"NL=")) == NULL) {
+    if (fread(scanbuff, (size_t) (labelsize-labelsofar),(size_t) 1,zf) == 1) {
+      if ((tmp = (char *) xv_strstr(scanbuff," NL=")) == NULL) {
 	SetISTR(ISTR_WARNING,"LoadPDS: bad NL in VICAR\n");
 	returnp=TRUE;
       }
       
-      if (sscanf(tmp,"NL = %d",&y) != 1) {
+      if (sscanf(tmp," NL = %d",&y) != 1) {
 	SetISTR(ISTR_WARNING,"LoadPDS: bad scan NL in VICAR\n");
 	returnp=TRUE;
       }
 
-      if ((tmp = (char *) pds_strstr(scanbuff, "NS=")) == NULL) {
+      if ((tmp = (char *) xv_strstr(scanbuff, " NS=")) == NULL) {
 	SetISTR(ISTR_WARNING,"LoadPDS: bad NS in VICAR\n");
 	returnp=TRUE;
       }
 
-      if (sscanf(tmp, "NS = %d",&x) != 1) {
+      if (sscanf(tmp, " NS = %d",&x) != 1) {
 	SetISTR(ISTR_WARNING,"LoadPDS: bad scan NS in VICAR\n");
 	returnp=TRUE;
       }
+
+      if ( (tmp=(char *) xv_strstr(scanbuff, " NBB=")))
+        if (sscanf(tmp, " NBB = %d",&lpsize) != 1) {
+	  SetISTR(ISTR_WARNING,"LoadPDS: bad scan NBB in VICAR\n");
+	  returnp=TRUE;
+        }
+
+      vaxbyte = (xv_strstr(scanbuff, " INTFMT='HIGH'") == NULL);
+
+      if (xv_strstr(scanbuff, " FORMAT='BYTE'"))
+        samplesize = 8;
+      else if (xv_strstr(scanbuff, " FORMAT='HALF'"))
+        samplesize = 16;
+      else if (xv_strstr(scanbuff, " FORMAT=")) {
+	SetISTR(ISTR_WARNING,"LoadPDS: unsupported FORMAT in VICAR\n");
+	returnp=TRUE;
+      } else {
+	SetISTR(ISTR_WARNING,"LoadPDS: bad FORMAT in VICAR\n");
+	returnp=TRUE;
+      }
+
     }
 
   } else {
@@ -561,7 +634,9 @@ int LoadPDS(fname, pinfo)
    * following are a few of the more interesting headers.  We'd do more, but
    * there's only so much space in the info box.  I tried to pick a
    * reasonable subset of these to display; if you want others, it's easy
-   * to change the set. */
+   * to change the set.  XV 3.00 has a comment window where we write this
+   * stuff in glorious detail.  Now, if I only had a clue as to what these
+   * fields actually *mean* ... */
 
   *infobuff='\0';
   if (*spacecraft) {
@@ -590,14 +665,16 @@ int LoadPDS(fname, pinfo)
 
   switch (itype) {
   case VICAR:
-    sprintf(pinfo->fullInfo,"VICAR, 8 bits per pixel. (%ld bytes)", filesize);
+    sprintf(pinfo->fullInfo,"VICAR, %d bits per pixel. (%ld bytes)",
+      samplesize, filesize);
     ftypstr = "VICAR";
     rewind(zf);
     break;
 
   case PDSFIXED:
   case VIKINGFIXED:
-    sprintf(pinfo->fullInfo,"PDS, 8 bits per pixel. (%ld bytes)", filesize);
+    sprintf(pinfo->fullInfo,"PDS, %d bits per pixel. (%ld bytes)",
+      samplesize, filesize);
     ftypstr = "PDS";
     rewind(zf);
     break;
@@ -605,7 +682,8 @@ int LoadPDS(fname, pinfo)
   case PDSVARIABLE:
   case VIKINGVARIABLE:
     sprintf(pinfo->fullInfo,
-	    "PDS, 8 bits per pixel, Huffman-encoded. (%ld bytes)", filesize);
+	    "PDS, %d bits per pixel, Huffman-encoded. (%ld bytes)",
+               samplesize, filesize);
     ftypstr = "PDS (Huffman)";
     fclose(zf);
 
@@ -623,19 +701,19 @@ int LoadPDS(fname, pinfo)
 
     /* pdsuncomp filters to a raw file */
 #ifndef VMS
-    if (tempnum=system(scanbuff)) {
+    if ( (tempnum=system(scanbuff)) ) {
       SetISTR(ISTR_WARNING,"Unable to de-Huffmanize '%s'.",fname);
       return 0;
     }
 #else
-    if (tempnum = !system(scanbuff)) {
+    if ( (tempnum = !system(scanbuff)) ) {
       SetISTR(ISTR_WARNING,"Unable to de-Huffmanize '%s'.",fname);
       return 0;
     }
 #endif
 
 
-    zf=fopen(pdsuncompfname,"r");
+    zf = xv_fopen(pdsuncompfname,"r");
     if (!zf) {
       SetISTR(ISTR_WARNING,"LoadPDS: can't open uncompressed file %s\n",
 	      pdsuncompfname);
@@ -646,10 +724,12 @@ int LoadPDS(fname, pinfo)
   /* skip headers */
 
   if ( isfixed == TRUE ) {
-    fread(scanbuff,labelsize,1,zf);
+    fread(scanbuff, (size_t) labelsize, (size_t) 1, zf);
   }
 
-  image=(byte *)malloc(x*y);
+  x *= samplesize/8;
+
+  image = (byte *) malloc((size_t) x*y);
   if (image == NULL) {
     SetISTR(ISTR_WARNING,"LoadPDS: couldn't malloc %d",x*y);
     fclose(zf);
@@ -658,32 +738,35 @@ int LoadPDS(fname, pinfo)
     exit(1);
   }
 
-  if ((lssize || lpsize) && ((itype == PDSFIXED) || (itype == VIKINGFIXED)) ) {
+  if ((lssize || lpsize) &&
+       ((itype == PDSFIXED) || (itype == VIKINGFIXED) || (itype == VICAR)) ) {
     /* ARrrrgh.  Some of these images have crud intermixed with the image, */
     /* preventing us from freading in one fell swoop */
     /* (whatever a fell swoop is */
 
     for (yy=0; yy<y; yy++) {
-      if (lpsize && ((teco=(fread(scanbuff,lpsize,1,zf))) != 1)) {
+      if (lpsize && 
+	  ((teco=(fread(scanbuff,(size_t) lpsize,(size_t) 1,zf))) != 1)) {
 	SetISTR(ISTR_WARNING, "LoadPDS: unexpected EOF reading prefix");
 	fclose(zf);
 	return 0;
       }
-
-      if ((teco=(fread(image+(yy*x),x,1,zf))) != 1) {
+      
+      if ((teco=(fread(image+(yy*x), (size_t) x, (size_t) 1,zf))) != 1) {
 	SetISTR(ISTR_WARNING, "LoadPDS: unexpected EOF reading line %d",yy);
 	fclose(zf);
 	return 0;
       }
 
-      if (lssize && ((teco=(fread(scanbuff,lssize,1,zf))) != 1)) {
+      if (lssize && 
+	  ((teco=(fread(scanbuff,(size_t) lssize,(size_t) 1,zf))) != 1)) {
 	SetISTR(ISTR_WARNING, "LoadPDS: unexpected EOF reading suffix");
 	fclose(zf);
 	return 0;
       }
     }
 
-  } else if ((yy=fread(image,x*y,1,zf)) != 1) {
+  } else if ((yy=fread(image, (size_t) x*y, (size_t) 1, zf)) != 1) {
     SetISTR(ISTR_WARNING,"LoadPDS: error reading image data");
     fclose(zf);
     if (itype==PDSVARIABLE || itype==VIKINGVARIABLE)
@@ -702,6 +785,44 @@ int LoadPDS(fname, pinfo)
   pinfo->w   = x;
   pinfo->h    = y;
 
+  if (samplesize == 16)
+     if (Convert16BitImage(fname, pinfo,
+	  vaxbyte ^ (*(char *)&samplesize == 16)) == 0)
+        return 0;
+
+  pinfo->frmType = -1;   /* can't save as PDS */
+  pinfo->colType = F_GREYSCALE;
+  sprintf(pinfo->shrtInfo, "%dx%d %s. ", pinfo->w, pinfo->h, ftypstr);
+
+  pinfo->comment = (char *) malloc((size_t) 2000);
+  if (pinfo->comment) {
+    char tmp[256];
+    *(pinfo->comment) = '\0';
+    
+    sprintf(tmp, "Spacecraft: %-28sTarget: %-32s\n", spacecraft, target);
+    strcat(pinfo->comment, tmp);
+    
+    sprintf(tmp, "Filter: %-32sMission phase: %-24s\n", filtname, mphase);
+    strcat(pinfo->comment, tmp);
+    
+    sprintf(tmp, "Image time: %-28sGain mode: %-29s\n", itime, gainmode);
+    strcat(pinfo->comment, tmp);
+    
+    sprintf(tmp, "Edit mode: %-29sScan mode: %-29s\n", editmode, scanmode);
+    strcat(pinfo->comment, tmp);
+    
+    sprintf(tmp, "Exposure: %-30sShutter mode: %-25s\n", exposure,shuttermode);
+    strcat(pinfo->comment, tmp);
+    
+    sprintf(tmp, "Instrument: %-28sImage time: %-28s\n", iname, itime);
+    strcat(pinfo->comment, tmp);
+    
+    sprintf(tmp, "Image Note: %-28s", inote);
+    strcat(pinfo->comment, tmp);
+  }
+
+  if (LoadPDSPalette(fname, pinfo))  return 1;
+
   /* these are grayscale, so cobble up a ramped colormap */
   /* Viking images on the CD's seem to be inverted.  Sigh */
 
@@ -721,14 +842,175 @@ int LoadPDS(fname, pinfo)
     }
   }
 
-  pinfo->frmType = -1;   /* can't save as PDS */
-  pinfo->colType = F_GREYSCALE;
-  sprintf(pinfo->shrtInfo, "%dx%d %s. ", pinfo->w, pinfo->h, ftypstr);
-  pinfo->comment = (char *) NULL;
+  pinfo->normw = pinfo->w;   pinfo->normh = pinfo->h;
+  return 1;
+}
 
-  if (pic8) return 1;
-  return 0;
+
+
+/*
+ *   16-bit to 8-bit conversion
+ */
+
+/*******************************************/
+static int Convert16BitImage(fname, pinfo, swab)
+     char *fname;
+     PICINFO *pinfo;
+     int swab;
+{
+  unsigned short *pShort;
+  long i, j, k, l, n, *hist, nTot;
+  size_t m;
+  byte *lut, *pPix8;
+  FILE *fp;
+  char  name[1024], *c;
+
+  m = 65536 * sizeof(byte);
+  lut = (byte *) malloc(m);
+  if (lut == NULL) {
+    SetISTR(ISTR_WARNING,"LoadPDS: couldn't malloc %d", m);
+    return 0;
+  }
+  pinfo->w /= 2;
+
+  /* allocate histogram table */
+  m = 65536 * sizeof(long);
+  hist = (long *) malloc(m);
+  if (hist == NULL) {
+    SetISTR(ISTR_WARNING,"LoadPDS: couldn't malloc %d", m);
+    free(lut);
+    return 0;
+  }
+
+  /* check whether histogram file exists */
+#ifdef VMS
+  c = rindex(strcpy(name, (c = rindex(fname, ':')) ? c+1 : fname), ']');
+#else
+  c = rindex(strcpy(name, fname), '/');
+#endif /* VMS */
+  (void)strcpy(c ? c+1 : name, "hist.tab");
+
+  /* read the histogram file which is always LSB_INTEGER */
+  if ((fp = xv_fopen(name, "r")) != NULL) {
+    for (n = k = nTot = 0; n < 65536 && k != EOF; n++, nTot += j) {
+      for (i = j = 0; i < 4 && (k = getc(fp)) != EOF; i++)
+        j = ((j >> 8) & 0xffffff) | ((k & 255) << 24);
+      hist[swab ? ((n & 255) << 8) | ((n >> 8) & 255) : n] = j;
+    }
+    fclose(fp);
+    if (k == EOF)
+      fp = NULL;
+  }
+
+  /* if no external histogram, make one */
+  if (fp == NULL) {
+    for (n = 0; n < 65536; n++)
+      hist[n] = 0;
+    nTot = n = pinfo->h * pinfo->w;
+    pShort = (unsigned short *)pinfo->pic;
+    while (--n >= 0) {
+      hist[*pShort++]++;
+    }
+  }
+
+  /* use either histogram equalization or linear stretch */
+  if (0) {      /* was: (conv24MB.flags[CONV24_HIST]) */
+
+    /* construct lookup table to invert the histogram */
+    m = l = hist[0];
+    i = (nTot - m) >> 8;
+    for (n = j = 1; n < 65536; n++) {
+      k = swab ? ((n & 255) << 8) | ((n >> 8) & 255) : n;
+      if ((l += hist[k]) > m && j < 255)
+        j++, m += i;
+      lut[k] = j;
+    }
+
+  } else {
+
+    /* find min and max of histogram */
+    n = nTot/10000;
+    for (m = 0, j = 1; j < 65536; j++)
+      if ((m += hist[swab ? ((j & 255) << 8) | ((j >> 8) & 255) : j]) > n)
+        break;
+    for (m = 0, k = 65535; --k > 1; )
+      if ((m += hist[swab ? ((k & 255) << 8) | ((k >> 8) & 255) : k]) > n)
+        break;
+
+    /* construct stretch table */
+    for (n = 0; n < 65536; n++) {
+      i = swab ? (((n & 255) << 8) | ((n >> 8) & 255)) : n;
+      m = 1 + (254 * (n - j)) / (k - j);
+      lut[i] = n < j ? 0 : (n > k ? 255 : m);
+    }
+  }
+
+  /* allocate new 8-bit image */
+  free(hist);
+  n = pinfo->w * pinfo->h;
+  pPix8 = (byte *)malloc(n*sizeof(byte));
+  if (pPix8 == NULL) {
+    SetISTR(ISTR_WARNING,"LoadPDS: couldn't malloc %d", n*sizeof(byte));
+    free(lut);
+    return 0;
+  }
+
+  /* convert the 16-bit image to 8-bit */
+  lut[0] = 0;
+  free(pShort = (unsigned short *)pinfo->pic);
+  pinfo->pic = pPix8;
+  while(--n >= 0)
+    *pPix8++ = lut[*pShort++];
+  free(lut);
+  return 1;
+}
+
+/*
+ *   If there is a PALETTE.TAB file in the same directory, use it
+ */
+
+/*******************************************/
+static int LoadPDSPalette(fname, pinfo)
+     char    *fname;
+     PICINFO *pinfo;
+{
+  FILE    *fp;
+  char    name[1024], buf[256], *c;
+  int     i, n, r, g, b;
+  
+#ifdef VMS
+  c = rindex(strcpy(name, (c = rindex(fname, ':')) ? c+1 : fname), ']');
+#else
+  c = rindex(strcpy(name, fname), '/');
+#endif /* VMS */
+  (void)strcpy(c ? c+1 : name, "palette.tab");
+  
+  if ((fp = xv_fopen(name, "r")) == NULL)
+    return 0;
+  for (i = 0; i < 256; i++) {
+    if (fgets(c = buf, (int) (sizeof(buf)-1), fp) == NULL)
+      break;
+    while (*c && isspace(*c))
+      c++;
+    if (*c == '\0' || *c == '#')
+      continue;
+    if (sscanf(buf, "%d %d %d %d", &n, &r, &g, &b) != 4)
+      continue;
+    if (n < 0 || n > 255)
+      continue;
+    pinfo->r[n] = r;
+    pinfo->g[n] = g;
+    pinfo->b[n] = b;
+    if (r != g && r != b)
+      pinfo->colType = F_FULLCOLOR;
+  }
+  (void)fclose(fp);
+  return 1;
 }
 
 
 #endif /* HAVE_PDS */
+
+
+
+
