@@ -4,58 +4,7 @@
  * LoadRLE(fname, numcols)  -  loads an RLE file
  */
 
-/* Copyright Notice
- * ================
- * Copyright 1989, 1990, 1991, 1992, 1993 by John Bradley
- * 
- * Permission to use, copy, and distribute XV in its entirety, for 
- * non-commercial purposes, is hereby granted without fee, provided that
- * this license information and copyright notice appear in all copies.
- * 
- * Note that distributing XV 'bundled' in with ANY product is considered
- * to be a 'commercial purpose'.
- *
- * Also note that any copies of XV that are distributed MUST be built
- * and/or configured to be in their 'unregistered copy' mode, so that it
- * is made obvious to the user that XV is shareware, and that they should
- * consider donating, or at least reading this License Info.
- * 
- * The software may be modified for your own purposes, but modified
- * versions may NOT be distributed without prior consent of the author.
- * 
- * This software is provided 'as-is', without any express or implied
- * warranty.  In no event will the author be held liable for any damages
- * arising from the use of this software.
- * 
- * If you would like to do something with XV that this copyright
- * prohibits (such as distributing it with a commercial product, 
- * using portions of the source in some other program, etc.), please
- * contact the author (preferably via email).  Arrangements can
- * probably be worked out.
- *
- * XV is shareware for PERSONAL USE only.  You may use XV for your own
- * amusement, and if you find it nifty, useful, generally cool, or of
- * some value to you, your non-deductable donation would be greatly
- * appreciated.  $25 is the suggested donation, though, of course,
- * larger donations are quite welcome.  Folks who donate $25 or more
- * can receive a Real Nice bound copy of the XV manual for no extra
- * charge.
- * 
- * Commercial, government, and institutional users MUST register their
- * copies of XV, for the exceedingly REASONABLE price of just $25 per
- * workstation/X terminal.  Site licenses are available for those who
- * wish to run XV on a large number of machines.  Contact the author
- * for more details.
- *
- * The author may be contacted via:
- *    US Mail:  John Bradley
- *              1053 Floyd Terrace
- *              Bryn Mawr, PA  19010
- *
- *    Phone:    (215) 898-8813
- *    EMail:    bradley@cis.upenn.edu
- */
-
+#include "copyright.h"
 
 #include "xv.h"
 
@@ -78,8 +27,8 @@
 
 #define GETINT(fp) (c=getc(fp), c1=getc(fp), (c1<<8) + c )
 
-static void read_rle();
-static int  rleError();
+static void read_rle PARM((FILE *, byte *, int, int, int, int));
+static int  rleError PARM((char *, char *));
 
 
 
@@ -106,7 +55,7 @@ int LoadRLE(fname, pinfo)
   bname = BaseName(fname);
 
   /* open the stream */
-  fp=fopen(fname,"r");
+  fp = xv_fopen(fname,"r");
   if (!fp) return (rleError(bname, "unable to open file"));
   
 
@@ -188,7 +137,7 @@ int LoadRLE(fname, pinfo)
   if (flags & H_COMMENT) {
     cmtlen = GETINT(fp);
     if (cmtlen) {
-      pinfo->comment = (char *) malloc(cmtlen + 1);
+      pinfo->comment = (char *) malloc((size_t) (cmtlen + 1));
 
       if (DEBUG) fprintf(stderr,"Comment: (%d bytes) '", cmtlen);
       for (i=0; i<cmtlen; i++) {
@@ -208,7 +157,7 @@ int LoadRLE(fname, pinfo)
   }
 
 
-  if (ferror(fp)) {
+  if (ferror(fp) || feof(fp)) {
     fclose(fp);
     if (pinfo->comment) free(pinfo->comment);  pinfo->comment = (char *) NULL;
     return rleError(bname, "EOF reached in RLE header.\n");
@@ -247,8 +196,8 @@ int LoadRLE(fname, pinfo)
 
 
   /* allocate image memory */
-  if (ncolors == 1) img = (byte *) calloc(w * h, 1);
-               else img = (byte *) calloc(w * h * 3, 1);
+  if (ncolors == 1) img = (byte *) calloc((size_t) w * h,     (size_t) 1);
+               else img = (byte *) calloc((size_t) w * h * 3, (size_t) 1);
   if (!img) {
     fclose(fp);
     if (pinfo->comment) free(pinfo->comment);  pinfo->comment = (char *) NULL;
@@ -271,7 +220,7 @@ int LoadRLE(fname, pinfo)
 
   read_rle(fp, img, w, h, ncolors, ncmap);
 
-  if (ferror(fp))    /* just a warning */
+  if (ferror(fp) || feof(fp))    /* just a warning */
     rleError(bname, "RLE file appears to be truncated.");
 
   fclose(fp);
@@ -303,11 +252,12 @@ int LoadRLE(fname, pinfo)
   pinfo->pic = img;
   pinfo->w   = w;  
   pinfo->h   = h;
+  pinfo->normw = pinfo->w;   pinfo->normh = pinfo->h;
   pinfo->frmType = -1;    /* no default format to save in */
 
   if (ncolors == 1) {     /* grayscale or PseudoColor */
     pinfo->type = PIC8;
-    if (ncmap == 1) {
+    if (ncmap == 0 || ncmap == 1) {   /* grey, or grey with gamma curve */
       pinfo->colType = F_GREYSCALE;
       sprintf(pinfo->fullInfo, "Greyscale RLE.  (%ld bytes)", filesize);
       for (i=0; i<256; i++) 
