@@ -1,8 +1,5 @@
 /*
  * xvsmooth.c - smoothing/color dither routines for XV
- * 
- *  Author:    John Bradley, University of Pennsylvania
- *                (bradley@cis.upenn.edu)
  *
  *  Contains:
  *            byte *SmoothResize(src8, swide, shigh, dwide, dhigh,
@@ -15,71 +12,16 @@
  *                                rdisp, gdisp, bdisp, maplen)
  */
 
-/* Copyright Notice
- * ================
- * Copyright 1989, 1990, 1991, 1992, 1993 by John Bradley
- * 
- * Permission to use, copy, and distribute XV in its entirety, for 
- * non-commercial purposes, is hereby granted without fee, provided that
- * this license information and copyright notice appear in all copies.
- * 
- * Note that distributing XV 'bundled' in with ANY product is considered
- * to be a 'commercial purpose'.
- *
- * Also note that any copies of XV that are distributed MUST be built
- * and/or configured to be in their 'unregistered copy' mode, so that it
- * is made obvious to the user that XV is shareware, and that they should
- * consider donating, or at least reading this License Info.
- * 
- * The software may be modified for your own purposes, but modified
- * versions may NOT be distributed without prior consent of the author.
- * 
- * This software is provided 'as-is', without any express or implied
- * warranty.  In no event will the author be held liable for any damages
- * arising from the use of this software.
- * 
- * If you would like to do something with XV that this copyright
- * prohibits (such as distributing it with a commercial product, 
- * using portions of the source in some other program, etc.), please
- * contact the author (preferably via email).  Arrangements can
- * probably be worked out.
- *
- * XV is shareware for PERSONAL USE only.  You may use XV for your own
- * amusement, and if you find it nifty, useful, generally cool, or of
- * some value to you, your non-deductable donation would be greatly
- * appreciated.  $25 is the suggested donation, though, of course,
- * larger donations are quite welcome.  Folks who donate $25 or more
- * can receive a Real Nice bound copy of the XV manual for no extra
- * charge.
- * 
- * Commercial, government, and institutional users MUST register their
- * copies of XV, for the exceedingly REASONABLE price of just $25 per
- * workstation/X terminal.  Site licenses are available for those who
- * wish to run XV on a large number of machines.  Contact the author
- * for more details.
- *
- * The author may be contacted via:
- *    US Mail:  John Bradley
- *              1053 Floyd Terrace
- *              Bryn Mawr, PA  19010
- *
- *    Phone:    (215) 898-8813
- *    EMail:    bradley@cis.upenn.edu
- */
-
+#include "copyright.h"
 
 #include "xv.h"
 
-#ifdef __STDC__
-static int SmoothX(byte *, byte *, int, int, int, int, int, 
-		   byte *, byte *, byte *);
-static int SmoothY(byte *, byte *, int, int, int, int, int, 
-		   byte *, byte *, byte *);
-static int SmoothXY(byte *, byte *, int, int, int, int, int, 
-		    byte *, byte *, byte *);
-#else
-static int  SmoothX(), SmoothY(), SmoothXY();
-#endif
+static int smoothX  PARM((byte *, byte *, int, int, int, int, int,
+			  byte *, byte *, byte *));
+static int smoothY  PARM((byte *, byte *, int, int, int, int, int, 
+			  byte *, byte *, byte *));
+static int smoothXY PARM((byte *, byte *, int, int, int, int, int,
+			  byte *, byte *, byte *));
 
 
 /***************************************************/
@@ -132,7 +74,7 @@ byte *Smooth24(pic824, is24, swide, shigh, dwide, dhigh, rmap, gmap, bmap)
   int   retval, bperpix;
 
   cA = cB = cC = cD = 0;
-  pp = pic24 = (byte *) malloc(dwide * dhigh * 3);
+  pp = pic24 = (byte *) malloc((size_t) (dwide * dhigh * 3));
   if (!pic24) {
     fprintf(stderr,"unable to malloc pic24 in 'Smooth24()'\n");
     return pic24;
@@ -142,15 +84,15 @@ byte *Smooth24(pic824, is24, swide, shigh, dwide, dhigh, rmap, gmap, bmap)
 
   /* decide which smoothing routine to use based on type of expansion */
   if      (dwide <  swide && dhigh <  shigh) 
-    retval = SmoothXY(pic24, pic824, is24, swide, shigh, dwide, dhigh,
+    retval = smoothXY(pic24, pic824, is24, swide, shigh, dwide, dhigh,
 		      rmap, gmap, bmap);
 
   else if (dwide <  swide && dhigh >= shigh) 
-    retval = SmoothX (pic24, pic824, is24, swide, shigh, dwide, dhigh,
+    retval = smoothX (pic24, pic824, is24, swide, shigh, dwide, dhigh,
 		      rmap, gmap, bmap);
 
   else if (dwide >= swide && dhigh <  shigh) 
-    retval = SmoothY (pic24, pic824, is24, swide, shigh, dwide, dhigh,
+    retval = smoothY (pic24, pic824, is24, swide, shigh, dwide, dhigh,
 		      rmap, gmap, bmap);
 
   else {
@@ -178,6 +120,10 @@ byte *Smooth24(pic824, is24, swide, shigh, dwide, dhigh, rmap, gmap, bmap)
     }
     
     for (ey=0; ey<dhigh; ey++) {
+      byte *pptr, rA, gA, bA, rB, gB, bB, rC, gC, bC, rD, gD, bD;
+
+      ProgressMeter(0, (dhigh)-1, ey, "Smooth");
+
       cy = (ey * shigh) / dhigh;
       py = (((ey * shigh) * 100) / dhigh) - (cy * 100) - 50;
       if (py<0) { y1 = cy-1;  if (y1<0) y1=0; }
@@ -189,7 +135,7 @@ byte *Smooth24(pic824, is24, swide, shigh, dwide, dhigh, rmap, gmap, bmap)
       if ((ey&15) == 0) WaitCursor();
 
       for (ex=0; ex<dwide; ex++) {
-	byte *pptr, rA, gA, bA, rB, gB, bB, rC, gC, bC, rD, gD, bD;
+	rA = rB = rC = rD = gA = gB = gC = gD = bA = bB = bC = bD = 0;
 
 	cx = cxtab[ex];
 	px = pxtab[ex];
@@ -232,24 +178,24 @@ byte *Smooth24(pic824, is24, swide, shigh, dwide, dhigh, rmap, gmap, bmap)
 	  pD = 100 - (pA + pB + pC);
 
 	  if (is24) {
-	    *pp++ = (pA * rA)/100 + (pB * rB)/100 + 
-	            (pC * rC)/100 + (pD * rD)/100;
+	    *pp++ = ((int) (pA * rA))/100 + ((int) (pB * rB))/100 + 
+	            ((int) (pC * rC))/100 + ((int) (pD * rD))/100;
 
-	    *pp++ = (pA * gA)/100 + (pB * gB)/100 + 
-	            (pC * gC)/100 + (pD * gD)/100;
+	    *pp++ = ((int) (pA * gA))/100 + ((int) (pB * gB))/100 + 
+	            ((int) (pC * gC))/100 + ((int) (pD * gD))/100;
 
-	    *pp++ = (pA * bA)/100 + (pB * bB)/100 + 
-	            (pC * bC)/100 + (pD * bD)/100;
+	    *pp++ = ((int) (pA * bA))/100 + ((int) (pB * bB))/100 + 
+	            ((int) (pC * bC))/100 + ((int) (pD * bD))/100;
 	  }
 	  else {  /* 8-bit pic */
-	    *pp++ = (pA * rmap[cA])/100 + (pB * rmap[cB])/100 + 
-	            (pC * rmap[cC])/100 + (pD * rmap[cD])/100;
+	    *pp++ = ((int) (pA * rmap[cA]))/100 + ((int)(pB * rmap[cB]))/100 + 
+	            ((int) (pC * rmap[cC]))/100 + ((int)(pD * rmap[cD]))/100;
 
-	    *pp++ = (pA * gmap[cA])/100 + (pB * gmap[cB])/100 + 
-	            (pC * gmap[cC])/100 + (pD * gmap[cD])/100;
+	    *pp++ = ((int) (pA * gmap[cA]))/100 + ((int)(pB * gmap[cB]))/100 + 
+	            ((int) (pC * gmap[cC]))/100 + ((int)(pD * gmap[cD]))/100;
 
-	    *pp++ = (pA * bmap[cA])/100 + (pB * bmap[cB])/100 + 
-  	            (pC * bmap[cC])/100 + (pD * bmap[cD])/100;
+	    *pp++ = ((int)(pA * bmap[cA]))/100 + ((int)(pB * bmap[cB]))/100 + 
+  	            ((int)(pC * bmap[cC]))/100 + ((int)(pD * bmap[cD]))/100;
 	  }
 	}
       }
@@ -272,7 +218,7 @@ byte *Smooth24(pic824, is24, swide, shigh, dwide, dhigh, rmap, gmap, bmap)
 
 
 /***************************************************/
-static int SmoothX(pic24, pic824, is24, swide, shigh, dwide, dhigh, 
+static int smoothX(pic24, pic824, is24, swide, shigh, dwide, dhigh, 
 		   rmap, gmap, bmap)
 byte *pic24, *pic824, *rmap, *gmap, *bmap;
 int   is24, swide, shigh, dwide, dhigh;
@@ -292,10 +238,11 @@ int   is24, swide, shigh, dwide, dhigh;
 
 
   /* malloc some arrays */
-  lbufR = (int *) calloc(swide, sizeof(int));
-  lbufG = (int *) calloc(swide, sizeof(int));
-  lbufB = (int *) calloc(swide, sizeof(int));
-  pixarr = (int *) calloc(swide+1, sizeof(int));
+  lbufR  = (int *) calloc((size_t) swide,   sizeof(int));
+  lbufG  = (int *) calloc((size_t) swide,   sizeof(int));
+  lbufB  = (int *) calloc((size_t) swide,   sizeof(int));
+  pixarr = (int *) calloc((size_t) swide+1, sizeof(int));
+
   if (!lbufR || !lbufG || !lbufB || !pixarr) {
     if (lbufR)  free(lbufR);
     if (lbufG)  free(lbufG);
@@ -312,6 +259,7 @@ int   is24, swide, shigh, dwide, dhigh;
   cptr = pic824;  cptr1 = cptr + swide * bperpix;
 
   for (i=0; i<dhigh; i++) {
+    ProgressMeter(0, (dhigh)-1, i, "Smooth");
     if ((i&15) == 0) WaitCursor();
 
     ypcnt = (((i*shigh)<<6) / dhigh) - 32;
@@ -328,16 +276,16 @@ int   is24, swide, shigh, dwide, dhigh;
 
     if (is24) {
       for (j=0; j<swide; j++) {
-	lbufR[j] = ((*cptr++ * pcnt0) + (*cptr1++ * pcnt1)) >> 6;
-	lbufG[j] = ((*cptr++ * pcnt0) + (*cptr1++ * pcnt1)) >> 6;
-	lbufB[j] = ((*cptr++ * pcnt0) + (*cptr1++ * pcnt1)) >> 6;
+	lbufR[j] = ((int) ((*cptr++ * pcnt0) + (*cptr1++ * pcnt1))) >> 6;
+	lbufG[j] = ((int) ((*cptr++ * pcnt0) + (*cptr1++ * pcnt1))) >> 6;
+	lbufB[j] = ((int) ((*cptr++ * pcnt0) + (*cptr1++ * pcnt1))) >> 6;
       }
     } 
     else {  /* 8-bit input pic */
       for (j=0; j<swide; j++, cptr++, cptr1++) {
-	lbufR[j] = ((rmap[*cptr] * pcnt0) + (rmap[*cptr1] * pcnt1)) >> 6;
-	lbufG[j] = ((gmap[*cptr] * pcnt0) + (gmap[*cptr1] * pcnt1)) >> 6;
-	lbufB[j] = ((bmap[*cptr] * pcnt0) + (bmap[*cptr1] * pcnt1)) >> 6;
+	lbufR[j] = ((int)((rmap[*cptr]* pcnt0) + (rmap[*cptr1]* pcnt1))) >> 6;
+	lbufG[j] = ((int)((gmap[*cptr]* pcnt0) + (gmap[*cptr1]* pcnt1))) >> 6;
+	lbufB[j] = ((int)((bmap[*cptr]* pcnt0) + (bmap[*cptr1]* pcnt1))) >> 6;
       }
     }
 
@@ -345,6 +293,7 @@ int   is24, swide, shigh, dwide, dhigh;
 
     for (j=0, paptr=pixarr; j<=swide; j++,paptr++) {
       if (*paptr != lastpix) {   /* write a pixel to pic24 */
+	if (!pixcnt) pixcnt = 1;    /* this NEVER happens:  quiets compilers */
 	*pic24++ = pixR / pixcnt;
 	*pic24++ = pixG / pixcnt;
 	*pic24++ = pixB / pixcnt;
@@ -371,7 +320,7 @@ int   is24, swide, shigh, dwide, dhigh;
 
 
 /***************************************************/
-static int SmoothY(pic24, pic824, is24, swide, shigh, dwide, dhigh,
+static int smoothY(pic24, pic824, is24, swide, shigh, dwide, dhigh,
 		   rmap, gmap, bmap)
 byte *pic24, *pic824, *rmap, *gmap, *bmap;
 int   is24, swide, shigh, dwide, dhigh;
@@ -394,12 +343,12 @@ int   is24, swide, shigh, dwide, dhigh;
   bperpix = (is24) ? 3 : 1;
 
   lbufR = lbufG = lbufB = pct0 = pct1 = cxarr = NULL;
-  lbufR = (int *) calloc(dwide, sizeof(int));
-  lbufG = (int *) calloc(dwide, sizeof(int));
-  lbufB = (int *) calloc(dwide, sizeof(int));
-  pct0  = (int *) calloc(dwide, sizeof(int));
-  pct1  = (int *) calloc(dwide, sizeof(int));
-  cxarr = (int *) calloc(dwide, sizeof(int));
+  lbufR = (int *) calloc((size_t) dwide, sizeof(int));
+  lbufG = (int *) calloc((size_t) dwide, sizeof(int));
+  lbufB = (int *) calloc((size_t) dwide, sizeof(int));
+  pct0  = (int *) calloc((size_t) dwide, sizeof(int));
+  pct1  = (int *) calloc((size_t) dwide, sizeof(int));
+  cxarr = (int *) calloc((size_t) dwide, sizeof(int));
 
   if (!lbufR || !lbufG || !lbufB || !pct0 || ! pct1 || !cxarr) {
     retval = 1;
@@ -421,6 +370,7 @@ int   is24, swide, shigh, dwide, dhigh;
   lastline = linecnt = 0;
 
   for (i=0, clptr=pic824; i<=shigh; i++, clptr+=swide*bperpix) {
+    ProgressMeter(0, shigh, i, "Smooth");
     if ((i&15) == 0) WaitCursor();
 
     thisline = (i * dhigh + (15*shigh)/16) / shigh;
@@ -445,14 +395,14 @@ int   is24, swide, shigh, dwide, dhigh;
                        else cptr1 = cptr;
 
       if (is24) {
-	lbufR[j] += (((*cptr++ * pct0[j]) + (*cptr1++ * pct1[j])) >> 6);
-	lbufG[j] += (((*cptr++ * pct0[j]) + (*cptr1++ * pct1[j])) >> 6);
-	lbufB[j] += (((*cptr++ * pct0[j]) + (*cptr1++ * pct1[j])) >> 6);
+	lbufR[j] += ((int)((*cptr++ * pct0[j]) + (*cptr1++ * pct1[j]))) >> 6;
+	lbufG[j] += ((int)((*cptr++ * pct0[j]) + (*cptr1++ * pct1[j]))) >> 6;
+	lbufB[j] += ((int)((*cptr++ * pct0[j]) + (*cptr1++ * pct1[j]))) >> 6;
       }
       else {  /* 8-bit input pic */
-	lbufR[j] += (((rmap[*cptr] * pct0[j])+(rmap[*cptr1] * pct1[j])) >> 6);
-	lbufG[j] += (((gmap[*cptr] * pct0[j])+(gmap[*cptr1] * pct1[j])) >> 6);
-	lbufB[j] += (((bmap[*cptr] * pct0[j])+(bmap[*cptr1] * pct1[j])) >> 6);
+	lbufR[j] += ((int)((rmap[*cptr]*pct0[j])+(rmap[*cptr1]*pct1[j]))) >> 6;
+	lbufG[j] += ((int)((gmap[*cptr]*pct0[j])+(gmap[*cptr1]*pct1[j]))) >> 6;
+	lbufB[j] += ((int)((bmap[*cptr]*pct0[j])+(bmap[*cptr1]*pct1[j]))) >> 6;
       }
     }
    
@@ -477,7 +427,7 @@ int   is24, swide, shigh, dwide, dhigh;
 
 
 /***************************************************/
-static int SmoothXY(pic24, pic824, is24, swide, shigh, dwide, dhigh, 
+static int smoothXY(pic24, pic824, is24, swide, shigh, dwide, dhigh, 
 		    rmap, gmap, bmap)
 byte *pic24, *pic824, *rmap, *gmap, *bmap;
 int   is24, swide, shigh, dwide, dhigh;
@@ -498,10 +448,10 @@ int   is24, swide, shigh, dwide, dhigh;
 
 
   /* malloc some arrays */
-  lbufR = (int *) calloc(swide, sizeof(int));
-  lbufG = (int *) calloc(swide, sizeof(int));
-  lbufB = (int *) calloc(swide, sizeof(int));
-  pixarr = (int *) calloc(swide+1, sizeof(int));
+  lbufR  = (int *) calloc((size_t) swide,   sizeof(int));
+  lbufG  = (int *) calloc((size_t) swide,   sizeof(int));
+  lbufB  = (int *) calloc((size_t) swide,   sizeof(int));
+  pixarr = (int *) calloc((size_t) swide+1, sizeof(int));
   if (!lbufR || !lbufG || !lbufB || !pixarr) {
     if (lbufR)  free(lbufR);
     if (lbufG)  free(lbufG);
@@ -519,6 +469,7 @@ int   is24, swide, shigh, dwide, dhigh;
   cptr = pic824;
 
   for (i=0; i<=shigh; i++) {
+    ProgressMeter(0, shigh, i, "Smooth");
     if ((i&15) == 0) WaitCursor();
 
     thisline = (i * dhigh + (15*shigh)/16 ) / shigh;
@@ -528,6 +479,7 @@ int   is24, swide, shigh, dwide, dhigh;
 
       for (j=0, paptr=pixarr; j<=swide; j++,paptr++) {
 	if (*paptr != lastpix) {                 /* write a pixel to pic24 */
+	  if (!pixcnt) pixcnt = 1;    /* NEVER happens: quiets compilers */
 	  *pic24++ = (pixR/linecnt) / pixcnt;
 	  *pic24++ = (pixG/linecnt) / pixcnt;
 	  *pic24++ = (pixB/linecnt) / pixcnt;
@@ -603,14 +555,24 @@ byte *DoColorDither(pic24, pic8, w, h, rmap, gmap, bmap,
   int  imax, jmax;
   int key;
   long cnt1, cnt2;
+  int fserrmap[512];   /* -255 .. 0 .. +255 */
+
+  /* compute somewhat non-linear floyd-steinberg error mapping table */
+  for (i=j=0; i<=0x40; i++,j++) 
+    { fserrmap[256+i] = j;  fserrmap[256-i] = -j; }
+  for (     ; i<0x80; i++, j += !(i&1) ? 1 : 0) 
+    { fserrmap[256+i] = j;  fserrmap[256-i] = -j; }
+  for (     ; i<=0xff; i++) 
+    { fserrmap[256+i] = j;  fserrmap[256-i] = -j; }
+
 
   cnt1 = cnt2 = 0;
   pwide3 = w*3;  imax = h-1;  jmax = w-1;
   ep = (pic24) ? pic24 : pic8;
 
   /* attempt to malloc things */
-  newpic = (byte *) malloc(w * h);
-  cache = (short *) calloc(2<<14, sizeof(short));
+  newpic = (byte *)  malloc((size_t) (w * h));
+  cache  = (short *) calloc((size_t) (2<<14), sizeof(short));
   thisline = (int *) malloc(pwide3 * sizeof(int));
   nextline = (int *) malloc(pwide3 * sizeof(int));
   if (!cache || !newpic || !thisline || !nextline) { 
@@ -640,7 +602,7 @@ byte *DoColorDither(pic24, pic8, w, h, rmap, gmap, bmap,
 
 
   for (i=0; i<h; i++) {
-    np = newpic + i*w;
+    ProgressMeter(0, h-1, i, "Dither");
     if ((i&15) == 0) WaitCursor();
 
     tmpptr = thisline;  thisline = nextline;  nextline = tmpptr;   /* swap */
@@ -656,17 +618,11 @@ byte *DoColorDither(pic24, pic8, w, h, rmap, gmap, bmap,
 	for (j=pwide3, tmpptr=nextline; j; j--, ep++) *tmpptr++ = (int) *ep;
     }
 
-    /* dither a line, doing odd-lines right-to-left (serpentine) */
-    thisptr = (i&1) ? thisline + w*3 - 3 : thisline;
-    nextptr = (i&1) ? nextline + w*3 - 3 : nextline;
-    if (i&1) np += w-1;
-
-
-    for (j=0; j<w; j++) {
+    /* dither a line */
+    for (j=0, thisptr=thisline, nextptr=nextline; j<w; j++,np++) {
       int k, d, mind, closest;
 
       r2 = *thisptr++;  g2 = *thisptr++;  b2 = *thisptr++;
-      if (i&1) thisptr -= 6;  /* move left */
 
       /* map r2,g2,b2 components (could be outside 0..255 range) 
 	 into 0..255 range */
@@ -732,20 +688,40 @@ byte *DoColorDither(pic24, pic8, w, h, rmap, gmap, bmap,
       gerr = g2 - gdisp[*np];
       berr = b2 - bdisp[*np];
 
-      if (j!=jmax) {  /* adjust LEFT/RIGHT pixel */
-	thisptr[0] += (rerr/2);  rerr -= (rerr/2);
-	thisptr[1] += (gerr/2);  gerr -= (gerr/2);
-	thisptr[2] += (berr/2);  berr -= (berr/2);
-      }
 
-      if (i!=imax) { /* adjust BOTTOM pixel */
-	nextptr[0] += rerr;    /* possibly all err if we're at l/r edge */
-	nextptr[1] += gerr;
-	nextptr[2] += berr;
-      }
+      RANGE(rerr, -255, 255);
+      RANGE(gerr, -255, 255);
+      RANGE(berr, -255, 255);
+      rerr = fserrmap[256+rerr];
+      gerr = fserrmap[256+gerr];
+      berr = fserrmap[256+berr];
 
-      if (i&1) { nextptr -= 3;  np--; }
-          else { nextptr += 3;  np++; }
+
+
+      if (j!=jmax) {  /* adjust RIGHT pixel */
+	thisptr[0] += (rerr*7)/16;
+	thisptr[1] += (gerr*7)/16;
+	thisptr[2] += (berr*7)/16;
+      }
+      
+      if (i!=imax) {	/* do BOTTOM pixel */
+	nextptr[0] += (rerr*5)/16;
+	nextptr[1] += (gerr*5)/16;
+	nextptr[2] += (berr*5)/16;
+
+	if (j>0) {  /* do BOTTOM LEFT pixel */
+	  nextptr[-3] += (rerr*3)/16;
+	  nextptr[-2] += (gerr*3)/16;
+	  nextptr[-1] += (berr*3)/16;
+	}
+
+	if (j!=jmax) {  /* do BOTTOM RIGHT pixel */
+	  nextptr[3] += rerr/16;
+	  nextptr[4] += gerr/16;
+	  nextptr[5] += berr/16;
+	}
+	nextptr += 3;
+      }
     }
   }
 
@@ -784,14 +760,24 @@ byte *Do332ColorDither(pic24, pic8, w, h, rmap, gmap, bmap,
   int  i, j, rerr, gerr, berr, pwide3;
   int  imax, jmax;
   long cnt1, cnt2;
+  int  fserrmap[512];   /* -255 .. 0 .. +255 */
+
+  /* compute somewhat non-linear floyd-steinberg error mapping table */
+  for (i=j=0; i<=0x40; i++,j++) 
+    { fserrmap[256+i] = j;  fserrmap[256-i] = -j; }
+  for (     ; i<0x80; i++, j += !(i&1) ? 1 : 0) 
+    { fserrmap[256+i] = j;  fserrmap[256-i] = -j; }
+  for (     ; i<=0xff; i++) 
+    { fserrmap[256+i] = j;  fserrmap[256-i] = -j; }
+
 
   cnt1 = cnt2 = 0;
   pwide3 = w*3;  imax = h-1;  jmax = w-1;
 
   /* attempt to malloc things */
-  newpic = (byte *) malloc(w * h);
-  thisline = (int *) malloc(pwide3 * sizeof(int));
-  nextline = (int *) malloc(pwide3 * sizeof(int));
+  newpic   = (byte *) malloc((size_t) (w * h));
+  thisline = (int *)  malloc(pwide3 * sizeof(int));
+  nextline = (int *)  malloc(pwide3 * sizeof(int));
   if (!newpic || !thisline || !nextline) { 
     if (newpic)   free(newpic);
     if (thisline) free(thisline);
@@ -820,7 +806,8 @@ byte *Do332ColorDither(pic24, pic8, w, h, rmap, gmap, bmap,
 
   for (i=0; i<h; i++) {
     np = newpic + i*w;
-    if ((i&15) == 0) WaitCursor();
+    ProgressMeter(0, h-1, i, "Dither");
+    if ((i&127) == 0) WaitCursor();
 
     tmpptr = thisline;  thisline = nextline;  nextline = tmpptr;   /* swap */
 
@@ -868,6 +855,15 @@ byte *Do332ColorDither(pic24, pic8, w, h, rmap, gmap, bmap,
       gerr = g2 - gdisp[*np];
       berr = b2 - bdisp[*np];
 
+
+      RANGE(rerr, -255, 255);
+      RANGE(gerr, -255, 255);
+      RANGE(berr, -255, 255);
+      rerr = fserrmap[256+rerr];
+      gerr = fserrmap[256+gerr];
+      berr = fserrmap[256+berr];
+
+
       if (j!=jmax) {  /* adjust LEFT/RIGHT pixel */
 	thisptr[0] += (rerr/2);  rerr -= (rerr/2);
 	thisptr[1] += (gerr/2);  gerr -= (gerr/2);
