@@ -16,58 +16,7 @@
  *
  */
 
-/* Copyright Notice
- * ================
- * Copyright 1989, 1990, 1991, 1992, 1993 by John Bradley
- * 
- * Permission to use, copy, and distribute XV in its entirety, for 
- * non-commercial purposes, is hereby granted without fee, provided that
- * this license information and copyright notice appear in all copies.
- * 
- * Note that distributing XV 'bundled' in with ANY product is considered
- * to be a 'commercial purpose'.
- *
- * Also note that any copies of XV that are distributed MUST be built
- * and/or configured to be in their 'unregistered copy' mode, so that it
- * is made obvious to the user that XV is shareware, and that they should
- * consider donating, or at least reading this License Info.
- * 
- * The software may be modified for your own purposes, but modified
- * versions may NOT be distributed without prior consent of the author.
- * 
- * This software is provided 'as-is', without any express or implied
- * warranty.  In no event will the author be held liable for any damages
- * arising from the use of this software.
- * 
- * If you would like to do something with XV that this copyright
- * prohibits (such as distributing it with a commercial product, 
- * using portions of the source in some other program, etc.), please
- * contact the author (preferably via email).  Arrangements can
- * probably be worked out.
- *
- * XV is shareware for PERSONAL USE only.  You may use XV for your own
- * amusement, and if you find it nifty, useful, generally cool, or of
- * some value to you, your non-deductable donation would be greatly
- * appreciated.  $25 is the suggested donation, though, of course,
- * larger donations are quite welcome.  Folks who donate $25 or more
- * can receive a Real Nice bound copy of the XV manual for no extra
- * charge.
- * 
- * Commercial, government, and institutional users MUST register their
- * copies of XV, for the exceedingly REASONABLE price of just $25 per
- * workstation/X terminal.  Site licenses are available for those who
- * wish to run XV on a large number of machines.  Contact the author
- * for more details.
- *
- * The author may be contacted via:
- *    US Mail:  John Bradley
- *              1053 Floyd Terrace
- *              Bryn Mawr, PA  19010
- *
- *    Phone:    (215) 898-8813
- *    EMail:    bradley@cis.upenn.edu
- */
-
+#include "copyright.h"
 
 #include "xv.h"
 
@@ -89,16 +38,6 @@
 #define TV_CLOSE    2
 #define TV_NBUTTS   3
 
-/* constants used in textKey(), below */
-#define TV_LEFT   0
-#define TV_RIGHT  1
-#define TV_UP     2
-#define TV_DOWN   3
-#define TV_PAGEUP 4
-#define TV_PAGEDN 5
-#define TV_HOME   6
-#define TV_END    7
-
 
 #define TITLELEN 128
 
@@ -129,28 +68,21 @@ static int      mfwide, mfhigh, mfascent;   /* size of chars in mono font */
 static int     *event_retP, *event_doneP;   /* used in tvChkEvent() */
 
 
-#ifdef __STDC__
-static void closeText      (TVINFO *);
-static int  tvChkEvent     (TVINFO *, XEvent *);
-static void resizeText     (TVINFO *, int, int);
-static void computeScrlVals(TVINFO *);
-static void doCmd          (TVINFO *, int);
-static void drawTextView   (TVINFO *);
-static void drawNumLines   (TVINFO *);
-static void eraseNumLines  (TVINFO *);
-static void drawTextW      (int, SCRL *);
-static void clickText      (TVINFO *, int, int);
-static void keyText        (TVINFO *, XKeyEvent *);
-static void textKey        (TVINFO *, int);
-static void doHexAsciiCmd  (TVINFO *, int);
-static void computeText    (TVINFO *);
-#else
-static int  tvChkEvent();
-static void closeText(), resizeText(), doCmd(), drawTextView(), drawTextW();
-static void drawNumLines(), eraseNumLines(), computeScrlVals();
-static void clickText(), keyText(), textKey();
-static void doHexAsciiCmd(), computeText();
-#endif
+static void closeText       PARM((TVINFO *));
+static int  tvChkEvent      PARM((TVINFO *, XEvent *));
+static void resizeText      PARM((TVINFO *, int, int));
+static void computeScrlVals PARM((TVINFO *));
+static void doCmd           PARM((TVINFO *, int));
+static void drawTextView    PARM((TVINFO *));
+static void drawNumLines    PARM((TVINFO *));
+static void eraseNumLines   PARM((TVINFO *));
+static void drawTextW       PARM((int, SCRL *));
+static void clickText       PARM((TVINFO *, int, int));
+static void keyText         PARM((TVINFO *, XKeyEvent *));
+static void textKey         PARM((TVINFO *, int));
+static void doHexAsciiCmd   PARM((TVINFO *, int));
+static void computeText     PARM((TVINFO *));
+
 
 /* HEXMODE output looks like this:
 0x00000000: 00 11 22 33 44 55 66 77 - 88 99 aa bb cc dd ee ff  0123456789abcdef
@@ -238,8 +170,8 @@ void CreateTextWins(geom, cmtgeom)
     BTCreate(&(tv->but[TV_CLOSE]), tv->win, 0,0,BUTTW,BUTTH,
 	     "Close",infofg,infobg,hicol,locol);
 
-    BTCreate(&(tv->nopBut), tv->win, 0,0, tv->vscrl.tsize+1,
-	     tv->vscrl.tsize+1, "", infofg, infobg, hicol, locol);
+    BTCreate(&(tv->nopBut), tv->win, 0,0, (u_int) tv->vscrl.tsize+1,
+	     (u_int) tv->vscrl.tsize+1, "", infofg, infobg, hicol, locol);
     tv->nopBut.active = 0;
 
     XMapSubwindows(theDisp, tv->win);
@@ -312,7 +244,7 @@ void TextView(fname)
     return;
   }
 
-  text = (char *) malloc(textlen);
+  text = (char *) malloc((size_t) textlen);
   if (!text) {
     sprintf(buf, "Couldn't malloc %ld bytes to read file '%s'", 
 	    textlen, rfname);
@@ -321,7 +253,7 @@ void TextView(fname)
     return;
   }
 
-  if (fread(text, 1, textlen, fp) != textlen) {
+  if (fread(text, (size_t) 1, (size_t) textlen, fp) != textlen) {
     sprintf(buf, "Warning:  Couldn't read all of '%s'.  Possibly truncated.",
 	    rfname);
     ErrPopUp(buf, "\nHmm...");
@@ -330,7 +262,7 @@ void TextView(fname)
   fclose(fp);
 
   sprintf(title, "File: '%s'", BaseName(fname));
-  OpenTextView(text, textlen, title, 1);
+  OpenTextView(text, (int) textlen, title, 1);
 
   /* note:  text gets freed when window gets closed */
 }
@@ -360,14 +292,14 @@ void OpenTextView(text, len, title, freeonclose)
   tv->text        = text;
   tv->textlen     = len;
   tv->freeonclose = freeonclose;
-  strncpy(tv->title, title, TITLELEN-1);
+  strncpy(tv->title, title, (size_t) TITLELEN-1);
 
   computeText(tv);      /* compute # lines and linestarts array */
 
   anyTextUp = 1;
   if (!tv->vis) XMapRaised(theDisp, tv->win);
   else {
-    XClearArea(theDisp, tv->win, 0, 0, tv->wide, 30, False);
+    XClearArea(theDisp, tv->win, 0, 0, (u_int) tv->wide, (u_int) 30, False);
     drawTextView(tv);
   }
   tv->vis = 1;
@@ -427,7 +359,7 @@ void ChangeCommentText()
   computeText(tv);      /* compute # lines and linestarts array */
 
   if (tv->vis) {
-    XClearArea(theDisp, tv->win, 0, 0, tv->wide, 30, False);
+    XClearArea(theDisp, tv->win, 0, 0, (u_int) tv->wide, (u_int) 30, False);
     drawTextView(tv);
   }
 
@@ -438,87 +370,6 @@ void ChangeCommentText()
 }
 
 
-static char license[10240];
-
-/***************************************************************/
-void ShowLicense()
-{
-  license[0] = '\0';
-
-  /* build the license text */
-#ifdef LC
-#undef LC
-#endif
-#define LC(x) (strcat(license, x), strcat(license, "\n"))
-
-  LC("XV License Info");
-  LC("===============");
-  LC("Thank you for acquiring a copy of XV.  I hope you enjoy it.");
-  LC("");
-  LC("XV is shareware for PERSONAL USE only.  You may use XV for your own");
-  LC("amusement, and if you find it nifty, useful, generally cool, or of");
-  LC("some value to you, your non-deductable donation would be greatly");
-  LC("appreciated.  $25 is the suggested donation, though, of course,");
-  LC("larger donations are quite welcome.  Folks who donate $25 or more");
-  LC("can receive a Real Nice bound copy of the XV manual for no extra");
-  LC("charge.  BE SURE TO SPECIFY THE VERSION OF XV THAT YOU ARE USING!");
-  LC("");
-  LC("Commercial, government, and institutional users MUST register their");
-  LC("copies of XV, for the exceedingly reasonable price of just $25 per");
-  LC("workstation/X terminal.  Site licenses are available for those who");
-  LC("wish to run XV on a large number of machines.  Contact the author");
-  LC("for more details.");
-  LC("");
-  LC("");
-  LC("Copyright Notice");
-  LC("================");
-  LC("XV is Copyright 1989, 1990, 1991, 1992, 1993 by John Bradley");
-  LC("");
-  LC("Permission to use, copy, and distribute XV in its entirety, for ");
-  LC("non-commercial purposes, is hereby granted without fee, provided that");
-  LC("this license information and copyright notice appear in all copies.");
-  LC("");
-  LC("Note that distributing XV 'bundled' in with ANY product is considered");
-  LC("to be a 'commercial purpose'.");
-  LC("");
-  LC("If you redistribute XV, the *entire* contents of this distribution");
-  LC("must be distributed, including the README and INSTALL files, the ");
-  LC("sources, and the entire contents of the 'docs' subdirectory.");
-  LC("");
-  LC("Also note that any copies of XV that are distributed MUST be built");
-  LC("and/or configured to be in their 'unregistered copy' mode, so that it");
-  LC("is made obvious to the user that XV is shareware, and that they should");
-  LC("consider donating, or at least reading this License Info.");
-  LC("");
-  LC("The software may be modified for your own purposes, but modified");
-  LC("versions may NOT be distributed without prior consent of the author.");
-  LC("");
-  LC("This software is provided 'as-is', without any express or implied");
-  LC("warranty.  In no event will the author be held liable for any damages");
-  LC("arising from the use of this software.");
-  LC("");
-  LC("If you would like to do something with XV that this copyright");
-  LC("prohibits (such as distributing it with a commercial product, ");
-  LC("using portions of the source in some other program, etc.), please");
-  LC("contact the author (preferably via email).  Arrangements can");
-  LC("probably be worked out.");
-  LC("");
-  LC("");
-  LC("The author may be contacted via:");
-  LC("    US Mail:  John Bradley");
-  LC("              1053 Floyd Terrace");
-  LC("              Bryn Mawr, PA  19010");
-  LC("");
-  LC("    Phone:    (215) 898-8813");
-  LC("    EMail:    bradley@cis.upenn.edu");
-  LC("");
-
-#undef LC
-
-  OpenTextView(license, strlen(license), "XV License", 0);
-}
-
-
 /***************************************************************/
 void HideTextWindows()
 {
@@ -543,7 +394,7 @@ void UnHideTextWindows()
     if (tinfo[i].wasvis) {
       XMapRaised(theDisp, tinfo[i].win);
       tinfo[i].wasvis = 0;
-      tinfo[i].vis = 0;
+      tinfo[i].vis = 1;
     }
   }
 }
@@ -764,7 +615,8 @@ static int tvChkEvent(tv, xev)
 
       if (tv->wide != e->width || tv->high != e->height) {
 	if (DEBUG) fprintf(stderr,"Forcing a redraw!  (from configure)\n");
-	XClearArea(theDisp, tv->win, 0, 0, e->width, e->height, True);
+	XClearArea(theDisp, tv->win, 0, 0, 
+		   (u_int) e->width, (u_int) e->height, True);
 	resizeText(tv, e->width, e->height);
       }
     }
@@ -806,7 +658,7 @@ static void resizeText(tv,w,h)
   tv->twHigh = tv->chhigh * mfhigh + 6;
 
   XMoveResizeWindow(theDisp, tv->textW, LRMARGINS, TOPMARGIN, 
-		    tv->twWide, tv->twHigh);
+		    (u_int) tv->twWide, (u_int) tv->twHigh);
 
   for (i=0; i<TV_NBUTTS; i++) {
     tv->but[i].x = tv->wide - (TV_NBUTTS-i) * (BUTTW+5);
@@ -858,7 +710,10 @@ static void doCmd(tv, cmd)
   switch (cmd) {
   case TV_ASCII:   doHexAsciiCmd(tv, 0);  break;
   case TV_HEX:     doHexAsciiCmd(tv, 1);  break;
-  case TV_CLOSE:   closeText(tv);   break;
+
+  case TV_CLOSE:   if (tv == &tinfo[CMTWIN]) CloseCommentText();
+                   else closeText(tv);
+                   break;
   }
 }
 
@@ -877,18 +732,17 @@ static void drawTextView(tv)
 
     XSetForeground(theDisp, theGC, infobg);
     XFillRectangle(theDisp, tv->win, theGC, 5+1, y+1, 
-		   StringWidth(tv->title)+6, CHIGH+4);
+		   (u_int) StringWidth(tv->title)+6, (u_int) CHIGH+4);
 
     XSetForeground(theDisp, theGC, infofg);
     XDrawRectangle(theDisp, tv->win, theGC, 5, y, 
-		   StringWidth(tv->title)+7, CHIGH+5);
+		   (u_int) StringWidth(tv->title)+7, (u_int) CHIGH+5);
 
-    Draw3dRect(tv->win, 5+1, y+1, StringWidth(tv->title)+5,
-	       CHIGH+3, R3D_IN, 2, hicol, locol, infobg);
+    Draw3dRect(tv->win, 5+1, y+1, (u_int) StringWidth(tv->title)+5,
+	       (u_int) CHIGH+3, R3D_IN, 2, hicol, locol, infobg);
 
     XSetForeground(theDisp, theGC, infofg);
-    XDrawString(theDisp, tv->win, theGC, 5+3, y+ASCENT+3,
-		tv->title, strlen(tv->title));
+    DrawString(tv->win, 5+3, y+ASCENT+3, tv->title);
   }
 
   drawNumLines(tv);
@@ -925,16 +779,17 @@ static void drawNumLines(tv)
   y = 6;
 
   XSetForeground(theDisp, theGC, infobg);
-  XFillRectangle(theDisp, tv->win, theGC, (x-w)+1, y+1, (w-1), CHIGH+4);
+  XFillRectangle(theDisp, tv->win, theGC, (x-w)+1, y+1, 
+		 (u_int) (w-1), (u_int) CHIGH+4);
 
   XSetForeground(theDisp, theGC, infofg);
-  XDrawRectangle(theDisp, tv->win, theGC, x-w, y, w, CHIGH+5);
+  XDrawRectangle(theDisp, tv->win, theGC, x-w, y, (u_int) w, (u_int) CHIGH+5);
 
-  Draw3dRect(tv->win, (x-w)+1, y+1, w-2,CHIGH+3,R3D_IN,2,hicol,locol,infobg);
+  Draw3dRect(tv->win, (x-w)+1, y+1, (u_int) (w-2), (u_int) CHIGH+3, 
+	     R3D_IN,2,hicol,locol,infobg);
 
   XSetForeground(theDisp, theGC, infofg);
-  XDrawString(theDisp, tv->win, theGC, (x-w)+3, y+ASCENT+3,
-	      tmpstr, strlen(tmpstr));
+  DrawString(tv->win, (x-w)+3, y+ASCENT+3, tmpstr);
 }
 
 
@@ -955,7 +810,7 @@ static void eraseNumLines(tv)
   x = LRMARGINS + tv->twWide + tv->vscrl.tsize+1;     /* right align point */
   y = 5;
 
-  XClearArea(theDisp, tv->win, x-w, y, w+1, CHIGH+7, False);
+  XClearArea(theDisp, tv->win, x-w, y, (u_int) w+1, (u_int) CHIGH+7, False);
 }
 
 
@@ -1051,8 +906,8 @@ static void drawTextW(delta, sptr)
 	    else if (*sp > 127) {
 	      if (!extrach) extrach = 4;
 	      if      (extrach == 4) *lp = '\\';
-	      else if (extrach == 3) *lp = ((*sp & 0700) >> 6) + '0';
-	      else if (extrach == 2) *lp = ((*sp & 0070) >> 3) + '0';
+	      else if (extrach == 3) *lp = ((u_char)(*sp & 0700) >> 6) + '0';
+	      else if (extrach == 2) *lp = ((u_char)(*sp & 0070) >> 3) + '0';
 	      else if (extrach == 1) *lp = ((*sp & 0007)) + '0';
 	    }
 
@@ -1139,8 +994,8 @@ static void drawTextW(delta, sptr)
 
   XSetFont(theDisp, theGC, mfont);
 
-  Draw3dRect(tv->textW, 0, 0, tv->twWide-1, tv->twHigh-1, R3D_IN, 2,
-	     hicol, locol, infobg);
+  Draw3dRect(tv->textW, 0, 0, (u_int) (tv->twWide-1), (u_int) (tv->twHigh-1), 
+	     R3D_IN, 2, hicol, locol, infobg);
 }
 
 
@@ -1173,25 +1028,17 @@ static void keyText(tv, kevt)
 {
   char buf[128];
   KeySym ks;
-  int stlen, shift, dealt;
+  int stlen, shift, dealt, ck;
 
   stlen = XLookupString(kevt, buf, 128, &ks, (XComposeStatus *) NULL);
   shift = kevt->state & ShiftMask;
+  ck    = CursorKey(ks, shift, 1);
   dealt = 1;  
 
+  RemapKeyCheck(ks, buf, &stlen);
+
   /* check for arrow keys, Home, End, PgUp, PgDown, etc. */
-  if      (ks==XK_Left  || ks==XK_KP_4 || ks==XK_F30)
-    textKey(tv,TV_LEFT);
-
-  else if (ks==XK_Right || ks==XK_KP_6 || ks==XK_F32) 
-    textKey(tv,TV_RIGHT);
-
-  else if (ks==XK_Home  || (ks==XK_Prior && shift))   textKey(tv,TV_HOME);
-  else if (ks==XK_End   || (ks==XK_Next  && shift))   textKey(tv,TV_END);
-  else if (ks==XK_Prior || (ks==XK_Up    && shift))   textKey(tv,TV_PAGEUP);
-  else if (ks==XK_Next  || (ks==XK_Down  && shift))   textKey(tv,TV_PAGEDN);
-  else if (ks==XK_Up    || ks==XK_KP_8 || ks==XK_F28) textKey(tv,TV_UP);
-  else if (ks==XK_Down  || ks==XK_KP_2 || ks==XK_F34) textKey(tv,TV_DOWN);
+  if (ck!=CK_NONE) textKey(tv,ck);
   else dealt = 0;
 
   if (dealt || !stlen) return;
@@ -1220,16 +1067,16 @@ static void textKey(tv, key)
   /* an arrow key (or something like that) was pressed in icon window.
      change selection/scrollbar accordingly */
 
-  if (key == TV_UP)     SCSetVal(&tv->vscrl, tv->vscrl.val - 1);
-  if (key == TV_DOWN)   SCSetVal(&tv->vscrl, tv->vscrl.val + 1);
+  if (key == CK_UP)     SCSetVal(&tv->vscrl, tv->vscrl.val - 1);
+  if (key == CK_DOWN)   SCSetVal(&tv->vscrl, tv->vscrl.val + 1);
 
-  if (key == TV_LEFT)   SCSetVal(&tv->hscrl, tv->hscrl.val - 1);
-  if (key == TV_RIGHT)  SCSetVal(&tv->hscrl, tv->hscrl.val + 1);
+  if (key == CK_LEFT)   SCSetVal(&tv->hscrl, tv->hscrl.val - 1);
+  if (key == CK_RIGHT)  SCSetVal(&tv->hscrl, tv->hscrl.val + 1);
 
-  if (key == TV_PAGEUP) SCSetVal(&tv->vscrl, tv->vscrl.val - tv->vscrl.page);
-  if (key == TV_PAGEDN) SCSetVal(&tv->vscrl, tv->vscrl.val + tv->vscrl.page);
-  if (key == TV_HOME)   SCSetVal(&tv->vscrl, tv->vscrl.min);
-  if (key == TV_END)    SCSetVal(&tv->vscrl, tv->vscrl.max);
+  if (key == CK_PAGEUP)   SCSetVal(&tv->vscrl, tv->vscrl.val - tv->vscrl.page);
+  if (key == CK_PAGEDOWN) SCSetVal(&tv->vscrl, tv->vscrl.val + tv->vscrl.page);
+  if (key == CK_HOME)     SCSetVal(&tv->vscrl, tv->vscrl.min);
+  if (key == CK_END)      SCSetVal(&tv->vscrl, tv->vscrl.max);
 }
 
 
@@ -1340,3 +1187,381 @@ static void computeText(tv)
   tv->hexlines = (tv->textlen + 15) / 16;
 }
 
+
+/**********************************************************************/
+/* BUILT-IN TEXT FILES ************************************************/
+/**********************************************************************/
+
+
+
+static char license[10240];
+
+/***************************************************************/
+void ShowLicense()
+{
+  license[0] = '\0';
+
+  /* build the license text */
+#ifdef LC
+#undef LC
+#endif
+#define LC(x) (strcat(license, x), strcat(license, "\n"))
+
+  LC("(Note: This has been changed, and hopefully clarified, from the 3.00");
+  LC("version of this info.  Please read it.)");
+  LC("");
+  LC("Thank you for acquiring a copy of XV, a pretty nifty X program.");
+  LC("I hope you enjoy using it, as I've enjoyed writing it.");
+  LC("");
+  LC("The latest version of XV (or at least a pointer to it) is available");
+  LC("via anonymous ftp on ftp.cis.upenn.edu, in the directory pub/xv.  If");
+  LC("you're not sure if you have the latest version, or you are missing the");
+  LC("source or documentation for XV, PLEASE pick up the latest version of");
+  LC("the xv distribution.  Do *not* send mail unless absolutely necessary");
+  LC("(ie, you don't have ftp capability).");
+  LC("");
+  LC("Note:  The documentation ('xvdocs.ps') may be installed in '/usr/local'.");
+  LC("");
+  LC("If you're viewing this information via the 'About XV' command, and");
+  LC("you'd like to print it out, a copy of this info can be found in the ");
+  LC("README file in the top-level XV source directory.  Print that.  If you");
+  LC("don't have it, see the previous paragraph.");
+  LC("");
+  LC("");
+  LC("XV Licensing Information");
+  LC("------------------------");
+  LC("XV IS SHAREWARE FOR PERSONAL USE ONLY.  ");
+  LC("");
+  LC("You may use XV for your own amusement, and if you find it nifty,");
+  LC("useful, generally cool, or of some value to you, your registration fee");
+  LC("would be greatly appreciated.  $25 is the standard registration fee,");
+  LC("though of course, larger amounts are quite welcome.  Folks who donate");
+  LC("$40 or more can receive a printed, bound copy of the XV manual for no");
+  LC("extra charge.  If you want one, just ask.  Be sure to specify the");
+  LC("version of xv that you are using!");
+  LC("");
+  LC("COMMERCIAL, GOVERNMENT, AND INSTITUTIONAL USERS MUST REGISTER THEIR");
+  LC("COPIES OF XV.");
+  LC("");
+  LC("This does *not* mean that you are required to register XV just because");
+  LC("you play with it on the workstation in your office.  This falls under");
+  LC("the heading of 'personal use'.  If you are a sysadmin, you can put XV");
+  LC("up in a public directory for your users amusement.  Again, 'personal");
+  LC("use', albeit plural.");
+  LC("");
+  LC("On the other hand, if you use XV in the course of doing your work,");
+  LC("whatever your 'work' may happen to be, you *must* register your");
+  LC("copy of XV.  (Note:  If you are a student, and you use XV to do");
+  LC("classwork or research, you should get your professor/teacher/advisor");
+  LC("to purchase an appropriate number of copies.)");
+  LC("");
+  LC("XV licenses are $25 each.  You should purchase one license per");
+  LC("workstation, or one per XV user, whichever is the smaller number.  XV");
+  LC("is *not* sold on a 'number of concurrent users' basis.  If XV was some");
+  LC("$1000 program, yes, that would be a reasonable request, but at $25,");
+  LC("it's not.  Also, given that XV is completely unlocked, there is no way");
+  LC("to enforce any 'number of concurrent users' limits, so it isn't sold");
+  LC("that way.");
+  LC("");
+  LC("Printed and bound copies of the 100-odd page XV manual are available");
+  LC("for $15 each.  Note that manuals are *only* sold with, at minimum, an");
+  LC("equal number of licenses.  (e.g.  if you purchase 5 licenses, you can");
+  LC("also purchase *up to* 5 copies of the manual)");
+  LC("");
+  LC("The source code to the program can be had (as a compressed 'tar' file");
+  LC("split over a couple 3.5\" MS-DOS formatted disks) for $15, for those");
+  LC("who don't have ftp capabilities.");
+  LC("");
+  LC("Orders outside the US and Canada must add an additional $5 per manual");
+  LC("ordered to cover the additional shipping charges.");
+  LC("");
+  LC("Checks, money orders, and purchase orders are accepted.  Credit cards");
+  LC("are not.  All forms of payment must be payable in US Funds.  Checks");
+  LC("must be payable through a US bank (or a US branch of a non-US bank).");
+  LC("Purchase orders for less than $50, while still accepted, are not");
+  LC("encouraged. ");
+  LC("");
+  LC("All payments should be payable to 'John Bradley', and mailed to:");
+  LC("        John Bradley");
+  LC("        1053 Floyd Terrace");
+  LC("        Bryn Mawr, PA  19010");
+  LC("        USA");
+  LC("");
+  LC("");
+  LC("Site Licenses");
+  LC("-------------");
+  LC("If you are planning to purchase 10 or more licenses, site licenses are");
+  LC("available, at a substantial discount.  Site licenses let you run XV on");
+  LC("any and all computing equipment at the site, for any purpose");
+  LC("whatsoever.  The site license covers the current version of XV, and");
+  LC("any versions released within one year of the licensing date.  You are");
+  LC("also allowed to duplicate and distribute an unlimited number of copies");
+  LC("of the XV manual, but only for use within the site.  Covered versions");
+  LC("of the software may be run in perpetuity.");
+  LC("");
+  LC("Also, it should be noted that a 'site' can be defined as anything");
+  LC("you'd like.  It can be a physical location (a room, building,");
+  LC("location, etc.), an organizational grouping (a workgroup, department,");
+  LC("division, etc.) or any other logical grouping (\"the seventeen");
+  LC("technical writers scattered about our company\", etc.).");
+  LC("");
+  LC("The site license cost will be based on your estimate of the number of");
+  LC("XV users or workstations at your site, whichever is the smaller");
+  LC("number.");
+  LC("");
+  LC("If you are interested in obtaining a site license, please contact the");
+  LC("author via electronic mail or FAX (see below for details).  Send");
+  LC("information regarding your site (the name or definition of the 'site',");
+  LC("a physical address, a fax number, and an estimate of the number of");
+  LC("users or workstations), and we'll get a site license out to you for");
+  LC("your examination.");
+  LC("");
+  LC("");
+  LC("Copyright Notice");
+  LC("----------------");
+  LC("XV is Copyright 1989, 1994 by John Bradley");
+  LC("");
+  LC("Permission to copy and distribute XV in its entirety, for");
+  LC("non-commercial purposes, is hereby granted without fee, provided that");
+  LC("this license information and copyright notice appear in all copies.");
+  LC("");
+  LC("If you redistribute XV, the *entire* contents of this distribution");
+  LC("must be distributed, including the README, and INSTALL files, the");
+  LC("sources, and the complete contents of the 'docs' directory.");
+  LC("");
+  LC("Note that distributing XV 'bundled' in with any product is considered");
+  LC("to be a 'commercial purpose'.");
+  LC("");
+  LC("Also note that any copies of XV that are distributed MUST be built");
+  LC("and/or configured to be in their 'unregistered copy' mode, so that it");
+  LC("is made obvious to the user that XV is shareware, and that they should");
+  LC("consider registering, or at least reading this information.");
+  LC("");
+  LC("The software may be modified for your own purposes, but modified");
+  LC("versions may not be distributed without prior consent of the author.");
+  LC("");
+  LC("This software is provided 'as-is', without any express or implied");
+  LC("warranty.  In no event will the author be held liable for any damages");
+  LC("arising from the use of this software.");
+  LC("");
+  LC("If you would like to do something with XV that this copyright");
+  LC("prohibits (such as distributing it with a commercial product, using");
+  LC("portions of the source in some other program, distributing registered");
+  LC("copies, etc.), please contact the author (preferably via email).");
+  LC("Arrangements can probably be worked out.");
+  LC("");
+  LC("");
+  LC("The author may be contacted via:");
+  LC("    US Mail:  John Bradley");
+  LC("              1053 Floyd Terrace");
+  LC("              Bryn Mawr, PA  19010");
+  LC("");
+  LC("    FAX:     (610) 520-2042");
+  LC("");
+  LC("Electronic Mail regarding XV should be sent to one of these addresses:");
+  LC("     xv@devo.dccs.upenn.edu               - general XV questions");
+  LC("     xvbiz@devo.dccs.upenn.edu            - all XV licensing questions");
+  LC("     xvtech@devo.dccs.upenn.edu           - bug reports, technical questions");
+  LC("");
+  LC("Please do *not* send electronic mail directly to the author, as he");
+  LC("gets more than enough as it is.");
+  LC("");
+
+#undef LC
+
+  OpenTextView(license, (int) strlen(license), "About XV", 0);
+}
+
+
+
+static char keyhelp[10240];
+
+/***************************************************************/
+void ShowKeyHelp()
+{
+  keyhelp[0] = '\0';
+
+#undef LC
+#define LC(x) (strcat(keyhelp, x), strcat(keyhelp, "\n"))
+
+  LC("XV Mouse and Keyboard Usage");
+  LC("===========================");
+  LC("");
+  LC("Part 1:  Mouse Usage in the Image Window");
+  LC("----------------------------------------");
+  LC("                 Button1 - draws a selection rectangle");
+  LC("                 Button2 - pixel values, measures dist., picks color");
+  LC("                 Button3 - opens/closes the 'xv controls' window");
+  LC("");
+  LC("          ctrl + Button1 - zooms in");
+  LC("          ctrl + Button2 - pans");
+  LC("          ctrl + Button3 - zooms out");
+  LC("");
+  LC("  shift        + Button1 - draws a square selection rectangle");
+  LC("  shift        + Button2 - freehand drawing tool");
+  LC("  shift + ctrl + Button2 - draws lines");
+  LC("  shift        + Button3 - smudging tool");
+  LC("  ");
+  LC("Part 1a:  Mouse Usage in Selection Rectangle");
+  LC("--------------------------------------------");
+  LC("                 Button1 - moves selection rectangle");
+  LC("  shift        + Button1 - moves selection, constrains motion");
+  LC("                 Button2 - 'drag-and-drop' cut and paste");
+  LC("  shift        + Button2 - 'drag-and-drop' cut and paste, constrain");
+  LC("          ctrl + Button2 - 'drag-and-drop' copy and paste");
+  LC("  shift + ctrl + Button2 - 'drag-and-drop' copy and paste, constrain");
+  LC("");
+  LC("");
+  LC("");
+  LC("Part 2:  Normal Keyboard Equivalents");
+  LC("------------------------------------");
+  LC("The following keys can be used in most xv windows, including the ");
+  LC("image, controls, and color editor windows, but *not* in the visual");
+  LC("schnauzer windows.");
+  LC("");
+  LC("  Tab or ");
+  LC("  Space         - 'Next' command");
+  LC("");
+  LC("  Return        - reload currently displayed image file");
+  LC("");
+  LC("  Del or ");
+  LC("  Backspace     - 'Prev' command");
+  LC("");
+  LC("  ctrl+'l'      - 'Load' command");
+  LC("  ctrl+'s'      - 'Save' command");
+  LC("  ctrl+'p'      - 'Print' command");
+  LC("  ctrl+'d'      - 'Delete' command");
+  LC("");
+  LC("  'q' or ");
+  LC("  ctrl+'q'      - 'Quit' command");
+  LC("");
+  LC("  meta+'x'      - 'cut' command");
+  LC("  meta+'c'      - 'copy' command");
+  LC("  meta+'v'      - 'paste' command");
+  LC("  meta+'d'      - 'clear' command");
+  LC("");
+  LC("  'n'           - reset image to normal (unexpanded) size");
+  LC("  'm'           - maximum image size");
+  LC("  'M'           - maximum image size, maintaining aspect ratio");
+  LC("  '>'           - double image size");
+  LC("  '<'           - half image size");
+  LC("  '.'           - make image 10% larger");
+  LC("  ','           - make image 10% smaller");
+  LC("  'S'           - set image to specified size/expansion");
+  LC("  'a'           - reset image to normal aspect ratio");
+  LC("  '4'           - make image have a 4x3 width/height ratio");
+  LC("  'I'           - round image size to integer expand/compres ratios");
+  LC("");
+  LC("  't'           - turn image 90 degrees clockwise");
+  LC("  'T'           - turn image 90 degrees counter-clockwise");
+  LC("  'h'           - flip image horizontally");
+  LC("  'v'           - flip image vertically");
+  LC("");
+  LC("  'P'           - pad image");
+  LC("  'A'           - image annotation");
+  LC("  'c'           - 'Crop' command");
+  LC("  'u'           - 'UnCrop' command");
+  LC("  'C'           - 'AutoCrop' command");
+  LC("");
+  LC("  'r'           - raw mode");
+  LC("  'd'           - dithered mode");
+  LC("  's'           - smooth mode");
+  LC("  meta+'8'      - toggle 8/24 bit mode");
+  LC("");
+  LC("  'V' or ");
+  LC("  ctrl+'v'      - Visual Schnauzer");
+  LC("  'e'           - Color Editor");
+  LC("  'i'           - Image Info");
+  LC("  ctrl+'c'      - Image Comments");
+  LC("  ctrl+'t'      - Text View");
+  LC("");
+  LC("  ctrl+'g'      - 'Grab' command");
+  LC("  ctrl+'a'      - 'About XV' command");
+  LC("");
+  LC("  meta+'b'      - Blur algorithm");
+  LC("  meta+'s'      - Sharpen algorithm");
+  LC("  meta+'e'      - Edge Detection algorithm");
+  LC("  meta+'m'      - Emboss algorithm");
+  LC("  meta+'o'      - Oil Paint algorithm");
+  LC("  meta+'B'      - Blend algorithm");
+  LC("  meta+'t'      - Copy rotate algorithm");
+  LC("  meta+'T'      - Clear rotate algorithm");
+  LC("  meta+'p'      - Pixelize algorithm");
+  LC("  meta+'S'      - Spread algorithm");
+  LC("");
+  LC("  'R' or");
+  LC("  meta+'r' or ");
+  LC("  meta+'0'      - 'Reset' command in color editor");
+  LC("");
+  LC("  meta+'1'      - select preset 1 in color editor");
+  LC("  meta+'2'      - select preset 2 in color editor");
+  LC("  meta+'3'      - select preset 3 in color editor");
+  LC("  meta+'4'      - select preset 4 in color editor");
+  LC("  meta+'a'      - 'Apply' command in color editor");
+  LC("");
+  LC("");
+  LC("");
+  LC("Part 2a:  Image Window Keys");
+  LC("---------------------------");
+  LC("The following keys can *only* be used inside the image window.");
+  LC("");
+  LC("  ctrl + Up     - crops 1 pixel off the bottom of the image");
+  LC("  ctrl + Down   - crops 1 pixel off the top of the image");
+  LC("  ctrl + Left   - crops 1 pixel off the right side of the image");
+  LC("  ctrl + Right  - crops 1 pixel off the left side of the image");
+  LC("");
+  LC("  If you're viewing a multi-page document:");
+  LC("  'p'           -  opens a 'go to page #' dialog box");
+  LC("");
+  LC("  PageUp, or");
+  LC("  Prev, or ");
+  LC("  shift+Up      - previous page");
+  LC("");
+  LC("  PageDown, or");
+  LC("  Next, or ");
+  LC("  shift+Down    - next page");
+  LC("");
+  LC("");
+  LC("  If a selection rectangle is active");
+  LC("  Up            - move rectangle up 1 pixel");
+  LC("  Down          - move rectangle down 1 pixel");
+  LC("  Left          - move rectangle left 1 pixel");
+  LC("  Right         - move rectangle right 1 pixel");
+  LC("  shift+Up      - shrink rectangle vertically by 1 pixel");
+  LC("  shift+Down    - expand rectangle vertically by 1 pixel");
+  LC("  shift+Left    - shrink rectangle horizontally by 1 pixel");
+  LC("  shift+Right   - expand rectangle horizontally by 1 pixel");
+  LC("");
+  LC("");
+  LC("Part 2b:  Visual Schnauzer Keys");
+  LC("-------------------------------");
+  LC("The following keys can only be used in the visual schnauzer windows.");
+  LC("");
+  LC("  ctrl+'d'      - delete file(s)");
+  LC("  ctrl+'n'      - create new directory");
+  LC("  ctrl+'r'      - rename file");
+  LC("  ctrl+'s'      - rescan directory");
+  LC("  ctrl+'w'      - open new window");
+  LC("  ctrl+'u'      - update icons");
+  LC("  ctrl+'g'      - generate icons for selected files");
+  LC("  ctrl+'a'      - select all files");
+  LC("  ctrl+'t'      - view selected file as text");
+  LC("  ctrl+'q'      - quit XV");
+  LC("  ctrl+'c'      - change directory");
+  LC("  ctrl+'f'      - select filenames");
+  LC("  ctrl+'e'      - recursive update");
+  LC("  Esc           - close window");
+  LC("  Return        - load currently selected file(s)");
+  LC("  Space         - load next file");
+  LC("  shift+Space   - load next file, keeping previous file(s) selected");
+  LC("  Backspace     - load previous file");
+
+#undef LC
+  OpenTextView(keyhelp, (int) strlen(keyhelp), "XV Help", 0);
+}
+
+
+
+
+
+
